import { AfterViewInit, ElementRef, EventEmitter, OnDestroy, OnInit } from '@angular/core';
import { Toggleable } from '../../abstract/toggleable';
/**
 * Available modal sizes.
 */
export declare type BsModalSize = 'sm' | 'lg';
/**
 * LF modal component.
 */
export declare class ModalComponent extends Toggleable implements OnInit, AfterViewInit, OnDestroy {
    /**
     * Size of the modal component. See available sizes at
     * [Bootstrap documentation](
     * https://getbootstrap.com/docs/4.0/components/modal/#optional-sizes).
     */
    size: BsModalSize;
    /**
     * Whether to center the modal vertically on screen.
     */
    isCentered: boolean;
    /**
     * Whether to animate the modal component.
     */
    animate: boolean;
    /**
     * Whether to enable closing the modal by pressing the `Esc` key.
     */
    closeWithEsc: boolean;
    /**
     * Whether to show a dark background underneath the modal when it appears.
     */
    showBackdrop: boolean;
    /**
     * Whether to disable closing the modal by clicking on the backdrop.
     */
    ignoreBackdropClick: boolean;
    /**
     * If the modal contains `<input>`, `<select>`, or `<textarea>` elements
     * within, focus the first found element after the modal opens.
     */
    autoFocus: boolean;
    /**
     * Event emitter which emits an event whenever the modal finishes showing.
     */
    shown: EventEmitter<void>;
    /**
     * Event emitter which emits an event whenever the modal finishes hiding.
     */
    hidden: EventEmitter<void>;
    _modal: ElementRef;
    private $modal;
    private isAnimating;
    private disposeOnOptionsChange;
    private onShowSubscription;
    private onHideSubscription;
    ngAfterViewInit(): void;
    ngOnDestroy(): void;
    /**
     * Method to call on long modals when their content changes, which forces
     * their height to also change.
     */
    refresh(): void;
    show(): Promise<void>;
    hide(): Promise<void>;
    toggle(): Promise<void>;
    /**
     * Creates a new modal and starts listening to events.
     */
    private createModal;
    /**
     * Unsubscribes from all events and destroys the modal (returns a promise that
     * resolves after the modal has been destroyed).
     * @returns Promise that resolves when the modal has been destroyed.
     */
    private destroyModal;
}
