import { OnInit } from '@angular/core';
import { Value } from '../../../abstract/value';
import { TableComponent } from '../table.component';
/**
 * LF table-row component.
 */
export declare class TableRowComponent extends Value implements OnInit {
    protected parentPathBasedComponent: TableComponent;
    ngOnInit(): void;
    /**
     * Whether the row can be removed.
     */
    readonly allowsRemoval: boolean;
    /**
     * Whether the row is selected (adds an `is-selected` class to the row).
     */
    readonly isSelected: boolean;
    /**
     * Class that should be added to the row when the row should show as having an
     * error.
     */
    readonly _classIsInvalid: boolean;
    /**
     * Class that should be added to the row when the row should show as having a
     * warning.
     */
    readonly _classIsWarning: boolean;
    /**
     * Sets the row's selection status.
     * @param selectedStatus `true` to set selected, `false` to set unselected.
     */
    setSelectedStatus(selectedStatus: boolean): void;
    /**
     * Sets the row selection status of all rows from the last row that has had
     * its selection status set to this row.
     * @param selectedStatus `true` to set selected, `false` to set unselected.
     */
    setSelectedStatusFromLastSelected(selectedStatus: boolean): void;
    /**
     * Action to run when the selection checkbox label is clicked: added as a fix
     * for Firefox, where `shift+click` on the label won't trigger a click on the
     * `input`; as such we manually call the event.
     * @param event Click event.
     * @param checkboxElement Element on which the even should be called instead.
     */
    _onLabelClick(event: any, checkboxElement: HTMLElement): void;
    /**
     * Action to run when the selection checkbox is clicked.
     * @param event Click event.
     */
    _onSelectionCheckboxClick(event: any): void;
}
