'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var schematics = require('@angular-devkit/schematics');
var schematics$1 = require('@lightweightform/theme-common/schematics');
var lintFix = require('@schematics/angular/utility/lint-fix');
var core = require('@angular-devkit/core');
var change = require('@schematics/angular/utility/change');

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function __values(o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
}

function __read(o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
}

function __spread() {
    for (var ar = [], i = 0; i < arguments.length; i++)
        ar = ar.concat(__read(arguments[i]));
    return ar;
}

/**
 * Returns the list of locales given a string representing them.
 * @param localesStr String representing the locales.
 * @returns List of locales.
 */
function getLocalesArrayFromString(localesStr) {
    return localesStr.split(',').map(function (l) { return l.trim(); });
}

/**
 * Default i18n folder name.
 */
// tslint:disable-next-line:no-var-requires
var defaultI18nFolderName = require('./ng-add/ng-add.schema.json')
    .properties.i18nFolderName.default;
/**
 * Locales supported by the Bootstrap theme.
 */
var themeSupportedLocales = {
    'en-US': 'I18N_EN_US',
    'pt-PT': 'I18N_PT_PT',
};
/**
 * Adds the application i18n to the project.
 * @param options Schematic options.
 * @returns Schematic rule.
 */
function addI18n(options) {
    return function (host) {
        var e_1, _a;
        var project = schematics$1.getProject(host, options.project);
        var modulePath = schematics$1.getModulePath(host, project, options.module);
        var moduleDir = core.dirname(modulePath);
        var moduleName = schematics$1.getModuleNameFromPath(modulePath);
        var localesArr = getLocalesArrayFromString(options.locales);
        // Theme supported locales filtered by the ones specified by the user
        var commonLocales = Object.keys(themeSupportedLocales).reduce(function (obj, locale) {
            if (localesArr.indexOf(locale) !== -1) {
                obj[locale] = themeSupportedLocales[locale];
            }
            return obj;
        }, {});
        var skipI18n = false;
        var templateSource = schematics.apply(schematics.url('ng-add/i18n/files'), [
            schematics.applyTemplates(__assign({}, core.strings, options, { moduleName: moduleName,
                commonLocales: commonLocales,
                localesArr: localesArr })),
            schematics.move(moduleDir),
            schematics.forEach(function (entry) {
                var path = entry.path;
                if (host.exists(path)) {
                    if (path.endsWith('.i18n.ts')) {
                        console.warn("File \"" + path.slice(1) + "\" already exists. Skipping i18n " +
                            'generation.');
                        skipI18n = true;
                    }
                    return null;
                }
                return entry;
            }),
        ]);
        // Per language sources
        var localeTemplateSources = [];
        if (!skipI18n) {
            try {
                for (var localesArr_1 = __values(localesArr), localesArr_1_1 = localesArr_1.next(); !localesArr_1_1.done; localesArr_1_1 = localesArr_1.next()) {
                    var locale = localesArr_1_1.value;
                    localeTemplateSources.push(schematics.apply(schematics.url('ng-add/i18n/per-locale-files'), [
                        schematics.applyTemplates(__assign({}, core.strings, options, { moduleName: moduleName,
                            localesArr: localesArr,
                            locale: locale })),
                        schematics.move(moduleDir + "/" + options.i18nFolderName),
                        schematics.forEach(function (entry) {
                            return host.exists(entry.path) ? null : entry;
                        }),
                    ]));
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (localesArr_1_1 && !localesArr_1_1.done && (_a = localesArr_1.return)) _a.call(localesArr_1);
                }
                finally { if (e_1) throw e_1.error; }
            }
        }
        return schematics.chain(__spread([
            saveI18nOptions(options)
        ], (skipI18n
            ? []
            : __spread([
                schematics.mergeWith(templateSource)
            ], localeTemplateSources.map(function (source) { return schematics.mergeWith(source); }), [
                importAppI18n(options),
            ]))));
    };
}
/**
 * Imports the i18n object in the module.
 * @param options Schematic options.
 * @returns Schematic rule.
 */
function importAppI18n(options) {
    return function (host) {
        var project = schematics$1.getProject(host, options.project);
        var modulePath = schematics$1.getModulePath(host, project, options.module);
        var moduleName = schematics$1.getModuleNameFromPath(modulePath);
        schematics$1.addNgModuleValueProvider(host, modulePath, 'LF_APP_I18N', '@lightweightform/core', core.strings.camelize(moduleName) + "I18n", "./" + core.strings.dasherize(moduleName) + ".i18n");
    };
}
/**
 * Saves the app locales and name of the i18n folder within `angular.json` when
 * necessary.
 * @param options Schematic options.
 * @returns Schematic rule.
 */
function saveI18nOptions(options) {
    return function (host) {
        schematics$1.setProjectSchematicOptions(host, options.project, '@lightweightform/bootstrap-theme:form', { locales: options.locales });
        if (options.i18nFolderName !== defaultI18nFolderName) {
            schematics$1.setProjectSchematicOptions(host, options.project, '@lightweightform/bootstrap-theme:form', { i18nFolderName: options.i18nFolderName });
        }
    };
}

/**
 * Content for the HTML file that should contain an `<lf-app>`.
 * @param setRouterOutlet Whether to set the `<router-outlet>`.
 * @returns File content containing an `<lf-app>`.
 */
function htmlComponentContent(setRouterOutlet) {
    if (setRouterOutlet === void 0) { setRouterOutlet = true; }
    return "<lf-app>\n  <!-- Top-bar of the application -->\n  <lf-top-bar [actions]=\"actions\">\n    <!-- Title -->\n    <lf-i18n key=\"label\"></lf-i18n>\n  </lf-top-bar>\n\n  <!-- Tree-nav -->\n  <lf-tree-nav></lf-tree-nav>\n\n  <!-- Content -->\n  " + (setRouterOutlet
        ? '<router-outlet></router-outlet>'
        : '<!-- <router-outlet></router-outlet> -->') + "\n</lf-app>";
}
/**
 * Actions to set as a property in the component's TS file.
 * @param localesArr Array of locales the application should support.
 * @returns Actions to set in the component's TS file.
 */
function tsComponentActions(localesArr) {
    return "[\n  {\n    id: 'save',\n    style: 'outline-secondary',\n    icon: 'save',\n    callback: () => this.save()\n  },\n  {\n    id: 'load',\n    style: 'outline-secondary',\n    icon: 'folder-open',\n    isDisabled: () => !this.lfFileStorage.loadIsSupported,\n    callback: () => this.load()\n  }," + (localesArr.length > 1
        ? "\n  {\n    id: 'language',\n    style: 'outline-primary',\n    icon: 'language',\n    options: this.lfI18n.languages.map(lang => ({\n      id: lang,\n      value: lang,\n      isActive: () => this.lfI18n.currentLanguage === lang\n    })),\n    callback: lang => this.lfI18n.setCurrentLanguage(lang)\n  },"
        : '') + "\n  {\n    id: 'validate',\n    style: 'outline-danger',\n    icon: 'check-square-o',\n    callback: () => this.lfApp.validate()\n  },\n  {\n    id: 'submit',\n    style: 'outline-success',\n    icon: 'send',\n    callback: () => this.submit()\n  }\n]";
}
/**
 * Method signature and body to set as the `ngOnInit` method of the component's
 * TS file.
 * @returns Method signature and body to set in the component's TS file.
 */
function tsComponentNgOnInit() {
    return "() {\n  if (!isDevMode()) {\n    this.lfUnloadAlert.enable();\n  }\n}";
}
/**
 * Method signature and body to set as the `save` method of the component's TS
 * file.
 * @returns Method signature and body to set in the component's TS file.
 */
function tsComponentSave(moduleName) {
    return "(): Promise<void> {\n  try {\n    const dateStr = new Date().toISOString().replace(/[T:.]/g, '-');\n    const fileName = `" + moduleName + "-${dateStr}`;\n    await this.lfFileStorage.saveToFile('/', fileName);\n    this.lfStorage.setPristine();\n  } catch (err) {\n    console.error('Error saving file:', err);\n  }\n}";
}
/**
 * Method signature and body to set as the `load` method of the component's TS
 * file.
 * @returns Method signature and body to set in the component's TS file.
 */
function tsComponentLoad() {
    return "(): Promise<void> {\n  try {\n    await this.lfFileStorage.loadFromFile('/');\n    this.lfStorage.setTouched('/', true);\n  } catch (err) {\n    console.error('Error loading file:', err);\n  }\n}";
}
/**
 * Method signature and body to set as the `submit` method of the component's TS
 * file.
 * @returns Method signature and body to set in the component's TS file.
 */
function tsComponentSubmit() {
    return "(): void {\n  if (!this.lfStorage.hasErrors()) {\n    const appJSON = JSON.stringify(this.lfStorage.getAsJS());\n    alert(appJSON); // Do something with the JSON\n  } else {\n    this.lfApp.validate();\n  }\n}";
}

/**
 * Adds an `<lf-app>` to the project.
 * @param options Schematic options.
 * @returns Schematic rule.
 */
function addLfApp(options) {
    return function (host) {
        var project = schematics$1.getProject(host, options.project);
        var modulePath = schematics$1.getModulePath(host, project, options.module);
        var moduleName = schematics$1.getModuleNameFromPath(modulePath);
        var moduleDir = core.dirname(modulePath);
        var componentTsPath = core.join(moduleDir, moduleName + ".component.ts");
        var componentHtmlPath = core.join(moduleDir, moduleName + ".component.html");
        var localesArr = getLocalesArrayFromString(options.locales);
        if (!host.exists(componentTsPath)) {
            return schematics.chain([
                schematics.externalSchematic('@schematics/angular', 'component', __assign({}, options, { flat: true, path: moduleDir, name: moduleName })),
                function (h) { return h.overwrite(componentHtmlPath, htmlComponentContent()); },
                editComponentTs(moduleName, localesArr, componentTsPath),
            ]);
        }
        return schematics.chain([
            appendLfAppHtml(moduleName, localesArr, componentTsPath, componentHtmlPath),
        ]);
    };
}
/**
 * Append the `<lf-app>` to the given HTML file and set the default `actions` in
 * the TS file.
 * @param componentTsPath Component's TS file path.
 * @param componentHtmlPath Component's HTML file path.
 * @returns Schematic rule.
 */
function appendLfAppHtml(moduleName, localesArr, componentTsPath, componentHtmlPath) {
    return function (host) {
        if (!host.exists(componentHtmlPath)) {
            return schematics.chain([
                function (h) { return h.create(componentHtmlPath, htmlComponentContent()); },
                editComponentTs(moduleName, localesArr, componentTsPath),
            ]);
        }
        var currentHtml = host.read(componentHtmlPath).toString('utf-8');
        if (!schematics$1.htmlContainsTag(currentHtml, 'lf-app')) {
            return schematics.chain([
                function (h) {
                    var newHtml;
                    if ((moduleName === 'app' &&
                        schematics$1.isUnmodifiedHtmlAppComponent(currentHtml)) ||
                        (moduleName !== 'app' &&
                            schematics$1.isUnmodifiedHtmlModuleComponent(currentHtml, moduleName))) {
                        newHtml = htmlComponentContent();
                    }
                    else {
                        console.warn("Prepended `<lf-app>` in \"" + componentHtmlPath.slice(1) + "\" " +
                            'since we cannot safely replace the existing content. Make ' +
                            'sure to adapt the file to your needs.');
                        newHtml =
                            htmlComponentContent(!schematics$1.htmlContainsTag(currentHtml, 'router-outlet')) + ("\n\n" + currentHtml);
                    }
                    h.overwrite(componentHtmlPath, newHtml);
                },
                editComponentTs(moduleName, localesArr, componentTsPath),
            ]);
        }
    };
}
/**
 * Sets the default `actions` in the component's TS file and the `ngOnInit`.
 * @param moduleName Name of the module we are working on.
 * @param componentTsPath Component's TS file path.
 * @returns Schematic rule.
 */
function editComponentTs(moduleName, localesArr, componentTsPath) {
    return function (host) {
        // Set actions property
        schematics$1.addTsComponentProperty(host, componentTsPath, 'actions', 'ActionsMenuAction[]', tsComponentActions(localesArr), undefined, [new schematics$1.TsImport('ActionsMenuAction', '@lightweightform/bootstrap-theme')]);
        // Add a property for referencing the `<lf-app>` (to call `validate` on)
        var importName = moduleName === 'app' ? 'LfAppComponent' : 'AppComponent';
        schematics$1.addTsComponentProperty(host, componentTsPath, 'lfApp', importName, undefined, ["@ViewChild(" + importName + ", {static: false})"], [
            new schematics$1.TsImport('AppComponent', '@lightweightform/bootstrap-theme', importName),
            new schematics$1.TsImport('ViewChild', '@angular/core'),
        ]);
        // Inject `LfStorage`, `LfFileStorage`, `LfI18n`, and `LfUnloadAlert` when
        // appropriate
        schematics$1.addTsComponentConstructorParameter(host, componentTsPath, 'lfStorage', 'LfStorage', ['public'], [new schematics$1.TsImport('LfStorage', '@lightweightform/core')]);
        schematics$1.addTsComponentConstructorParameter(host, componentTsPath, 'lfFileStorage', 'LfFileStorage', ['public'], [new schematics$1.TsImport('LfFileStorage', '@lightweightform/core')]);
        if (localesArr.length > 1) {
            schematics$1.addTsComponentConstructorParameter(host, componentTsPath, 'lfI18n', 'LfI18n', ['public'], [new schematics$1.TsImport('LfI18n', '@lightweightform/core')]);
        }
        schematics$1.addTsComponentConstructorParameter(host, componentTsPath, 'lfUnloadAlert', 'LfUnloadAlert', ['public'], [new schematics$1.TsImport('LfUnloadAlert', '@lightweightform/core')]);
        // Add mock ngOnInit, save, load, and submit methods
        schematics$1.addTsComponentInterface(host, componentTsPath, 'OnInit', new schematics$1.TsImport('OnInit', '@angular/core'));
        schematics$1.addTsComponentMethod(host, componentTsPath, 'ngOnInit', tsComponentNgOnInit(), undefined, [new schematics$1.TsImport('isDevMode', '@angular/core')]);
        schematics$1.addTsComponentMethod(host, componentTsPath, 'save', tsComponentSave(moduleName), ['async']);
        schematics$1.addTsComponentMethod(host, componentTsPath, 'load', tsComponentLoad(), [
            'async',
        ]);
        schematics$1.addTsComponentMethod(host, componentTsPath, 'submit', tsComponentSubmit());
    };
}

/**
 * Adds the theme module and required services to the root `NgModule`.
 * @param options Schematic options.
 * @returns Schematic rule.
 */
function addThemeModuleAndProviders(options) {
    return function (host) {
        var project = schematics$1.getProject(host, options.project);
        var modulePath = schematics$1.getModulePath(host, project, options.module);
        // Add `LfBootstrapThemeModule`
        schematics$1.addNgModuleImport(host, modulePath, 'LfBootstrapThemeModule', '@lightweightform/bootstrap-theme');
        // Add `LfStorage`, `LfFileStorage`, `LfI18n`, and `LfUnloadAlert` services
        schematics$1.addNgModuleProvider(host, modulePath, 'LfStorage', '@lightweightform/core');
        schematics$1.addNgModuleProvider(host, modulePath, 'LfFileStorage', '@lightweightform/core');
        schematics$1.addNgModuleProvider(host, modulePath, 'LfI18n', '@lightweightform/core');
        schematics$1.addNgModuleProvider(host, modulePath, 'LfUnloadAlert', '@lightweightform/core');
        // Add the JSON serialiser `LfJsonSerializer` as the file storage serialiser
        schematics$1.addNgModuleClassProvider(host, modulePath, 'LfSerializer', '@lightweightform/core', 'LfJsonSerializer', '@lightweightform/core');
        return host;
    };
}

/**
 * Adds the application schema to the project.
 * @param options Schematic options.
 * @returns Schematic rule.
 */
function addLfRouter(options) {
    return function (host) {
        var project = schematics$1.getProject(host, options.project);
        var modulePath;
        try {
            modulePath = schematics$1.getModulePath(host, project, options.module, true);
        }
        catch (err) {
            return console.warn('Routing module not found. Skipping incorporation of `LfRouter`.');
        }
        schematics$1.addNgModuleProvider(host, modulePath, 'LfRouter', '@lightweightform/core');
        schematics$1.addNgModuleValueProvider(host, modulePath, 'LF_ROUTER_BASE_PATH', '@lightweightform/core', "'" + options.routerBasePath + "'");
        var localesArr = getLocalesArrayFromString(options.locales);
        if (localesArr.length > 1) {
            schematics$1.addNgModuleValueProvider(host, modulePath, 'LF_I18N_LANGUAGE_QUERY_PARAM', '@lightweightform/core', "'l'");
        }
        schematics$1.changeRoutesToLfRoutes(host, modulePath);
    };
}

/**
 * Adds the application schema to the project.
 * @param options Schematic options.
 * @returns Schematic rule.
 */
function addAppSchema(options) {
    return function (host) {
        var project = schematics$1.getProject(host, options.project);
        var modulePath = schematics$1.getModulePath(host, project, options.module);
        var moduleDir = core.dirname(modulePath);
        var moduleName = schematics$1.getModuleNameFromPath(modulePath);
        var skipSchema = false;
        var templateSource = schematics.apply(schematics.url('ng-add/schema/files'), [
            options.skipTests
                ? schematics.filter(function (p) { return !p.endsWith('.spec.ts.template'); })
                : schematics.noop(),
            schematics.applyTemplates(__assign({}, core.strings, options, { moduleName: moduleName })),
            schematics.move(moduleDir),
            schematics.forEach(function (entry) {
                var path = entry.path;
                if (host.exists(path)) {
                    if (path.endsWith('.schema.ts')) {
                        console.warn("File \"" + path.slice(1) + "\" already exists. Skipping schema " +
                            'generation.');
                        skipSchema = true;
                    }
                    return null;
                }
                return entry;
            }),
        ]);
        return skipSchema
            ? schematics.noop()
            : schematics.chain([schematics.mergeWith(templateSource), importAppSchema(options)]);
    };
}
/**
 * Imports the schema in the module.
 * @param options Schematic options.
 * @returns Schematic rule.
 */
function importAppSchema(options) {
    return function (host) {
        var project = schematics$1.getProject(host, options.project);
        var modulePath = schematics$1.getModulePath(host, project, options.module);
        var moduleName = schematics$1.getModuleNameFromPath(modulePath);
        schematics$1.addNgModuleValueProvider(host, modulePath, 'LF_APP_SCHEMA', '@lightweightform/core', core.strings.camelize(moduleName) + "Schema", "./" + core.strings.dasherize(moduleName) + ".schema");
    };
}

/**
 * Content for the file that imports LF's Bootstrap-theme styles.
 * @param fileType Type of file where the content will be inserted.
 * @returns File content to import the theme.
 */
function themeStylesContent(fileType) {
    var semi = fileType === 'scss' ? ';' : '';
    // TODO: Get Bootstrap version from dependencies
    return "// Upstream Bootstrap variables and theme variables may be overriden here. See:\n// https://getbootstrap.com/docs/4.3/getting-started/theming/#variable-defaults\n// and https://github.com/twbs/bootstrap/blob/v4.3.1/scss/_variables.scss to\n// override Bootstrap variables. LF theme variables names can be found at:\n// https://bitbucket.org/opensoftgitrepo/lightweightform/src/master/libs/bootstrap-theme/src/scss-utils/_variables.scss\n\n@import '~@lightweightform/bootstrap-theme/scss/bootstrap-theme'" + semi;
}
/**
 * Whether an existing file content already imports the theme's styles.
 * @param content Current file content.
 * @returns Whether the current file current already imports the theme's styles.
 */
function containsImport(content) {
    var importRegExp = /@import\s+('|")~@lightweightform\/bootstrap-theme\/scss\/bootstrap-theme\1/;
    return content.match(importRegExp) !== null;
}

/**
 * File used to import the Bootstrap-theme styles.
 */
var themeStylesFilename = 'lf-bootstrap-theme.scss';
/**
 * Imports the Bootstrap-theme styles to the project.
 * @param options Schematic options.
 * @returns Schematic rule.
 */
function addThemeStyles(options) {
    var projectName = options.project;
    return function (host) {
        var project = schematics$1.getProject(host, projectName);
        var stylesPath = schematics$1.getProjectStyleFilePath(project, 'scss') ||
            schematics$1.getProjectStyleFilePath(project, 'sass');
        var themeStylesPath = themeStylesFilePath(project);
        if (!stylesPath || stylesPath === themeStylesPath) {
            if (host.exists(themeStylesPath)) {
                return console.warn("File \"" + themeStylesPath + "\" already exists. Skipping import of " +
                    'styles.');
            }
            host.create(themeStylesPath, themeStylesContent('scss'));
            schematics$1.addStylePathToProjectTarget(host, projectName, 'build', themeStylesPath);
            schematics$1.addStylePathToProjectTarget(host, projectName, 'test', themeStylesPath);
            return;
        }
        // Add theme styles only if they haven't already been imported
        if (!containsImport(host.read(stylesPath).toString('utf-8'))) {
            var ext = stylesPath.slice(-4);
            var insertion = new change.InsertChange(stylesPath, 0, themeStylesContent(ext) + "\n\n");
            var recorder = host.beginUpdate(stylesPath);
            recorder.insertLeft(insertion.pos, insertion.toAdd);
            host.commitUpdate(recorder);
        }
    };
}
/**
 * File path of the file used to import the Bootstrap-theme styles.
 * @param project Project on which the styles are being added.
 * @returns File path.
 */
function themeStylesFilePath(project) {
    return core.normalize(core.join(project.sourceRoot || core.join(project.root, 'src'), themeStylesFilename));
}

/**
 * Adds application specific styles to the project.
 * @param options Schematic options.
 * @returns Schematic rule.
 */
function addAppStyles(options) {
    return function (host) {
        var project = schematics$1.getProject(host, options.project);
        var styleFilePath = schematics$1.getProjectStyleFilePath(project);
        // Verify that the obtained path is not the path used by the schematics to
        // imports the theme's styles
        if (!styleFilePath || styleFilePath === themeStylesFilePath(project)) {
            return console.error('Could not find the default style file for the project.');
        }
        var buffer = host.read(styleFilePath);
        if (!buffer) {
            return console.error("Could not read \"" + styleFilePath + "\" in the project.");
        }
        var content = buffer.toString('utf-8');
        var indent = schematics$1.getIndentation(content);
        var toInsert = '\n' +
            (styleFilePath.endsWith('sass')
                ? "html, body\n" + indent + "height: 100%\n"
                : "html, body {\n" + indent + "height: 100%;\n}\n");
        if (content.includes(toInsert)) {
            return;
        }
        var recorder = host.beginUpdate(styleFilePath);
        recorder.insertLeft(content.length, toInsert);
        host.commitUpdate(recorder);
    };
}

/**
 * `ng-add-setup` schematic function.
 * @param options Schematic options.
 * @returns Schematic rule.
 */
function ngAddSetup(options) {
    return schematics.chain([
        copyAngularComponentSchematicOptions(options),
        addThemeModuleAndProviders(options),
        addThemeStyles(options),
        addAppStyles(options),
        addAppSchema(options),
        addLfRouter(options),
        addLfApp(options),
        addI18n(options),
        options.lintFix ? lintFix.applyLintFix() : schematics.noop(),
    ]);
}
/**
 * Copies the Angular component schematic options to our form schematic options
 * so that both schematics use the same defaults.
 * @param options Schematic options.
 * @returns Schematic rule.
 */
function copyAngularComponentSchematicOptions(options) {
    return function (host) {
        var ngComponentOptions = schematics$1.getProjectSchematicOptions(host, options.project, '@schematics/angular:component');
        if (ngComponentOptions) {
            schematics$1.setProjectSchematicOptions(host, options.project, '@lightweightform/bootstrap-theme:form', ngComponentOptions);
        }
    };
}

/**
 * HTML content for the form's component.
 * @param storagePath Form's storage path.
 * @returns HTML content for the form's component.
 */
function htmlFormComponentContent(storagePath, routePlaceholders) {
    var pathAssignment = routePlaceholders.length === 0 ? "path=\"" + storagePath + "\"" : '[path]="path"';
    return "<lf-form " + pathAssignment + ">\n</lf-form>";
}
/**
 * HTML content for the collection's component.
 * @param storagePath Collection's storage path.
 * @returns HTML content for the collection's component.
 */
function htmlCollectionComponentContent(storagePath, routePlaceholders) {
    var pathAssignment = routePlaceholders.length === 0 ? "path=\"" + storagePath + "\"" : '[path]="path"';
    return "<lf-form " + pathAssignment + ">\n  <lf-table path=\".\">\n    <lf-table-header>\n      <lf-table-column id=\"navigation\"></lf-table-column>\n    </lf-table-header>\n\n    <ng-template #lfTableRowTemplate let-id=\"id\">\n      <tr lf-table-row [path]=\"id\">\n        <td lf-table-cell>\n          <a\n            lf-button\n            size=\"sm\"\n            [style]=\"'outline-success'\"\n            [block]=\"true\"\n            [routerLink]=\"id\"\n            queryParamsHandling=\"merge\"\n          >\n            <lf-icon icon=\"share-square-o\"></lf-icon>\n            <lf-i18n path=\"..\" key=\"navigateButtonLabel\"></lf-i18n>\n          </a>\n        </td>\n      </tr>\n    </ng-template>\n  </lf-table>\n</lf-form>";
}
/**
 * Content for the getter used to get a component's storage path.
 * @param storagePath Component's storage path.
 * @param routeParams Param names for the component's route.
 * @returns `path` getter.
 */
function tsComponentPathGetterContent(storagePath, routeParams) {
    var paramsAsIds = routeParams.map(function (p) {
        return schematics$1.validTsIdRegExp.test(p) ? p : core.strings.camelize(p);
    });
    var paramsAsProps = routeParams.map(function (p, i) {
        return p === paramsAsIds[i] ? p : schematics$1.stringAsProperty(p) + ": " + paramsAsIds[i];
    });
    var j = 0;
    var pathWithParams = storagePath
        .split('/')
        .map(function (id) { return (id === '?' ? "${" + paramsAsIds[j++] + "}" : id); })
        .join('/');
    return "{\n  const { " + paramsAsProps.join(', ') + " } = this.lfRouter.params;\n  return `" + pathWithParams + "`;\n}\n";
}

/**
 * `form` schematic function.
 * @param options Schematic options.
 * @returns Schematic rule.
 */
function form(options) {
    return function (host) {
        var project = schematics$1.getProject(host, options.project);
        var moduleDir = core.dirname(schematics$1.getModulePath(host, project, options.module));
        var formPath = core.join(moduleDir, core.dirname(options.name));
        var formName = core.basename(options.name);
        options.formName = formName;
        var collectionPath;
        var collectionName;
        if (options.collection) {
            collectionPath = core.dirname(formPath);
            if (options.flat) {
                formPath = collectionPath;
            }
            collectionName = core.basename(core.dirname(options.name));
            if (collectionName === '.') {
                throw new schematics.SchematicsException('The provided name does not include the collection name. A name ' +
                    'is expected in the format: "collection-name/form-name", ' +
                    'possibly prepended by the path where the collection should be ' +
                    'created.');
            }
        }
        var formStoragePathId = core.strings.dasherize(formName);
        var formSchemaPath = core.join(formPath, options.flat ? '' : core.strings.dasherize(formName), core.strings.dasherize(formName) + '.schema.ts');
        var collectionStoragePathId;
        var collectionSchemaPath;
        if (options.collection) {
            formStoragePathId = '?';
            collectionStoragePathId = core.strings.dasherize(collectionName);
            collectionSchemaPath = core.join(collectionPath, options.flat ? '' : core.strings.dasherize(collectionName), core.strings.dasherize(collectionName) + '.schema.ts');
        }
        var schemaExplorer = new schematics$1.SchemaExplorer(host, moduleDir);
        var parentStoragePath = options.parentPath ||
            schematics$1.getParentStoragePath(schemaExplorer, options.collection ? collectionSchemaPath : formSchemaPath);
        var routerModulePath = schematics$1.getModulePath(host, project, options.module, true);
        var parentRouteInfo = schematics$1.getLfRouteInfo(host, routerModulePath, parentStoragePath);
        var routeParams = parentRouteInfo
            ? parentRouteInfo.params
            : // Default params from parent storage path
                parentStoragePath
                    .split('/')
                    .filter(function (id) { return id === '?'; })
                    .map(function (_, i) { return "param-" + (i + 1); });
        return schematics.chain([
            options.collection
                ? createCollection(options, schemaExplorer, moduleDir, routerModulePath, formName, collectionPath, collectionName, collectionSchemaPath, formSchemaPath, collectionStoragePathId, parentStoragePath, routeParams)
                : schematics.noop(),
            createForm(options, schemaExplorer, moduleDir, routerModulePath, formPath, formName, formSchemaPath, formStoragePathId, options.collection
                ? core.join(parentStoragePath, collectionStoragePathId)
                : parentStoragePath, options.collection
                ? __spread(routeParams, [core.strings.dasherize(formName) + "-id"]) : routeParams),
        ]);
    };
}
/**
 * Creates the form collection.
 * @param options Schematic options.
 * @param schemaExplorer `SchemaExplorer` instance.
 * @param moduleDir Module directory where to find the root schema.
 * @param routerModulePath Path of the router module.
 * @param formName Form name.
 * @param path Path where the collection is being created (independently of
 * whether it is flat).
 * @param name Collection name.
 * @param schemaPath Path of the collection's schema.
 * @param formSchemaPath Path of the form's schema.
 * @param storagePathId Id of the collection in the storage path.
 * @param parentStoragePath Storage path of the collection's parent.
 * @param routeParams Param names for the collection's route.
 * @returns Schematic rule.
 */
function createCollection(options, schemaExplorer, moduleDir, routerModulePath, formName, path, name, schemaPath, formSchemaPath, storagePathId, parentStoragePath, routeParams) {
    return function () {
        var formSchemaImportPath = schematics$1.relativeTsImportPath(schemaPath, formSchemaPath);
        var storagePath = core.join(parentStoragePath, storagePathId);
        return schematics.chain([
            options.skipCollectionComponent
                ? schematics.noop()
                : schematics.externalSchematic('@schematics/angular', 'component', __assign({}, options, { path: path,
                    name: name, lintFix: false })),
            addSchema(options, schemaExplorer, path, name, schemaPath, storagePathId, storagePath, parentStoragePath, 'collection-files', { formName: formName, formSchemaImportPath: formSchemaImportPath }),
            addI18n$1(options, moduleDir, path, name, storagePath, 'collection-per-locale-files'),
            options.skipCollectionComponent
                ? schematics.noop()
                : setHtml(options, path, name, htmlCollectionComponentContent(storagePath, routeParams)),
            options.skipCollectionComponent
                ? schematics.noop()
                : editComponentTs$1(options, path, name, storagePath, routeParams),
            options.skipCollectionComponent
                ? schematics.noop()
                : addRoute(options, routerModulePath, path, name, storagePath, routeParams),
            options.lintFix ? lintFix.applyLintFix() : schematics.noop(),
        ]);
    };
}
/**
 * Creates the form.
 * @param options Schematic options.
 * @param schemaExplorer `SchemaExplorer` instance.
 * @param moduleDir Module directory where to find the root schema.
 * @param routerModulePath Path of the router module.
 * @param path Path where the form is being created (independently of whether it
 * is flat).
 * @param name Form name.
 * @param schemaPath Path of the form's schema.
 * @param storagePathId Id of the form in the storage path.
 * @param parentStoragePath Storage path of the form's parent.
 * @param routeParams Param names for the form's route.
 * @returns Schematic rule.
 */
function createForm(options, schemaExplorer, moduleDir, routerModulePath, path, name, schemaPath, storagePathId, parentStoragePath, routeParams) {
    return function () {
        var storagePath = core.join(parentStoragePath, storagePathId);
        return schematics.chain([
            schematics.externalSchematic('@schematics/angular', 'component', __assign({}, options, { path: path,
                name: name, lintFix: false })),
            addSchema(options, schemaExplorer, path, name, schemaPath, storagePathId, storagePath, parentStoragePath, 'form-files', {}, !options.collection),
            addI18n$1(options, moduleDir, path, name, storagePath, 'form-per-locale-files'),
            setHtml(options, path, name, htmlFormComponentContent(storagePath, routeParams)),
            editComponentTs$1(options, path, name, storagePath, routeParams),
            addRoute(options, routerModulePath, path, name, storagePath, routeParams),
            options.lintFix ? lintFix.applyLintFix() : schematics.noop(),
        ]);
    };
}
/**
 * Add needed schema files.
 * @param options Schematic options.
 * @param schemaExplorer `SchemaExplorer` instance.
 * @param path Path where the component is being created (independently of
 * whether it is flat).
 * @param name Component name.
 * @param schemaPath Path of the component's schema to add.
 * @param storagePathId Identifier of the component's schema in the storage
 * path.
 * @param storagePath Path of the component's schema in the storage.
 * @param parentStoragePath Parent path of the component's schema in the
 * storage.
 * @param filesDir Directory containing the files to be created.
 * @param addToParent Whether to add the component's schema to the parent
 * schema.
 * @returns Schematic rule.
 */
function addSchema(options, schemaExplorer, path, name, schemaPath, storagePathId, storagePath, parentStoragePath, filesDir, extraTemplateBindings, addToParent) {
    if (extraTemplateBindings === void 0) { extraTemplateBindings = {}; }
    if (addToParent === void 0) { addToParent = true; }
    return function (host) {
        var tsSchemaName = core.strings.camelize(name) + "Schema";
        var rootSchemaInfo = schemaExplorer.getSchemaInfoFromStoragePath('/');
        var rootSchemaImportPath = schematics$1.relativeTsImportPath(schemaPath, rootSchemaInfo.fileInfo.filePath);
        var templateSource = schematics.apply(schematics.url("form/" + filesDir), [
            options.skipTests
                ? schematics.filter(function (p) { return !p.endsWith('.spec.ts.template'); })
                : schematics.noop(),
            schematics.applyTemplates(__assign({}, core.strings, options, { 'if-flat': function (str) { return (options.flat ? '' : str); }, name: name,
                storagePath: storagePath, rootTsSchemaName: schemaExplorer.rootSchemaName, rootSchemaImportPath: rootSchemaImportPath }, extraTemplateBindings)),
            schematics.move(path),
        ]);
        if (addToParent) {
            schematics$1.addSchemaToParentSchema(host, schemaExplorer, schemaPath, tsSchemaName, storagePathId, parentStoragePath);
        }
        return schematics.chain([schematics.mergeWith(templateSource)]);
    };
}
/**
 * Add component's i18n objects.
 * @param options Schematic options.
 * @param moduleDir Module directory where to find the root i18n file.
 * @param path Path where the component is being created (independently of
 * whether it is flat).
 * @param name Component name.
 * @param storagePath Path of the component's schema in the storage.
 * @param filesDir Directory where to find the "per-locale" i18n files.
 * @returns Schematic rule.
 */
function addI18n$1(options, moduleDir, path, name, storagePath, filesDir) {
    return function (host) {
        var e_1, _a;
        var localesArr = getLocalesArrayFromString(options.locales);
        var rootI18nPath = schematics$1.getRootI18nPath(host, moduleDir);
        var i18nDirPath = core.join(path, options.flat ? '' : core.strings.dasherize(name), options.i18nFolderName);
        // Per language sources
        var languageTemplateSources = [];
        try {
            for (var localesArr_1 = __values(localesArr), localesArr_1_1 = localesArr_1.next(); !localesArr_1_1.done; localesArr_1_1 = localesArr_1.next()) {
                var locale = localesArr_1_1.value;
                languageTemplateSources.push(schematics.apply(schematics.url("form/" + filesDir), [
                    schematics.applyTemplates(__assign({}, core.strings, options, { name: name,
                        locale: locale,
                        storagePath: storagePath })),
                    schematics.move(i18nDirPath),
                ]));
                // Add locale to the root i18n
                var localePath = core.join(i18nDirPath, core.strings.dasherize(name) + ".i18n." + locale + ".ts");
                var localeTsName = core.strings.camelize(name) + "I18n" + core.strings.classify(locale);
                schematics$1.addFormLocaleToRootI18n(host, rootI18nPath, locale, localePath, localeTsName);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (localesArr_1_1 && !localesArr_1_1.done && (_a = localesArr_1.return)) _a.call(localesArr_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return schematics.chain(__spread(languageTemplateSources.map(function (source) { return schematics.mergeWith(source); })));
    };
}
/**
 * Sets the component's HTML content.
 * @param options Schematic options.
 * @param path Path where the component is being created (independently of
 * whether it is flat).
 * @param name Component name.
 * @param htmlContent HTML content to set.
 * @returns Schematic rule.
 */
function setHtml(options, path, name, htmlContent) {
    return function (host) {
        if (!options.inlineTemplate) {
            var componentHtmlPath = core.join(path, options.flat ? '' : core.strings.dasherize(name), core.strings.dasherize(name) + '.component.html');
            host.overwrite(componentHtmlPath, htmlContent);
        }
        else {
            var componentTsPath = core.join(path, options.flat ? '' : core.strings.dasherize(name), core.strings.dasherize(name) + '.component.ts');
            schematics$1.setTsComponentInlineTemplate(host, componentTsPath, htmlContent);
        }
    };
}
/**
 * Adds a way of accessing the component's path when needed.
 * @param options Schematic options.
 * @param path Path where the component is being created (independently of
 * whether it is flat).
 */
function editComponentTs$1(options, path, name, storagePath, routeParams) {
    return function (host) {
        if (routeParams.length > 0) {
            var componentTsPath = core.join(path, options.flat ? '' : core.strings.dasherize(name), core.strings.dasherize(name) + '.component.ts');
            // Add `lfRouter` to constructor
            schematics$1.addTsComponentConstructorParameter(host, componentTsPath, 'lfRouter', 'LfRouter', ['private'], [new schematics$1.TsImport('LfRouter', '@lightweightform/core')]);
            // Set `path` getter
            schematics$1.addTsComponentGetter(host, componentTsPath, 'path', 'string', tsComponentPathGetterContent(storagePath, routeParams), ['@computed'], [new schematics$1.TsImport('computed', 'mobx')]);
        }
    };
}
/**
 * Add a route to the form.
 * @param options Schematic options.
 * @param routerModulePath Path of the router module.
 * @param name Form name.
 * @param project Project where the form is being created.
 * @param storagePath Path of the form in the storage.
 * @returns Schematic rule.
 */
function addRoute(options, routerModulePath, path, name, storagePath, routeParams) {
    return function (host) {
        if (!routerModulePath) {
            return console.warn('Routing module not found. Skipping insertion of component route.');
        }
        var i = 0;
        var routePath = storagePath
            .slice(1)
            .split('/')
            .map(function (id) { return (id === '?' ? ":" + routeParams[i++] : id); })
            .join('/');
        // Add a redirect to the route when there are no routes
        if (!schematics$1.hasRoutes(host, routerModulePath)) {
            schematics$1.addLfRoute(host, routerModulePath, {
                routePath: '',
                storagePath: '/',
                redirectTo: routePath,
                pathMatch: 'full',
            });
        }
        var componentTsName = core.strings.classify(name) + "Component";
        var componentFilePath = core.join(path, options.flat ? '' : core.strings.dasherize(name), core.strings.dasherize(name) + '.component.ts');
        schematics$1.addLfRoute(host, routerModulePath, {
            routePath: routePath,
            storagePath: storagePath,
            componentTsName: componentTsName,
            componentFilePath: componentFilePath,
        });
    };
}

exports.createCollection = createCollection;
exports.createForm = createForm;
exports.form = form;
exports.ngAddSetup = ngAddSetup;
//# sourceMappingURL=index.js.map
