import { ViewportRuler, ScrollingModule } from '@angular/cdk/scrolling';
import { CommonModule } from '@angular/common';
import { Component, ChangeDetectionStrategy, Inject, Input, EventEmitter, Output, HostBinding, ViewChild, Optional, ElementRef, ViewChildren, ContentChild, Directive, HostListener, NgModule } from '@angular/core';
import { Router, RouterModule } from '@angular/router';
import { LfI18n, PathBased, LfRouter, LfStorage, LfFileStorage, LfCoreModule, I18N_UNLOAD_ALERT_MESSAGE_KEY } from '@lightweightform/core';
import { asap, ScrollbarUtils, TableHeaderDirective, LfTableHeaderModule } from '@lightweightform/theme-common';
import 'bootstrap';
import { observable as observable$1, computed, action, MobxAngularModule } from 'mobx-angular';
import { BsDatepickerModule } from 'ngx-bootstrap/datepicker';
import { TooltipModule } from 'ngx-bootstrap/tooltip';
import { MarkdownService, MarkdownModule } from 'ngx-markdown';
import { __decorate, __metadata, __awaiter } from 'tslib';
import { extendObservable, observable, observe, runInAction, autorun, comparer, reaction } from 'mobx';
import { isRecordSchema, isNullableSchema, pathError, isListSchema, isBooleanSchema, tupleSchema, dateSchema, isTupleSchema, isDateSchema, isNumberSchema, isTableSchema, getRowId, appendToPath, isStringSchema, isComplexSchema, isReservedStateProp, getStateProperty, resolvePath, IS_REJECTED_CODE, IS_REQUIRED_CODE, UNALLOWED_VALUE_CODE, DATE_OUT_OF_BOUNDS_CODE, SIZE_OUT_OF_BOUNDS_CODE, NUMBER_OUT_OF_BOUNDS_CODE, LENGTH_OUT_OF_BOUNDS_CODE } from '@lightweightform/storage';
import { PENDING, isPromiseBasedObservable } from 'mobx-utils';
import $ from 'jquery';
import { animationFrameScheduler, asapScheduler } from 'rxjs';
import { auditTime } from 'rxjs/operators';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// Value =======================================================================
/**
 * I18n key used to associate a label with a value.
 * @type {?}
 */
const I18N_VALUE_LABEL_KEY = 'label';
/**
 * I18n key used to associate a code with a value.
 * @type {?}
 */
const I18N_VALUE_CODE_KEY = 'code';
/**
 * I18n key used to associate a help message with a value.
 * @type {?}
 */
const I18N_VALUE_HELP_MESSAGE_KEY = 'helpMessage';
/**
 * I18n key used to associate a legend with a value.
 * @type {?}
 */
const I18N_VALUE_LEGEND_KEY = 'legend';
/**
 * I18n key used to associate messages with validation errors.
 * @type {?}
 */
const I18N_VALIDATIONS_KEY = 'validations';
/**
 * I18n key used to ask the user to retry validating the value.
 * @type {?}
 */
const I18N_VALUE_RETRY_VALIDATE_TEXT_KEY = 'retryValidateText';
// Tree-nav ====================================================================
/**
 * I18n key used to set the text of the dropdown button used to add optional
 * forms.
 * @type {?}
 */
const I18N_TREE_NAV_ADD_OPTIONAL_VALUE_BUTTON_TEXT_KEY = 'addOptionalValueButtonText';
/**
 * I18n key used to set the text of the button used to add forms to a list of
 * forms.
 * @type {?}
 */
const I18N_TREE_NAV_ADD_VALUE_TO_LIST_BUTTON_TEXT_KEY = 'addValueToListButtonText';
// Closeable ===================================================================
/**
 * I18n key used to set the text of the tooltip of the "close" button in
 * components.
 * @type {?}
 */
const I18N_CLOSE_BUTTON_TOOLTIP_KEY = 'closeButtonTooltip';
// Actions-menu ================================================================
/**
 * I18n key used to set the text of the actions-menu button when it is in
 * "mobile" mode.
 * @type {?}
 */
const I18N_ACTIONS_MENU_BUTTON_TEXT_KEY = 'actionsButtonText';
/**
 * I18n key used to translate the text of the buttons.
 * @type {?}
 */
const I18N_ACTIONS_MENU_KEY = 'actionsMenu';
/**
 * I18n key used to translate the text of the options of the buttons.
 * @type {?}
 */
const I18N_ACTIONS_MENU_OPTIONS_KEY = 'actionsMenuOptions';
// Validation panel ============================================================
/**
 * I18n key used to set the "errors" title of validation panel.
 * @type {?}
 */
const I18N_VALIDATION_PANEL_ERRORS_TEXT_KEY = 'errorsText';
/**
 * I18n key used to set the "warnings" title of validation panel.
 * @type {?}
 */
const I18N_VALIDATION_PANEL_WARNINGS_TEXT_KEY = 'warningsText';
/**
 * I18n key used to set the text that shows up in the validation panel when
 * there are no errors or warnings.
 * @type {?}
 */
const I18N_VALIDATION_PANEL_NO_ISSUES_TEXT_KEY = 'noIssuesText';
/**
 * I18n key used to set the text of the tooltip of the "focus" button in the
 * validation panel.
 * @type {?}
 */
const I18N_VALIDATION_PANEL_FOCUS_TEXT_KEY = 'focusButtonTooltip';
/**
 * I18n key used to set the text of the tooltip of the "next" button in the
 * validation panel.
 * @type {?}
 */
const I18N_VALIDATION_PANEL_NEXT_BUTTON_TOOLTIP_KEY = 'nextButtonTooltip';
/**
 * I18n key used to set the text of the tooltip of the "previous" button in the
 * validation panel.
 * @type {?}
 */
const I18N_VALIDATION_PANEL_PREV_BUTTON_TOOLTIP_KEY = 'prevButtonTooltip';
// Form ========================================================================
/**
 * I18n key used to set the text of a form component's help action.
 * @type {?}
 */
const I18N_FORM_HELP_ACTION_TEXT_KEY = 'helpActionText';
/**
 * I18n key used to set the text of a form component's remove action.
 * @type {?}
 */
const I18N_FORM_REMOVE_ACTION_TEXT_KEY = 'removeActionText';
/**
 * I18n key used to set the message displayed to the user in a form component to
 * confirm removal of a form.
 * @type {?}
 */
const I18N_FORM_ENSURE_REMOVE_TEXT_KEY = 'ensureRemoveText';
/**
 * I18n key used to set the text of a form component's "cancel" button in the
 * removal modal.
 * @type {?}
 */
const I18N_FORM_CANCEL_REMOVE_BUTTON_TEXT_KEY = 'cancelRemoveButtonText';
/**
 * I18n key used to set the text of a form component's "confirm" button in the
 * removal modal.
 * @type {?}
 */
const I18N_FORM_CONFIRM_REMOVE_BUTTON_TEXT_KEY = 'confirmRemoveButtonText';
// Date ========================================================================
/**
 * I18n key used to set the format of a date component.
 * @type {?}
 */
const I18N_DATE_FORMAT_KEY = 'format';
/**
 * I18n keys used to set the placeholder of a date component's input field.
 * @type {?}
 */
const I18N_DATE_PLACEHOLDER_KEY = 'placeholder';
// Date range ==================================================================
/**
 * I18n keys used to set the separator of the date range.
 * @type {?}
 */
const I18N_DATE_RANGE_SEPARATOR_KEY = 'separator';
// Number ======================================================================
/**
 * I18n key used to set the prefix of a number component.
 * @type {?}
 */
const I18N_NUMBER_PREFIX_KEY = 'prefix';
/**
 * I18n key used to set the suffix of a number component.
 * @type {?}
 */
const I18N_NUMBER_SUFFIX_KEY = 'suffix';
/**
 * I18n key used to set the decimal separator of a number component.
 * @type {?}
 */
const I18N_NUMBER_DECIMAL_SEPARATOR_KEY = 'decimalSeparator';
/**
 * I18n key used to set the "thousands separator" of a number component.
 * @type {?}
 */
const I18N_NUMBER_THOUSANDS_SEPARATOR_KEY = 'thousandsSeparator';
// Table =======================================================================
/**
 * I18n key used to translate the "add row" text of the table component.
 * @type {?}
 */
const I18N_TABLE_ADD_ROW_ACTION_TEXT_KEY = 'addRowActionText';
/**
 * I18n key used to translate the "remove rows" text of the table component.
 * @type {?}
 */
const I18N_TABLE_REMOVE_ROWS_ACTION_TEXT_KEY = 'removeRowsActionText';
/**
 * I18n key used to translate the "no rows" text of the table component.
 * @type {?}
 */
const I18N_TABLE_NO_ROWS_TEXT_KEY = 'noRowsText';
/**
 * I18n key used to set the labels of the table columns in the table component.
 * @type {?}
 */
const I18N_TABLE_COLUMN_LABELS_KEY = 'columnLabels';
// Selection ===================================================================
/**
 * I18n key used to specify the list of options.
 * @type {?}
 */
const I18N_SELECTION_OPTIONS_KEY = 'options';
/**
 * I18n key used to translate the "no options" message on components that
 * require options.
 * @type {?}
 */
const I18N_SELECTION_NO_OPTIONS_TEXT_KEY = 'noOptionsText';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Action of an actions-menu.
 * @record
 */
function ActionsMenuAction() { }
if (false) {
    /**
     * Identifier of the action (used to identify the action in the i18n file).
     * @type {?|undefined}
     */
    ActionsMenuAction.prototype.id;
    /**
     * Text of the action (which may also be specified via the i18n file).
     * @type {?|undefined}
     */
    ActionsMenuAction.prototype.text;
    /**
     * Class name to add to the action.
     * @type {?|undefined}
     */
    ActionsMenuAction.prototype.class;
    /**
     * Style of the action (relevant when the actions aren't collapsable).
     * @type {?|undefined}
     */
    ActionsMenuAction.prototype.style;
    /**
     * (Font Awesome) icon of the action.
     * @type {?|undefined}
     */
    ActionsMenuAction.prototype.icon;
    /**
     * Whether the action should show as active.
     * @type {?|undefined}
     */
    ActionsMenuAction.prototype.isActive;
    /**
     * Whether the action should show as disabled.
     * @type {?|undefined}
     */
    ActionsMenuAction.prototype.isDisabled;
    /**
     * Action to execute when the action is pressed.
     * \@param value Value of the selected option, when the action has options.
     * @type {?|undefined}
     */
    ActionsMenuAction.prototype.callback;
    /**
     * Use the option as an external link.
     * @type {?|undefined}
     */
    ActionsMenuAction.prototype.href;
    /**
     * Options of the action (when defined, the action's button will become the
     * toggle for a dropdown menu containing the options as items).
     * @type {?|undefined}
     */
    ActionsMenuAction.prototype.options;
    /**
     * Whether the dropdown menu should be aligned to the right of the dropdown
     * toggle. Relevant only when `options` is declared.
     * @type {?|undefined}
     */
    ActionsMenuAction.prototype.alignRight;
}
/**
 * Option of an action of an actions-menu.
 * @record
 */
function ActionsMenuActionOption() { }
if (false) {
    /**
     * Identifier of the option (used to identify the option in the i18n file).
     * @type {?|undefined}
     */
    ActionsMenuActionOption.prototype.id;
    /**
     * Text of the option (which may also be specified via the i18n file).
     * @type {?|undefined}
     */
    ActionsMenuActionOption.prototype.text;
    /**
     * Class name to add to the option.
     * @type {?|undefined}
     */
    ActionsMenuActionOption.prototype.class;
    /**
     * Value of the option (passed to the callback of the action when the option
     * is selected).
     * @type {?}
     */
    ActionsMenuActionOption.prototype.value;
    /**
     * Whether to display the option as active.
     * @type {?|undefined}
     */
    ActionsMenuActionOption.prototype.isActive;
    /**
     * Whether to display the option as disabled.
     * @type {?|undefined}
     */
    ActionsMenuActionOption.prototype.isDisabled;
    /**
     * Action to execute when the option is pressed. Will be executed before the
     * action's callback, when defined.
     * @type {?|undefined}
     */
    ActionsMenuActionOption.prototype.callback;
    /**
     * Use the option as an external link.
     * @type {?|undefined}
     */
    ActionsMenuActionOption.prototype.href;
}
/**
 * Component that responsively displays a menu of actions.
 */
class ActionsMenuComponent {
    /**
     * @param {?} lfI18n
     */
    constructor(lfI18n) {
        this.lfI18n = lfI18n;
        /**
         * Screen size at which the actions-menu should collapse.
         */
        this.collapseSize = 'md';
        /**
         * Screen size at which the actions-menu button should collapse.
         */
        this.actionsLabelCollapseSize = 'sm';
        /**
         * Whether the actions-menu (when collapsed) should align its menu to the
         * right.
         */
        this.alignMenusRight = false;
        extendObservable(this, { actions: [] }, { actions: observable.struct });
    }
    // I18n ======================================================================
    /**
     * @return {?}
     */
    get actionsLabel() {
        return this.lfI18n.get(I18N_ACTIONS_MENU_BUTTON_TEXT_KEY);
    }
    /**
     * @param {?} act
     * @return {?}
     */
    actionText(act) {
        return act.text != null
            ? act.text
            : act.id != null
                ? this.lfI18n.get(`${I18N_ACTIONS_MENU_KEY}.${act.id}`)
                : '';
    }
    /**
     * @param {?} act
     * @param {?} option
     * @return {?}
     */
    actionOptionText(act, option) {
        return option.text != null
            ? option.text
            : act.id != null && option.id != null
                ? this.lfI18n.get(`${I18N_ACTIONS_MENU_OPTIONS_KEY}.${act.id}.${option.id}`)
                : '';
    }
}
ActionsMenuComponent.decorators = [
    { type: Component, args: [{
                selector: 'lf-actions-menu',
                changeDetection: ChangeDetectionStrategy.OnPush,
                template: "<ng-container *mobxAutorun>\n  <!-- Large screen: inline buttons -->\n  <div\n    *ngIf=\"actions && actions.length > 0 && collapseSize !== 'every'\"\n    class=\"btn-group {{\n      collapseSize === 'none' ? '' : 'd-none d-' + collapseSize + '-block'\n    }}\"\n  >\n    <ng-template ngFor let-action [ngForOf]=\"actions\">\n      <!-- Anchor action -->\n      <a\n        lf-button\n        *ngIf=\"action.href\"\n        [class]=\"action.class || ''\"\n        [style]=\"action.style\"\n        [size]=\"buttonsSize\"\n        [href]=\"action.href\"\n      >\n        <lf-icon *ngIf=\"action.icon\" [icon]=\"action.icon\"></lf-icon>\n        {{ actionText(action) }}\n      </a>\n      <!-- Regular action (button) -->\n      <button\n        lf-button\n        *ngIf=\"!action.options && !action.href\"\n        [class]=\"action.class || ''\"\n        [style]=\"action.style\"\n        [size]=\"buttonsSize\"\n        [isActive]=\"action.isActive && action.isActive()\"\n        [isDisabled]=\"action.isDisabled && action.isDisabled()\"\n        (click)=\"action.callback && action.callback()\"\n      >\n        <lf-icon *ngIf=\"action.icon\" [icon]=\"action.icon\"></lf-icon>\n        {{ actionText(action) }}\n      </button>\n      <!-- Action with options (dropdown) -->\n      <lf-dropdown *ngIf=\"action.options\">\n        <button\n          lf-button\n          [class]=\"action.class || ''\"\n          [style]=\"action.style\"\n          [size]=\"buttonsSize\"\n          [isDisabled]=\"action.isDisabled && action.isDisabled()\"\n          [dropdownToggle]=\"true\"\n        >\n          <lf-icon *ngIf=\"action.icon\" [icon]=\"action.icon\"></lf-icon>\n          {{ actionText(action) }}\n        </button>\n        <lf-dropdown-menu\n          [alignRight]=\"\n            action.alignRight == null ? alignMenusRight : action.alignRight\n          \"\n        >\n          <ng-container *ngFor=\"let option of action.options\">\n            <!-- Anchor option -->\n            <a\n              lf-dropdown-item\n              *ngIf=\"option.href\"\n              [class]=\"option.class || ''\"\n              [href]=\"option.href\"\n            >\n              <lf-icon\n                *ngIf=\"option.icon\"\n                [icon]=\"option.icon\"\n                [fixedWidth]=\"true\"\n              >\n              </lf-icon>\n              {{ actionOptionText(action, option) }}\n            </a>\n            <!-- Regular option (button) -->\n            <button\n              lf-dropdown-item\n              *ngIf=\"!option.href\"\n              type=\"button\"\n              [class]=\"option.class || ''\"\n              [isActive]=\"option.isActive && option.isActive()\"\n              [isDisabled]=\"option.isDisabled && option.isDisabled()\"\n              (click)=\"\n                option.callback && option.callback();\n                action.callback && action.callback(option.value)\n              \"\n            >\n              <lf-icon\n                *ngIf=\"option.icon\"\n                [icon]=\"option.icon\"\n                [fixedWidth]=\"true\"\n              >\n              </lf-icon>\n              {{ actionOptionText(action, option) }}\n            </button>\n          </ng-container>\n        </lf-dropdown-menu>\n      </lf-dropdown>\n    </ng-template>\n  </div>\n\n  <!-- Small screen: menu buttons -->\n  <div\n    *ngIf=\"actions && actions.length > 0 && collapseSize !== 'none'\"\n    [class]=\"\n      collapseSize === 'every' ? '' : 'd-' + collapseSize + '-none d-block'\n    \"\n  >\n    <lf-dropdown>\n      <button\n        lf-button\n        [style]=\"'outline-secondary'\"\n        [size]=\"buttonsSize\"\n        [dropdownToggle]=\"true\"\n      >\n        <lf-icon icon=\"gear\"></lf-icon>\n        <!-- Show a label for the actions toggle depending on screen size -->\n        <span\n          *ngIf=\"actionsLabelCollapseSize !== 'every'\"\n          [class]=\"\n            actionsLabelCollapseSize === 'none'\n              ? ''\n              : 'd-none d-' + actionsLabelCollapseSize + '-inline'\n          \"\n        >\n          {{ actionsLabel }}\n        </span>\n      </button>\n      <lf-dropdown-menu [alignRight]=\"alignMenusRight\">\n        <ng-container *ngFor=\"let action of actions; let i = index\">\n          <!-- Anchor action -->\n          <a\n            lf-dropdown-item\n            *ngIf=\"action.href\"\n            [class]=\"action.class || ''\"\n            [href]=\"action.href\"\n          >\n            <lf-icon\n              *ngIf=\"action.icon\"\n              [icon]=\"action.icon\"\n              [fixedWidth]=\"true\"\n            >\n            </lf-icon>\n            {{ actionText(action) }}\n          </a>\n          <!-- Regular action (button) -->\n          <button\n            lf-dropdown-item\n            type=\"button\"\n            *ngIf=\"!action.options && !action.href\"\n            [class]=\"action.class || ''\"\n            [isActive]=\"action.isActive && action.isActive()\"\n            [isDisabled]=\"action.isDisabled && action.isDisabled()\"\n            (click)=\"action.callback && action.callback()\"\n          >\n            <lf-icon\n              *ngIf=\"action.icon\"\n              [icon]=\"action.icon\"\n              [fixedWidth]=\"true\"\n            >\n            </lf-icon>\n            {{ actionText(action) }}\n          </button>\n          <!-- Action with options -->\n          <ng-container *ngIf=\"action.options\">\n            <!-- Surround actions with options with dividers -->\n            <div lf-dropdown-item *ngIf=\"i > 0\" [itemType]=\"'divider'\"></div>\n            <h6\n              lf-dropdown-item\n              [class]=\"action.class || ''\"\n              [itemType]=\"'header'\"\n            >\n              <lf-icon\n                *ngIf=\"action.icon\"\n                [icon]=\"action.icon\"\n                [fixedWidth]=\"true\"\n              >\n              </lf-icon>\n              {{ actionText(action) }}\n            </h6>\n            <ng-container *ngFor=\"let option of action.options\">\n              <!-- Anchor option -->\n              <a\n                lf-dropdown-item\n                *ngIf=\"option.href\"\n                [class]=\"option.class || ''\"\n                [href]=\"option.href\"\n              >\n                {{ actionOptionText(action, option) }}\n              </a>\n              <!-- Regular option (button) -->\n              <button\n                lf-dropdown-item\n                type=\"button\"\n                *ngIf=\"!option.href\"\n                [class]=\"option.class || ''\"\n                [isActive]=\"option.isActive && option.isActive()\"\n                [isDisabled]=\"option.isDisabled && option.isDisabled()\"\n                (click)=\"\n                  option.callback && option.callback();\n                  action.callback && action.callback(option.value)\n                \"\n              >\n                {{ actionOptionText(action, option) }}\n              </button>\n            </ng-container>\n            <div\n              lf-dropdown-item\n              *ngIf=\"i < actions.length - 1 && !actions[i + 1].options\"\n              [itemType]=\"'divider'\"\n            ></div>\n          </ng-container>\n        </ng-container>\n      </lf-dropdown-menu>\n    </lf-dropdown>\n  </div>\n</ng-container>\n"
            }] }
];
/** @nocollapse */
ActionsMenuComponent.ctorParameters = () => [
    { type: LfI18n, decorators: [{ type: Inject, args: [LfI18n,] }] }
];
ActionsMenuComponent.propDecorators = {
    actions: [{ type: Input }],
    buttonsSize: [{ type: Input }],
    collapseSize: [{ type: Input }],
    actionsLabelCollapseSize: [{ type: Input }],
    alignMenusRight: [{ type: Input }]
};
__decorate([
    observable$1,
    __metadata("design:type", String)
], ActionsMenuComponent.prototype, "buttonsSize", void 0);
__decorate([
    observable$1,
    __metadata("design:type", String)
], ActionsMenuComponent.prototype, "collapseSize", void 0);
__decorate([
    observable$1,
    __metadata("design:type", String)
], ActionsMenuComponent.prototype, "actionsLabelCollapseSize", void 0);
__decorate([
    observable$1,
    __metadata("design:type", Object)
], ActionsMenuComponent.prototype, "alignMenusRight", void 0);
__decorate([
    computed,
    __metadata("design:type", String),
    __metadata("design:paramtypes", [])
], ActionsMenuComponent.prototype, "actionsLabel", null);
if (false) {
    /**
     * Actions of the actions-menu.
     * @type {?}
     */
    ActionsMenuComponent.prototype.actions;
    /**
     * Size of the buttons of the actions-menu.
     * @type {?}
     */
    ActionsMenuComponent.prototype.buttonsSize;
    /**
     * Screen size at which the actions-menu should collapse.
     * @type {?}
     */
    ActionsMenuComponent.prototype.collapseSize;
    /**
     * Screen size at which the actions-menu button should collapse.
     * @type {?}
     */
    ActionsMenuComponent.prototype.actionsLabelCollapseSize;
    /**
     * Whether the actions-menu (when collapsed) should align its menu to the
     * right.
     * @type {?}
     */
    ActionsMenuComponent.prototype.alignMenusRight;
    /**
     * @type {?}
     * @protected
     */
    ActionsMenuComponent.prototype.lfI18n;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Abstract class for components that are toggleable and path based.
 * @abstract
 */
class Toggleable extends PathBased {
    constructor() {
        super(...arguments);
        /**
         * Whether the toggleable element should start visible.
         */
        this.startVisible = false;
        /**
         * Event emitter which emits an event whenever the toggleable is set to be
         * shown.
         */
        // tslint:disable-next-line:no-output-rename
        this._show = new EventEmitter();
        /**
         * Event emitter which emits an event whenever the toggleable is set to be
         * hidden.
         */
        // tslint:disable-next-line:no-output-rename
        this._hide = new EventEmitter();
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this._isVisible = this.startVisible;
    }
    /**
     * Whether the toggleable is visible.
     * @return {?}
     */
    get isVisible() {
        return this._isVisible;
    }
    /**
     * @param {?} setVisible
     * @return {?}
     */
    set isVisible(setVisible) {
        if (!this._isVisible && setVisible) {
            this._isVisible = true;
            this._show.emit();
        }
        else if (this._isVisible && !setVisible) {
            this._isVisible = false;
            this._hide.emit();
        }
    }
    /**
     * Show the toggleable element.
     * @return {?}
     */
    show() {
        this.isVisible = true;
    }
    /**
     * Hide the toggleable element.
     * @return {?}
     */
    hide() {
        this.isVisible = false;
    }
    /**
     * Toggle the visibility of the toggleable element.
     * @return {?}
     */
    toggle() {
        this.isVisible = !this.isVisible;
    }
}
Toggleable.propDecorators = {
    startVisible: [{ type: Input }],
    _show: [{ type: Output, args: ['show',] }],
    _hide: [{ type: Output, args: ['hide',] }]
};
__decorate([
    observable$1,
    __metadata("design:type", Boolean)
], Toggleable.prototype, "_isVisible", void 0);
__decorate([
    computed,
    __metadata("design:type", Boolean),
    __metadata("design:paramtypes", [Boolean])
], Toggleable.prototype, "isVisible", null);
__decorate([
    action,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], Toggleable.prototype, "show", null);
__decorate([
    action,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], Toggleable.prototype, "hide", null);
__decorate([
    action,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], Toggleable.prototype, "toggle", null);
if (false) {
    /**
     * Whether the toggleable element should start visible.
     * @type {?}
     */
    Toggleable.prototype.startVisible;
    /**
     * Event emitter which emits an event whenever the toggleable is set to be
     * shown.
     * @type {?}
     */
    Toggleable.prototype._show;
    /**
     * Event emitter which emits an event whenever the toggleable is set to be
     * hidden.
     * @type {?}
     */
    Toggleable.prototype._hide;
    /**
     * @type {?}
     * @protected
     */
    Toggleable.prototype._isVisible;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * LF alert component.
 */
class AlertComponent extends Toggleable {
    // I18n ======================================================================
    /**
     * @return {?}
     */
    get closeButtonTooltip() {
        return this.translate(I18N_CLOSE_BUTTON_TOOLTIP_KEY);
    }
}
AlertComponent.decorators = [
    { type: Component, args: [{
                selector: 'lf-alert',
                changeDetection: ChangeDetectionStrategy.OnPush,
                template: "<ng-container *mobxAutorun>\n  <div\n    class=\"alert {{ style ? 'alert-' + style : '' }}\"\n    role=\"alert\"\n    *ngIf=\"isVisible\"\n  >\n    <button\n      type=\"button\"\n      class=\"close\"\n      [title]=\"closeButtonTooltip\"\n      [attr.aria-label]=\"closeButtonTooltip\"\n      *ngIf=\"showCloseButton\"\n      (click)=\"hide()\"\n    >\n      <span aria-hidden=\"true\">&times;</span>\n    </button>\n    <ng-content></ng-content>\n  </div>\n</ng-container>\n"
            }] }
];
AlertComponent.propDecorators = {
    style: [{ type: Input }],
    showCloseButton: [{ type: Input }]
};
__decorate([
    observable$1,
    __metadata("design:type", String)
], AlertComponent.prototype, "style", void 0);
__decorate([
    observable$1,
    __metadata("design:type", Boolean)
], AlertComponent.prototype, "showCloseButton", void 0);
__decorate([
    computed,
    __metadata("design:type", String),
    __metadata("design:paramtypes", [])
], AlertComponent.prototype, "closeButtonTooltip", null);
if (false) {
    /** @type {?} */
    AlertComponent.prototype.style;
    /** @type {?} */
    AlertComponent.prototype.showCloseButton;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Abstract class that all components which "encapsulate" a storage value should
 * inherit.
 * @abstract
 */
class Value extends PathBased {
    constructor() {
        super(...arguments);
        /**
         * Whether the component should render in "read-only" mode (this has no effect
         * if this component is within an already "read-only" component).
         */
        // tslint:disable-next-line:no-input-rename
        this._isReadOnly = false;
        /**
         * Add an `lf-value` class to all value components.
         */
        this._classLfValue = true;
    }
    // I18n ======================================================================
    /**
     * Label associated with this value.
     * @return {?}
     */
    get label() {
        return this.translate(I18N_VALUE_LABEL_KEY);
    }
    /**
     * Code associated with this value.
     * @return {?}
     */
    get code() {
        return this.translate(I18N_VALUE_CODE_KEY);
    }
    /**
     * Help message associated with this value.
     * @return {?}
     */
    get helpMessage() {
        return this.translate(I18N_VALUE_HELP_MESSAGE_KEY);
    }
    /**
     * Legend associated with this value.
     * @return {?}
     */
    get legend() {
        return this.translate(I18N_VALUE_LEGEND_KEY);
    }
    /**
     * Message used to ask the user to retry validating the value.
     * @return {?}
     */
    get retryValidateText() {
        return this.translate(I18N_VALUE_RETRY_VALIDATE_TEXT_KEY);
    }
    // ===========================================================================
    /**
     * Whether the component should render in "read-only" mode. Inherits the
     * "read-only" state of parent components (meaning that it's not possible to
     * have an "editable" component within a "read-only" one). Furthermore, if the
     * component's value is computed, the component will always be in "read-only"
     * mode.
     * @return {?}
     */
    get isReadOnly() {
        return ((this.parentPathBasedComponent !== null &&
            ((/** @type {?} */ (this.parentPathBasedComponent))).isReadOnly) ||
            this.relativeStorage.isComputed() ||
            this._isReadOnly);
    }
}
Value.propDecorators = {
    _isReadOnly: [{ type: Input, args: ['isReadOnly',] }],
    _classLfValue: [{ type: HostBinding, args: ['class.lf-value',] }]
};
__decorate([
    observable$1,
    __metadata("design:type", Object)
], Value.prototype, "_isReadOnly", void 0);
__decorate([
    computed,
    __metadata("design:type", String),
    __metadata("design:paramtypes", [])
], Value.prototype, "label", null);
__decorate([
    computed,
    __metadata("design:type", String),
    __metadata("design:paramtypes", [])
], Value.prototype, "code", null);
__decorate([
    computed,
    __metadata("design:type", String),
    __metadata("design:paramtypes", [])
], Value.prototype, "helpMessage", null);
__decorate([
    computed,
    __metadata("design:type", String),
    __metadata("design:paramtypes", [])
], Value.prototype, "legend", null);
__decorate([
    computed,
    __metadata("design:type", String),
    __metadata("design:paramtypes", [])
], Value.prototype, "retryValidateText", null);
__decorate([
    computed,
    __metadata("design:type", Boolean),
    __metadata("design:paramtypes", [])
], Value.prototype, "isReadOnly", null);
if (false) {
    /**
     * Whether the component should render in "read-only" mode (this has no effect
     * if this component is within an already "read-only" component).
     * @type {?}
     */
    Value.prototype._isReadOnly;
    /**
     * Add an `lf-value` class to all value components.
     * @type {?}
     */
    Value.prototype._classLfValue;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Validation panel component.
 */
class ValidationPanelComponent extends Toggleable {
    // Angular lifecycle =========================================================
    /**
     * @return {?}
     */
    ngOnInit() {
        this.lfRouter = this.injector.get(LfRouter);
        super.ngOnInit();
    }
    // I18n ======================================================================
    /**
     * @return {?}
     */
    get errorsText() {
        return this.translate(I18N_VALIDATION_PANEL_ERRORS_TEXT_KEY);
    }
    /**
     * @return {?}
     */
    get warningsText() {
        return this.translate(I18N_VALIDATION_PANEL_WARNINGS_TEXT_KEY);
    }
    /**
     * @return {?}
     */
    get noIssuesText() {
        return this.translate(I18N_VALIDATION_PANEL_NO_ISSUES_TEXT_KEY);
    }
    /**
     * @return {?}
     */
    get focusButtonTooltip() {
        return this.translate(I18N_VALIDATION_PANEL_FOCUS_TEXT_KEY);
    }
    /**
     * @return {?}
     */
    get nextButtonTooltip() {
        return this.translate(I18N_VALIDATION_PANEL_NEXT_BUTTON_TOOLTIP_KEY);
    }
    /**
     * @return {?}
     */
    get prevButtonTooltip() {
        return this.translate(I18N_VALIDATION_PANEL_PREV_BUTTON_TOOLTIP_KEY);
    }
    /**
     * @return {?}
     */
    get closeButtonTooltip() {
        return this.translate(I18N_CLOSE_BUTTON_TOOLTIP_KEY);
    }
    // ===========================================================================
    /**
     * @return {?}
     */
    show() {
        super.show();
        this.navigateToCurrentPath();
    }
    /**
     * @return {?}
     */
    get isPending() {
        return this.relativeStorage.validationStatus() === PENDING;
    }
    /**
     * @return {?}
     */
    get currentPath() {
        return this._currentPath && this._validationIssues[this._currentPath]
            ? this._currentPath
            : this._pages[0];
    }
    /**
     * @return {?}
     */
    get currentLabel() {
        return this.currentPath && this.labelOf(this.currentPath);
    }
    /**
     * @return {?}
     */
    get hasNext() {
        return this._pages.length - 1 > this._pages.indexOf(this.currentPath);
    }
    /**
     * @return {?}
     */
    triggerFocus() {
        this.navigateToCurrentPath();
    }
    /**
     * @return {?}
     */
    next() {
        if (this.hasNext) {
            this._currentPath = this._pages[this._pages.indexOf(this.currentPath) + 1];
            this.navigateToCurrentPath();
        }
    }
    /**
     * @return {?}
     */
    get hasPrevious() {
        return this._pages.indexOf(this.currentPath) > 0;
    }
    /**
     * @return {?}
     */
    previous() {
        if (this.hasPrevious) {
            this._currentPath = this._pages[this._pages.indexOf(this.currentPath) - 1];
            this.navigateToCurrentPath();
        }
    }
    /**
     * @return {?}
     */
    get errors() {
        return ((this._validationIssues[this.currentPath] || []).filter((/**
         * @param {?} issue
         * @return {?}
         */
        issue => !issue.isWarning)) || []);
    }
    /**
     * @return {?}
     */
    get warnings() {
        return ((this._validationIssues[this.currentPath] || []).filter((/**
         * @param {?} issue
         * @return {?}
         */
        issue => issue.isWarning)) || []);
    }
    /**
     * @return {?}
     */
    get hasNoIssues() {
        return this._pages.length === 0;
    }
    /**
     * @return {?}
     */
    get panelType() {
        return this.isPending
            ? 'secondary'
            : this.errors.length > 0
                ? 'danger'
                : this.warnings.length > 0
                    ? 'warning'
                    : 'success';
    }
    /**
     * @private
     * @return {?}
     */
    get _pages() {
        return Object.keys(this._validationIssues);
    }
    /**
     * @private
     * @return {?}
     */
    get _validationIssues() {
        return this.relativeStorage.validationIssues() || {};
    }
    /**
     * Label associated with the given path.
     * @private
     * @param {?} path Path from which to fetch label.
     * @return {?} Label of given path.
     */
    labelOf(path) {
        return this.lfI18n.getFromPath(path, I18N_VALUE_LABEL_KEY) || path;
    }
    /**
     * @private
     * @return {?}
     */
    navigateToCurrentPath() {
        if (this.currentPath) {
            // Check if the first route that matches the path is navigateable, if not
            // don't navigate
            /** @type {?} */
            const firstMatching = this.lfRouter.matchingRoutes(this.currentPath, {
                navigateToParent: true,
            })[0];
            /** @type {?} */
            const navigateable = firstMatching &&
                this.lfRouter.navigateableRouteFromMatchingRoutes([firstMatching]);
            if (navigateable) {
                this.lfRouter.navigateFromNavigateableRoute(navigateable, {
                    queryParamsHandling: 'merge',
                });
            }
            else {
                // Remove fragment TODO: Find a better way of doing it
                this.lfRouter.navigate('.', { queryParamsHandling: 'merge' });
            }
        }
    }
}
ValidationPanelComponent.decorators = [
    { type: Component, args: [{
                selector: 'lf-validation-panel',
                changeDetection: ChangeDetectionStrategy.OnPush,
                template: "<ng-container *mobxAutorun>\n  <div class=\"validation-panel-container\" *ngIf=\"isVisible\">\n    <div class=\"card border-{{ panelType }}\">\n      <div\n        class=\"card-header bg-{{ panelType }} text-white d-inline-flex\n               justify-content-between\"\n      >\n        <span class=\"header-title\">{{ currentLabel }}</span>\n        <div class=\"actions-bar d-inline-flex justify-content-between\">\n          <button\n            lf-button\n            class=\"panel-focus\"\n            size=\"sm\"\n            [title]=\"focusButtonTooltip\"\n            [attr.aria-label]=\"focusButtonTooltip\"\n            [style]=\"'outline-light'\"\n            (click)=\"triggerFocus()\"\n          >\n            <lf-icon icon=\"crosshairs\"></lf-icon>\n          </button>\n          <div class=\"btn-group mx-4\">\n            <button\n              lf-button\n              class=\"panel-prev\"\n              size=\"sm\"\n              [title]=\"prevButtonTooltip\"\n              [attr.aria-label]=\"prevButtonTooltip\"\n              [style]=\"'outline-light'\"\n              [isDisabled]=\"!hasPrevious\"\n              (click)=\"previous()\"\n            >\n              <lf-icon icon=\"chevron-left\"></lf-icon>\n            </button>\n            <button\n              lf-button\n              class=\"panel-next\"\n              size=\"sm\"\n              [title]=\"nextButtonTooltip\"\n              [attr.aria-label]=\"nextButtonTooltip\"\n              [style]=\"'outline-light'\"\n              [isDisabled]=\"!hasNext\"\n              (click)=\"next()\"\n            >\n              <lf-icon icon=\"chevron-right\"></lf-icon>\n            </button>\n          </div>\n          <button\n            lf-button\n            class=\"panel-close\"\n            size=\"sm\"\n            [title]=\"closeButtonTooltip\"\n            [attr.aria-label]=\"closeButtonTooltip\"\n            [style]=\"'outline-light'\"\n            (click)=\"hide()\"\n          >\n            <lf-icon icon=\"times\"></lf-icon>\n          </button>\n        </div>\n      </div>\n      <div class=\"card-body bg-{{ panelType }}\">\n        <div *ngIf=\"isPending\" class=\"pending\"></div>\n        <ng-container *ngIf=\"!isPending && errors.length > 0\">\n          <strong>\n            <lf-icon icon=\"exclamation-circle\"></lf-icon> {{ errorsText }}\n          </strong>\n          <ul>\n            <li\n              *ngFor=\"let error of errors\"\n              class=\"validation-issue\"\n              lfValidationIssue\n              [path]=\"currentPath\"\n              [issue]=\"error\"\n            ></li>\n          </ul>\n        </ng-container>\n        <ng-container *ngIf=\"!isPending && warnings.length > 0\">\n          <strong>\n            <lf-icon icon=\"exclamation-triangle\"></lf-icon> {{ warningsText }}\n          </strong>\n          <ul>\n            <li\n              *ngFor=\"let warning of warnings\"\n              class=\"validation-issue\"\n              lfValidationIssue\n              [path]=\"currentPath\"\n              [issue]=\"warning\"\n            ></li>\n          </ul>\n        </ng-container>\n        <ng-container *ngIf=\"!isPending && hasNoIssues\">\n          {{ noIssuesText }}\n        </ng-container>\n      </div>\n    </div>\n  </div>\n</ng-container>\n"
            }] }
];
__decorate([
    observable$1,
    __metadata("design:type", String)
], ValidationPanelComponent.prototype, "_currentPath", void 0);
__decorate([
    computed,
    __metadata("design:type", String),
    __metadata("design:paramtypes", [])
], ValidationPanelComponent.prototype, "errorsText", null);
__decorate([
    computed,
    __metadata("design:type", String),
    __metadata("design:paramtypes", [])
], ValidationPanelComponent.prototype, "warningsText", null);
__decorate([
    computed,
    __metadata("design:type", String),
    __metadata("design:paramtypes", [])
], ValidationPanelComponent.prototype, "noIssuesText", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], ValidationPanelComponent.prototype, "focusButtonTooltip", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], ValidationPanelComponent.prototype, "nextButtonTooltip", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], ValidationPanelComponent.prototype, "prevButtonTooltip", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], ValidationPanelComponent.prototype, "closeButtonTooltip", null);
__decorate([
    action,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ValidationPanelComponent.prototype, "show", null);
__decorate([
    computed,
    __metadata("design:type", Boolean),
    __metadata("design:paramtypes", [])
], ValidationPanelComponent.prototype, "isPending", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], ValidationPanelComponent.prototype, "currentPath", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], ValidationPanelComponent.prototype, "currentLabel", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], ValidationPanelComponent.prototype, "hasNext", null);
__decorate([
    action,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ValidationPanelComponent.prototype, "triggerFocus", null);
__decorate([
    action,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ValidationPanelComponent.prototype, "next", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], ValidationPanelComponent.prototype, "hasPrevious", null);
__decorate([
    action,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ValidationPanelComponent.prototype, "previous", null);
__decorate([
    computed,
    __metadata("design:type", Array),
    __metadata("design:paramtypes", [])
], ValidationPanelComponent.prototype, "errors", null);
__decorate([
    computed,
    __metadata("design:type", Array),
    __metadata("design:paramtypes", [])
], ValidationPanelComponent.prototype, "warnings", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], ValidationPanelComponent.prototype, "hasNoIssues", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], ValidationPanelComponent.prototype, "panelType", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], ValidationPanelComponent.prototype, "_pages", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], ValidationPanelComponent.prototype, "_validationIssues", null);
if (false) {
    /**
     * Path currently being viewed.
     * @type {?}
     * @private
     */
    ValidationPanelComponent.prototype._currentPath;
    /**
     * @type {?}
     * @private
     */
    ValidationPanelComponent.prototype.lfRouter;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Lf app component.
 */
class AppComponent extends Value {
    constructor() {
        super(...arguments);
        // State of the tree nav visibility
        this._isTreeNavVisible = false;
        // Whether the backdrop element should be in visible or invisible state
        this._isBackdropVisible = false;
        // Whether the backdrop element should exist
        this._shouldBackdropExist = false;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.lfRouter = this.injector.get(LfRouter, null);
        if (this._path === undefined) {
            this._path = '/';
        }
        super.ngOnInit();
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        // Hide tree nav when `Escape` is pressed
        this.onPressEsc = (/**
         * @param {?} evt
         * @return {?}
         */
        evt => {
            if (this.isTreeNavVisible && evt.key === 'Escape') {
                this.hideTreeNav();
            }
        });
        window.addEventListener('keyup', this.onPressEsc);
        // Scroll viewport to the top whenever the router path changes
        if (this.lfRouter !== null) {
            this.disposeOnRouterPathChange = observe(this.lfRouter, 'path', (/**
             * @return {?}
             */
            () => {
                this.scrollTo({ top: 0, left: 0 });
            }));
        }
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.onPressEsc && window.removeEventListener('keyup', this.onPressEsc);
        this.disposeOnRouterPathChange && this.disposeOnRouterPathChange();
    }
    /**
     * @return {?}
     */
    get isTreeNavVisible() {
        return this._isTreeNavVisible;
    }
    /**
     * @return {?}
     */
    showTreeNav() {
        if (!this._isTreeNavVisible) {
            this._isTreeNavVisible = true;
            this._isBackdropVisible = true;
            this._shouldBackdropExist = true;
            this.lastTreeNavTranslateX = this.treeNavTranslateX;
        }
    }
    /**
     * @return {?}
     */
    hideTreeNav() {
        if (this._isTreeNavVisible) {
            this._isTreeNavVisible = false;
            // When there was no time for an animation and `transitionend` isn't called
            if (this.lastTreeNavTranslateX === this.treeNavTranslateX) {
                this._onTreeNavTransitionEnd();
            }
        }
    }
    /**
     * @return {?}
     */
    validate() {
        this.relativeStorage.setTouched('.', true);
        this._validationPanel.show();
    }
    /**
     * Scroll the main content area.
     * @param {?} options Options for the scrolling.
     * @return {?}
     */
    scrollTo(options) {
        // Depending on whether we're in mobile or desktop mode, the scrollable
        // element differs
        this._mainContainer.nativeElement.scrollTo(options);
        this._mainContent.nativeElement.scrollTo(options);
    }
    // Action to run after the tree nav finishes hiding (hide backdrop)
    /**
     * @return {?}
     */
    _onTreeNavTransitionEnd() {
        if (!this.isTreeNavVisible && this._isBackdropVisible) {
            runInAction('hideTreeNavBackdrop', (/**
             * @return {?}
             */
            () => (this._isBackdropVisible = false)));
        }
    }
    // Action to run after the backdrop finishes hiding (delete backdrop)
    /**
     * @return {?}
     */
    _onBackdropTransitionEnd() {
        runInAction('removeTreeNavBackdrop', (/**
         * @return {?}
         */
        () => (this._shouldBackdropExist = false)));
    }
    /**
     * @protected
     * @return {?}
     */
    validatePath() {
        /** @type {?} */
        const schema = this.relativeStorage.schema();
        if (!isRecordSchema(schema) || isNullableSchema(schema)) {
            throw pathError(this.path, 'Schema must be of the "record" type and not "nullable"');
        }
    }
    // Current `translateX` value of the tree nav
    /**
     * @private
     * @return {?}
     */
    get treeNavTranslateX() {
        // Obtain `tx` from `'matrix(a, b, c, d, tx, ty)'`
        return +window
            .getComputedStyle(this._treeNavContainer.nativeElement)
            .getPropertyValue('transform')
            .split(',')[4];
    }
}
AppComponent.decorators = [
    { type: Component, args: [{
                selector: 'lf-app',
                changeDetection: ChangeDetectionStrategy.OnPush,
                providers: [{ provide: PathBased, useExisting: AppComponent }],
                template: "<ng-container *mobxAutorun>\n  <div class=\"d-flex flex-column h-100\">\n    <!-- Application header -->\n    <header id=\"lf-header\">\n      <ng-content select=\"lf-top-bar\"></ng-content>\n    </header>\n\n    <!-- Content -->\n    <div #mainContainer id=\"lf-main-container\" class=\"container-fluid\">\n      <div id=\"lf-main-row\" class=\"row\">\n        <!-- Container of the tree nav (collapsible from the `top-bar`) -->\n        <div\n          #treeNavContainer\n          id=\"lf-tree-nav-container\"\n          class=\"col-xl-2 col-md-3 col-sm-12\"\n          [class.show]=\"isTreeNavVisible\"\n          [class.animate]=\"_shouldBackdropExist\"\n          (transitionend)=\"_onTreeNavTransitionEnd()\"\n        >\n          <ng-content select=\"lf-tree-nav\"></ng-content>\n        </div>\n\n        <!-- Main content -->\n        <div\n          #mainContent\n          id=\"lf-main-content\"\n          class=\"col-xl-10 col-md-9 col-sm-12\"\n        >\n          <ng-content></ng-content>\n\n          <!-- Validation panel -->\n          <lf-validation-panel></lf-validation-panel>\n        </div>\n      </div>\n    </div>\n  </div>\n\n  <div\n    *ngIf=\"_shouldBackdropExist\"\n    class=\"modal-backdrop fade d-md-none\"\n    [class.show]=\"_isBackdropVisible\"\n    (click)=\"hideTreeNav()\"\n    (transitionend)=\"_onBackdropTransitionEnd()\"\n  ></div>\n</ng-container>\n"
            }] }
];
AppComponent.propDecorators = {
    _validationPanel: [{ type: ViewChild, args: [ValidationPanelComponent, { static: false },] }],
    _treeNavContainer: [{ type: ViewChild, args: ['treeNavContainer', { static: false },] }],
    _mainContainer: [{ type: ViewChild, args: ['mainContainer', { static: false },] }],
    _mainContent: [{ type: ViewChild, args: ['mainContent', { static: false },] }]
};
__decorate([
    observable$1,
    __metadata("design:type", Object)
], AppComponent.prototype, "_isTreeNavVisible", void 0);
__decorate([
    observable$1,
    __metadata("design:type", Object)
], AppComponent.prototype, "_isBackdropVisible", void 0);
__decorate([
    observable$1,
    __metadata("design:type", Object)
], AppComponent.prototype, "_shouldBackdropExist", void 0);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], AppComponent.prototype, "isTreeNavVisible", null);
__decorate([
    action,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], AppComponent.prototype, "showTreeNav", null);
__decorate([
    action,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], AppComponent.prototype, "hideTreeNav", null);
__decorate([
    action,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], AppComponent.prototype, "validate", null);
if (false) {
    /** @type {?} */
    AppComponent.prototype._validationPanel;
    /** @type {?} */
    AppComponent.prototype._treeNavContainer;
    /** @type {?} */
    AppComponent.prototype._mainContainer;
    /** @type {?} */
    AppComponent.prototype._mainContent;
    /** @type {?} */
    AppComponent.prototype._isTreeNavVisible;
    /** @type {?} */
    AppComponent.prototype._isBackdropVisible;
    /** @type {?} */
    AppComponent.prototype._shouldBackdropExist;
    /**
     * @type {?}
     * @private
     */
    AppComponent.prototype.lfRouter;
    /**
     * @type {?}
     * @private
     */
    AppComponent.prototype.lastTreeNavTranslateX;
    /**
     * @type {?}
     * @private
     */
    AppComponent.prototype.disposeOnRouterPathChange;
    /**
     * @type {?}
     * @private
     */
    AppComponent.prototype.onPressEsc;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * LF component used to display the "top-bar" of an application.
 */
class TopBarComponent {
    /**
     * @param {?} lfStorage
     * @param {?} lfRouter
     * @param {?} lfI18n
     * @param {?} lfFileStorage
     * @param {?} _appComponent
     */
    constructor(lfStorage, lfRouter, lfI18n, lfFileStorage, _appComponent) {
        this.lfStorage = lfStorage;
        this.lfRouter = lfRouter;
        this.lfI18n = lfI18n;
        this.lfFileStorage = lfFileStorage;
        this._appComponent = _appComponent;
        extendObservable(this, { actions: undefined }, { actions: observable.struct });
    }
    // I18n ======================================================================
    /**
     * @return {?}
     */
    get retryValidationText() {
        return this.lfI18n.get(I18N_VALUE_RETRY_VALIDATE_TEXT_KEY);
    }
    // ===========================================================================
    /**
     * @return {?}
     */
    get rootUrl() {
        try {
            return this.lfRouter.createUrl('/');
        }
        catch (err) {
            return '/';
        }
    }
    /**
     * @return {?}
     */
    get appIsPending() {
        return (this.lfStorage.shouldShowPending() ||
            (this.lfFileStorage && this.lfFileStorage.isLoading));
    }
    /**
     * @return {?}
     */
    get shouldShowAppRecompute() {
        return !!this.lfStorage.shouldShowRejected();
    }
    /**
     * @return {?}
     */
    recomputeAppStateProperties() {
        this.lfStorage.recomputeStateProperties();
    }
    /**
     * @return {?}
     */
    get isTreeNavVisible() {
        return this._appComponent.isTreeNavVisible;
    }
    /**
     * @return {?}
     */
    showTreeNav() {
        this._appComponent.showTreeNav();
    }
}
TopBarComponent.decorators = [
    { type: Component, args: [{
                selector: 'lf-top-bar',
                changeDetection: ChangeDetectionStrategy.OnPush,
                template: "<ng-container *mobxAutorun>\n  <nav class=\"navbar navbar-expand-lg justify-content-between\">\n    <!-- Tree toggle + title + pending icon -->\n    <div class=\"navbar-brand\">\n      <!-- Small screen: show tree-nav button -->\n      <button\n        lf-button\n        class=\"tree-nav-show d-md-none\"\n        [style]=\"'outline-secondary'\"\n        (click)=\"showTreeNav()\"\n      >\n        <lf-icon icon=\"bars\"></lf-icon>\n      </button>\n\n      <!-- Title -->\n      <a\n        class=\"navbar-title\"\n        [routerLink]=\"rootUrl\"\n        queryParamsHandling=\"merge\"\n      >\n        <ng-content></ng-content>\n      </a>\n\n      <!-- Pending app validation -->\n      <div class=\"pending-app\" *ngIf=\"appIsPending\"></div>\n\n      <!-- Rejected app validation -->\n      <div\n        class=\"rejected-app\"\n        *ngIf=\"shouldShowAppRecompute\"\n        (click)=\"recomputeAppStateProperties()\"\n      >\n        <lf-tooltip [help]=\"retryValidationText\" placement=\"right\">\n          <lf-icon icon=\"warning\"></lf-icon>\n        </lf-tooltip>\n      </div>\n    </div>\n\n    <!-- Application actions -->\n    <lf-actions-menu [actions]=\"actions\" [alignMenusRight]=\"true\">\n    </lf-actions-menu>\n  </nav>\n</ng-container>\n"
            }] }
];
/** @nocollapse */
TopBarComponent.ctorParameters = () => [
    { type: LfStorage },
    { type: LfRouter },
    { type: LfI18n, decorators: [{ type: Inject, args: [LfI18n,] }] },
    { type: LfFileStorage, decorators: [{ type: Optional }, { type: Inject, args: [LfFileStorage,] }] },
    { type: AppComponent, decorators: [{ type: Inject, args: [AppComponent,] }] }
];
TopBarComponent.propDecorators = {
    actions: [{ type: Input }]
};
__decorate([
    computed,
    __metadata("design:type", String),
    __metadata("design:paramtypes", [])
], TopBarComponent.prototype, "retryValidationText", null);
__decorate([
    computed,
    __metadata("design:type", String),
    __metadata("design:paramtypes", [])
], TopBarComponent.prototype, "rootUrl", null);
__decorate([
    computed,
    __metadata("design:type", Boolean),
    __metadata("design:paramtypes", [])
], TopBarComponent.prototype, "appIsPending", null);
__decorate([
    computed,
    __metadata("design:type", Boolean),
    __metadata("design:paramtypes", [])
], TopBarComponent.prototype, "shouldShowAppRecompute", null);
__decorate([
    action,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], TopBarComponent.prototype, "recomputeAppStateProperties", null);
__decorate([
    computed,
    __metadata("design:type", Boolean),
    __metadata("design:paramtypes", [])
], TopBarComponent.prototype, "isTreeNavVisible", null);
__decorate([
    action,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], TopBarComponent.prototype, "showTreeNav", null);
if (false) {
    /** @type {?} */
    TopBarComponent.prototype.actions;
    /**
     * @type {?}
     * @private
     */
    TopBarComponent.prototype.lfStorage;
    /**
     * @type {?}
     * @private
     */
    TopBarComponent.prototype.lfRouter;
    /**
     * @type {?}
     * @private
     */
    TopBarComponent.prototype.lfI18n;
    /**
     * @type {?}
     * @private
     */
    TopBarComponent.prototype.lfFileStorage;
    /**
     * @type {?}
     * @protected
     */
    TopBarComponent.prototype._appComponent;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Bootstrap button component.
 */
class ButtonComponent {
    /**
     * @param {?} _host
     */
    constructor(_host) {
        /**
         * HTML button type.
         */
        this.type = 'button';
        /**
         * Match the width of parent if `true`.
         */
        this.block = false;
        /**
         * Display a dropdown icon (this is automatic if `dropdownToggleSplit` is
         * set).
         */
        this.dropdownToggle = false;
        /**
         * Reduces the padding of the button so it can be used as the dropdown part of
         * a split dropdown.
         */
        this.dropdownToggleSplit = false;
        /**
         * Set the button as disabled.
         */
        this.isDisabled = false;
        /**
         * Set the button as active.
         */
        this.isActive = false;
        /**
         * Always add the class `btn` to LF buttons.
         */
        this._hostClassBtn = true;
        this._tagName = _host.nativeElement.tagName.toLowerCase();
    }
    // Set the role as `'button'` when the element is an `a`.
    /**
     * @return {?}
     */
    get _role() {
        return this._tagName === 'a' ? 'button' : undefined;
    }
    // Set the corresponding style class depending on the button style (setting
    // individual classes instead of `class` itself makes it possible for the user
    // to still set `class` on the component without it being overriden)
    /**
     * @return {?}
     */
    get _hostClassBtnPrimary() {
        return this.style === 'primary';
    }
    /**
     * @return {?}
     */
    get _hostClassBtnSecondary() {
        return this.style === 'secondary';
    }
    /**
     * @return {?}
     */
    get _hostClassBtnSuccess() {
        return this.style === 'success';
    }
    /**
     * @return {?}
     */
    get _hostClassBtnDanger() {
        return this.style === 'danger';
    }
    /**
     * @return {?}
     */
    get _hostClassBtnWarning() {
        return this.style === 'warning';
    }
    /**
     * @return {?}
     */
    get _hostClassBtnInfo() {
        return this.style === 'info';
    }
    /**
     * @return {?}
     */
    get _hostClassBtnLight() {
        return this.style === 'light';
    }
    /**
     * @return {?}
     */
    get _hostClassBtnDark() {
        return this.style === 'dark';
    }
    /**
     * @return {?}
     */
    get _hostClassBtnLink() {
        return this.style === 'link';
    }
    /**
     * @return {?}
     */
    get _hostClassBtnOutlinePrimary() {
        return this.style === 'outline-primary';
    }
    /**
     * @return {?}
     */
    get _hostClassBtnOutlineSecondary() {
        return this.style === 'outline-secondary';
    }
    /**
     * @return {?}
     */
    get _hostClassBtnOutlineSuccess() {
        return this.style === 'outline-success';
    }
    /**
     * @return {?}
     */
    get _hostClassBtnOutlineDanger() {
        return this.style === 'outline-danger';
    }
    /**
     * @return {?}
     */
    get _hostClassBtnOutlineWarning() {
        return this.style === 'outline-warning';
    }
    /**
     * @return {?}
     */
    get _hostClassBtnOutlineInfo() {
        return this.style === 'outline-info';
    }
    /**
     * @return {?}
     */
    get _hostClassBtnOutlineLight() {
        return this.style === 'outline-light';
    }
    /**
     * @return {?}
     */
    get _hostClassBtnOutlineDark() {
        return this.style === 'outline-dark';
    }
    // Set the corresponding style class
    /**
     * @return {?}
     */
    get _hostClassBtnSm() {
        return this.size === 'sm';
    }
    /**
     * @return {?}
     */
    get _hostClassBtnLg() {
        return this.size === 'lg';
    }
    // Set the dropdown toggle class when possible
    /**
     * @return {?}
     */
    get _hostClassDropdownToggle() {
        return (this._tagName !== 'input' &&
            (this.dropdownToggle || this.dropdownToggleSplit));
    }
    // Set type to `'button'` by default (on `button`s and `input`s)
    /**
     * @return {?}
     */
    get _hostType() {
        return this._tagName !== 'a' ? this.type || 'button' : undefined;
    }
    /**
     * @return {?}
     */
    get _hostDisabled() {
        return this._tagName !== 'a' && this.isDisabled ? '' : undefined;
    }
    /**
     * @return {?}
     */
    get _hostAriaDisabled() {
        return this._tagName !== 'a' && this.isDisabled ? 'true' : undefined;
    }
    /**
     * @return {?}
     */
    get _hostAriaPressed() {
        return this.isActive ? 'true' : undefined;
    }
}
ButtonComponent.decorators = [
    { type: Component, args: [{
                // tslint:disable-next-line:component-selector
                selector: 'button[lf-button], input[lf-button], a[lf-button]',
                changeDetection: ChangeDetectionStrategy.OnPush,
                template: '<ng-container *mobxAutorun><ng-content></ng-content></ng-container>'
            }] }
];
/** @nocollapse */
ButtonComponent.ctorParameters = () => [
    { type: ElementRef }
];
ButtonComponent.propDecorators = {
    style: [{ type: Input }],
    size: [{ type: Input }],
    type: [{ type: Input }],
    block: [{ type: Input }, { type: HostBinding, args: ['class.btn-block',] }],
    dropdownToggle: [{ type: Input }],
    dropdownToggleSplit: [{ type: Input }, { type: HostBinding, args: ['class.dropdown-toggle-split',] }],
    isDisabled: [{ type: Input }],
    isActive: [{ type: Input }, { type: HostBinding, args: ['class.active',] }],
    _hostClassBtn: [{ type: HostBinding, args: ['class.btn',] }],
    _role: [{ type: HostBinding, args: ['attr.role',] }],
    _hostClassBtnPrimary: [{ type: HostBinding, args: ['class.btn-primary',] }],
    _hostClassBtnSecondary: [{ type: HostBinding, args: ['class.btn-secondary',] }],
    _hostClassBtnSuccess: [{ type: HostBinding, args: ['class.btn-success',] }],
    _hostClassBtnDanger: [{ type: HostBinding, args: ['class.btn-danger',] }],
    _hostClassBtnWarning: [{ type: HostBinding, args: ['class.btn-warning',] }],
    _hostClassBtnInfo: [{ type: HostBinding, args: ['class.btn-info',] }],
    _hostClassBtnLight: [{ type: HostBinding, args: ['class.btn-light',] }],
    _hostClassBtnDark: [{ type: HostBinding, args: ['class.btn-dark',] }],
    _hostClassBtnLink: [{ type: HostBinding, args: ['class.btn-link',] }],
    _hostClassBtnOutlinePrimary: [{ type: HostBinding, args: ['class.btn-outline-primary',] }],
    _hostClassBtnOutlineSecondary: [{ type: HostBinding, args: ['class.btn-outline-secondary',] }],
    _hostClassBtnOutlineSuccess: [{ type: HostBinding, args: ['class.btn-outline-success',] }],
    _hostClassBtnOutlineDanger: [{ type: HostBinding, args: ['class.btn-outline-danger',] }],
    _hostClassBtnOutlineWarning: [{ type: HostBinding, args: ['class.btn-outline-warning',] }],
    _hostClassBtnOutlineInfo: [{ type: HostBinding, args: ['class.btn-outline-info',] }],
    _hostClassBtnOutlineLight: [{ type: HostBinding, args: ['class.btn-outline-light',] }],
    _hostClassBtnOutlineDark: [{ type: HostBinding, args: ['class.btn-outline-dark',] }],
    _hostClassBtnSm: [{ type: HostBinding, args: ['class.btn-sm',] }],
    _hostClassBtnLg: [{ type: HostBinding, args: ['class.btn-lg',] }],
    _hostClassDropdownToggle: [{ type: HostBinding, args: ['class.dropdown-toggle',] }],
    _hostType: [{ type: HostBinding, args: ['attr.type',] }],
    _hostDisabled: [{ type: HostBinding, args: ['attr.disabled',] }],
    _hostAriaDisabled: [{ type: HostBinding, args: ['attr.aria-disabled',] }],
    _hostAriaPressed: [{ type: HostBinding, args: ['attr.aria-pressed',] }]
};
__decorate([
    observable$1,
    __metadata("design:type", String)
], ButtonComponent.prototype, "style", void 0);
__decorate([
    observable$1,
    __metadata("design:type", String)
], ButtonComponent.prototype, "size", void 0);
__decorate([
    observable$1,
    __metadata("design:type", String)
], ButtonComponent.prototype, "type", void 0);
__decorate([
    observable$1,
    __metadata("design:type", Object)
], ButtonComponent.prototype, "block", void 0);
__decorate([
    observable$1,
    __metadata("design:type", Object)
], ButtonComponent.prototype, "dropdownToggle", void 0);
__decorate([
    observable$1,
    __metadata("design:type", Object)
], ButtonComponent.prototype, "dropdownToggleSplit", void 0);
__decorate([
    observable$1,
    __metadata("design:type", Object)
], ButtonComponent.prototype, "isDisabled", void 0);
__decorate([
    observable$1,
    __metadata("design:type", Object)
], ButtonComponent.prototype, "isActive", void 0);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], ButtonComponent.prototype, "_hostClassBtnPrimary", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], ButtonComponent.prototype, "_hostClassBtnSecondary", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], ButtonComponent.prototype, "_hostClassBtnSuccess", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], ButtonComponent.prototype, "_hostClassBtnDanger", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], ButtonComponent.prototype, "_hostClassBtnWarning", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], ButtonComponent.prototype, "_hostClassBtnInfo", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], ButtonComponent.prototype, "_hostClassBtnLight", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], ButtonComponent.prototype, "_hostClassBtnDark", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], ButtonComponent.prototype, "_hostClassBtnLink", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], ButtonComponent.prototype, "_hostClassBtnOutlinePrimary", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], ButtonComponent.prototype, "_hostClassBtnOutlineSecondary", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], ButtonComponent.prototype, "_hostClassBtnOutlineSuccess", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], ButtonComponent.prototype, "_hostClassBtnOutlineDanger", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], ButtonComponent.prototype, "_hostClassBtnOutlineWarning", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], ButtonComponent.prototype, "_hostClassBtnOutlineInfo", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], ButtonComponent.prototype, "_hostClassBtnOutlineLight", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], ButtonComponent.prototype, "_hostClassBtnOutlineDark", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], ButtonComponent.prototype, "_hostClassBtnSm", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], ButtonComponent.prototype, "_hostClassBtnLg", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], ButtonComponent.prototype, "_hostClassDropdownToggle", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], ButtonComponent.prototype, "_hostType", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], ButtonComponent.prototype, "_hostDisabled", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], ButtonComponent.prototype, "_hostAriaDisabled", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], ButtonComponent.prototype, "_hostAriaPressed", null);
if (false) {
    /**
     * Button style. See available styles at [Bootstrap documentation](
     * https://getbootstrap.com/docs/4.0/components/buttons/#examples).
     * @type {?}
     */
    ButtonComponent.prototype.style;
    /**
     * Button size. See available sizes at [Bootstrap documentation](
     * https://getbootstrap.com/docs/4.0/components/buttons/#sizes).
     * @type {?}
     */
    ButtonComponent.prototype.size;
    /**
     * HTML button type.
     * @type {?}
     */
    ButtonComponent.prototype.type;
    /**
     * Match the width of parent if `true`.
     * @type {?}
     */
    ButtonComponent.prototype.block;
    /**
     * Display a dropdown icon (this is automatic if `dropdownToggleSplit` is
     * set).
     * @type {?}
     */
    ButtonComponent.prototype.dropdownToggle;
    /**
     * Reduces the padding of the button so it can be used as the dropdown part of
     * a split dropdown.
     * @type {?}
     */
    ButtonComponent.prototype.dropdownToggleSplit;
    /**
     * Set the button as disabled.
     * @type {?}
     */
    ButtonComponent.prototype.isDisabled;
    /**
     * Set the button as active.
     * @type {?}
     */
    ButtonComponent.prototype.isActive;
    /**
     * Always add the class `btn` to LF buttons.
     * @type {?}
     */
    ButtonComponent.prototype._hostClassBtn;
    /**
     * Tag of the element (`'button'` or `'a'`).
     * @type {?}
     * @protected
     */
    ButtonComponent.prototype._tagName;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Content of an `lf-card`.
 */
class CardContentComponent extends PathBased {
    // Classes for each type of card content
    /**
     * @return {?}
     */
    get _hostClassCardHeader() {
        return this.contentType === 'header';
    }
    /**
     * @return {?}
     */
    get _hostClassCardBody() {
        return this.contentType === 'body';
    }
    /**
     * @return {?}
     */
    get _hostClassCardFooter() {
        return this.contentType === 'footer';
    }
    /**
     * @return {?}
     */
    get _hostClassCardTitle() {
        return this.contentType === 'title';
    }
    /**
     * @return {?}
     */
    get _hostClassCardSubtitle() {
        return this.contentType === 'subtitle';
    }
    /**
     * @return {?}
     */
    get _hostClassCardText() {
        return this.contentType === 'text';
    }
    /**
     * @return {?}
     */
    get _hostClassCardLink() {
        return this.contentType === 'link';
    }
    /**
     * @return {?}
     */
    get _hostClassCardImg() {
        return this.contentType === 'img';
    }
    /**
     * @return {?}
     */
    get _hostClassCardImgOverlay() {
        return this.contentType === 'img-overlay';
    }
    /**
     * @return {?}
     */
    get _hostClassCardImgTop() {
        return this.contentType === 'img-top';
    }
    /**
     * @return {?}
     */
    get _hostClassCardImgBottom() {
        return this.contentType === 'img-bottom';
    }
}
CardContentComponent.decorators = [
    { type: Component, args: [{
                selector: 'lf-card-content, [lf-card-content]',
                changeDetection: ChangeDetectionStrategy.OnPush,
                providers: [{ provide: PathBased, useExisting: CardContentComponent }],
                template: '<ng-container *mobxAutorun><ng-content></ng-content></ng-container>'
            }] }
];
CardContentComponent.propDecorators = {
    contentType: [{ type: Input }],
    _hostClassCardHeader: [{ type: HostBinding, args: ['class.card-header',] }],
    _hostClassCardBody: [{ type: HostBinding, args: ['class.card-body',] }],
    _hostClassCardFooter: [{ type: HostBinding, args: ['class.card-footer',] }],
    _hostClassCardTitle: [{ type: HostBinding, args: ['class.card-title',] }],
    _hostClassCardSubtitle: [{ type: HostBinding, args: ['class.card-subtitle',] }],
    _hostClassCardText: [{ type: HostBinding, args: ['class.card-text',] }],
    _hostClassCardLink: [{ type: HostBinding, args: ['class.card-link',] }],
    _hostClassCardImg: [{ type: HostBinding, args: ['class.card-img',] }],
    _hostClassCardImgOverlay: [{ type: HostBinding, args: ['class.card-img-overlay',] }],
    _hostClassCardImgTop: [{ type: HostBinding, args: ['class.card-img-top',] }],
    _hostClassCardImgBottom: [{ type: HostBinding, args: ['class.card-img-bottom',] }]
};
__decorate([
    observable$1,
    __metadata("design:type", String)
], CardContentComponent.prototype, "contentType", void 0);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], CardContentComponent.prototype, "_hostClassCardHeader", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], CardContentComponent.prototype, "_hostClassCardBody", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], CardContentComponent.prototype, "_hostClassCardFooter", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], CardContentComponent.prototype, "_hostClassCardTitle", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], CardContentComponent.prototype, "_hostClassCardSubtitle", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], CardContentComponent.prototype, "_hostClassCardText", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], CardContentComponent.prototype, "_hostClassCardLink", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], CardContentComponent.prototype, "_hostClassCardImg", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], CardContentComponent.prototype, "_hostClassCardImgOverlay", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], CardContentComponent.prototype, "_hostClassCardImgTop", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], CardContentComponent.prototype, "_hostClassCardImgBottom", null);
if (false) {
    /**
     * Type of the content.
     * @type {?}
     */
    CardContentComponent.prototype.contentType;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Bootstrap card component, see options on how to further customise the card
 * with background and border utilities at [Bootstrap documentation](
 * https://getbootstrap.com/docs/4.1/components/card/#card-styles).
 */
class CardComponent extends PathBased {
    constructor() {
        super(...arguments);
        /**
         * Always add the class `card` to LF cards.
         */
        this._hostClassCard = true;
    }
}
CardComponent.decorators = [
    { type: Component, args: [{
                selector: 'lf-card, [lf-card]',
                changeDetection: ChangeDetectionStrategy.OnPush,
                providers: [{ provide: PathBased, useExisting: CardComponent }],
                template: '<ng-container *mobxAutorun><ng-content></ng-content></ng-container>'
            }] }
];
CardComponent.propDecorators = {
    _hostClassCard: [{ type: HostBinding, args: ['class.card',] }]
};
if (false) {
    /**
     * Always add the class `card` to LF cards.
     * @type {?}
     */
    CardComponent.prototype._hostClassCard;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Abstract class used to represent typical "inline-able" values.
 * @abstract
 */
class InlineValue extends Value {
    constructor() {
        super(...arguments);
        /**
         * Whether the component should be hidden when its value isn't required.
         * **NOT READY FOR USE**
         */
        // TODO: What about components that aren't nullable? Example: tables, text, we
        // should use minSize/minLength instead. Allow each component to implement its
        // own "isRequired" functionality?
        this._hideWhenNotRequired = false;
        /**
         * Event emitter which emits an event whenever the component's DOM input is
         * focused.
         */
        this.focus = new EventEmitter();
        /**
         * Event emitter which emits an event whenever the component's DOM input is
         * blurred.
         */
        this.blur = new EventEmitter();
        /**
         * Add an `lf-inline-value` class to components.
         */
        this._classLfInlineValue = true;
    }
    // Angular lifecycle =========================================================
    /**
     * @return {?}
     */
    ngOnInit() {
        this.lfRouter = this.injector.get(LfRouter);
        super.ngOnInit();
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        // Auto-focus the field when this component's path is activated
        this.disposeOnActive = autorun((/**
         * @return {?}
         */
        () => {
            if (this.lfRouter.isActiveWithFragment(this.path)) {
                /** @type {?} */
                const res = this.onActivate && this.onActivate((/** @type {?} */ (this.lfRouter.pathWithFragment)));
                // Trigger the `focus` only when path matches exactly
                if (res !== false &&
                    this.lfRouter.isActiveWithFragment(this.path, true)) {
                    asap((/**
                     * @return {?}
                     */
                    () => this.triggerFocus()));
                }
            }
        }), { name: 'fieldAutorunFocusOnActive' });
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        super.ngOnDestroy();
        this.disposeOnActive && this.disposeOnActive();
    }
    // ===========================================================================
    /**
     * Whether this component should be displayed.
     * @return {?}
     */
    get shouldDisplay() {
        return (this.relativeStorage.has() &&
            (!this._hideWhenNotRequired ||
                this.relativeStorage.getStateProperty('.', 'isRequired')));
    }
    /**
     * Scrolls this component into the center of the viewport and focuses it.
     * @return {?}
     */
    triggerFocus() {
        if (this.targets && this.targets.length > 0) {
            /** @type {?} */
            const target = this.targets.first.nativeElement;
            target.scrollIntoView({ block: 'center' });
            target.focus();
        }
    }
    /**
     * Action to run whenever a focusable part of the field is focused.
     * @param {?} evt `focus` event.
     * @return {?}
     */
    _onFocus(evt) {
        this.focus.emit(evt);
    }
    /**
     * Action to run whenever a focusable part of the field is blurred.
     * @param {?} evt `blur` event.
     * @return {?}
     */
    _onBlur(evt) {
        if (!this.relativeStorage.isComputed()) {
            this.relativeStorage.setTouched();
        }
        // Remove the fragment when the path exactly matches the URL
        if (this.lfRouter.isActiveWithFragment(this.path, true)) {
            this.lfRouter.navigate('.', { queryParamsHandling: 'merge' });
        }
        this.blur.emit(evt);
    }
}
InlineValue.propDecorators = {
    _hideWhenNotRequired: [{ type: Input }],
    focus: [{ type: Output }],
    blur: [{ type: Output }],
    _classLfInlineValue: [{ type: HostBinding, args: ['class.lf-inline-value',] }],
    targets: [{ type: ViewChildren, args: ['focusTarget',] }]
};
__decorate([
    observable$1,
    __metadata("design:type", Object)
], InlineValue.prototype, "_hideWhenNotRequired", void 0);
__decorate([
    computed,
    __metadata("design:type", Boolean),
    __metadata("design:paramtypes", [])
], InlineValue.prototype, "shouldDisplay", null);
if (false) {
    /**
     * Whether the component should be hidden when its value isn't required.
     * **NOT READY FOR USE**
     * @type {?}
     */
    InlineValue.prototype._hideWhenNotRequired;
    /**
     * Event emitter which emits an event whenever the component's DOM input is
     * focused.
     * @type {?}
     */
    InlineValue.prototype.focus;
    /**
     * Event emitter which emits an event whenever the component's DOM input is
     * blurred.
     * @type {?}
     */
    InlineValue.prototype.blur;
    /**
     * Add an `lf-inline-value` class to components.
     * @type {?}
     */
    InlineValue.prototype._classLfInlineValue;
    /**
     * Target elements that should be the target of focus/blur events; the first
     * element of que query list is the one that will be focused on route change
     * or when demanded via `focus()`.
     * @type {?}
     * @protected
     */
    InlineValue.prototype.targets;
    /**
     * @type {?}
     * @protected
     */
    InlineValue.prototype.lfRouter;
    /**
     * Disposer for the "auto-focus" of the field when it becomes active.
     * @type {?}
     * @private
     */
    InlineValue.prototype.disposeOnActive;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Type of an option supported by components which present a list of options
 * from which to select.
 * @record
 * @template T
 */
function Option() { }
if (false) {
    /**
     * Value of the option.
     * @type {?}
     */
    Option.prototype.value;
    /**
     * Human-readable string to represent the option.
     * @type {?|undefined}
     */
    Option.prototype.label;
    /**
     * Code of the option.
     * @type {?|undefined}
     */
    Option.prototype.code;
    /**
     * Help message associated with the option.
     * @type {?|undefined}
     */
    Option.prototype.helpMessage;
    /**
     * Whether the option should show up as disabled.
     * @type {?|undefined}
     */
    Option.prototype.isDisabled;
}
/**
 * Class which represents a component whose value may be restricted by a set of
 * options.
 * @abstract
 */
class SelectionValue extends InlineValue {
    // I18n ======================================================================
    /**
     * Selectable options (possibly fetched asynchronously).
     * @return {?}
     */
    // Not `computed` due to `useState` (cannot be used within a `computed`)
    get options() {
        return this.translate(I18N_SELECTION_OPTIONS_KEY, { useState: true }) || [];
    }
    /**
     * Text to be shown when there are no options available.
     * @return {?}
     */
    get noOptionsText() {
        return this.translate(I18N_SELECTION_NO_OPTIONS_TEXT_KEY);
    }
    /**
     * Promise-status of the options.
     * @return {?}
     */
    get optionsStatus() {
        return this.translationStatus(I18N_SELECTION_OPTIONS_KEY);
    }
}
__decorate([
    computed,
    __metadata("design:type", String),
    __metadata("design:paramtypes", [])
], SelectionValue.prototype, "noOptionsText", null);
__decorate([
    computed,
    __metadata("design:type", String),
    __metadata("design:paramtypes", [])
], SelectionValue.prototype, "optionsStatus", null);

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Component used for selecting a list of options by using inputs of type
 * `checkbox`.
 */
class CheckboxGroupComponent extends SelectionValue {
    constructor() {
        super(...arguments);
        /**
         * Whether the list of checkbox inputs should be displayed inline. The
         * component will still display in "block" mode in small screens.
         */
        this.displayInline = false;
    }
    /**
     * Whether a given option is selected.
     * @param {?} option Option to check if it is selected.
     * @return {?}
     */
    isSelected(option) {
        return this.relativeStorage
            .get()
            .some((/**
         * @param {?} el
         * @return {?}
         */
        el => comparer.structural(el, option.value)));
    }
    /**
     * Whether to show the "is required" icon on the component.
     * @return {?}
     */
    get showIsRequired() {
        /** @type {?} */
        const minSize = this.relativeStorage.getStateProperty('.', 'minSize');
        return minSize !== undefined && minSize > 0;
    }
    /**
     * Action to run whenever a checkbox `input` changes.
     * @param {?} option Option that changed.
     * @param {?} evt
     * @return {?}
     */
    _onOptionChange(option, evt) {
        /** @type {?} */
        const checked = !!evt.target.checked;
        /** @type {?} */
        const isSelected = this.isSelected(option);
        if (isSelected && !checked) {
            this.relativeStorage.filter('.', (/**
             * @param {?} el
             * @return {?}
             */
            el => !comparer.structural(el, option.value)));
        }
        else if (!isSelected && checked) {
            this.relativeStorage.push('.', option.value);
        }
        this.relativeStorage.setDirty();
    }
    /**
     * @protected
     * @return {?}
     */
    validatePath() {
        /** @type {?} */
        const schema = this.relativeStorage.schema();
        if (!isListSchema(schema) || isNullableSchema(schema)) {
            throw pathError(this.path, 'Schema must be of the "list" type and not "nullable"');
        }
    }
}
CheckboxGroupComponent.decorators = [
    { type: Component, args: [{
                selector: 'lf-checkbox-group',
                changeDetection: ChangeDetectionStrategy.OnPush,
                template: "<ng-container *mobxAutorun>\n  <ng-container *ngIf=\"shouldDisplay\">\n    <!-- Label -->\n    <span *ngIf=\"label\" class=\"label\">{{ label }}</span>\n    <lf-help-tooltip *ngIf=\"helpMessage\" [help]=\"helpMessage\"></lf-help-tooltip>\n\n    <!-- Required icon -->\n    <lf-icon\n      class=\"is-required\"\n      icon=\"asterisk\"\n      *ngIf=\"showIsRequired\"\n    ></lf-icon>\n\n    <!-- Pending validation -->\n    <div\n      class=\"status pending\"\n      *ngIf=\"relativeStorage.shouldShowPending()\"\n    ></div>\n    <!-- Rejected validation -->\n    <div\n      class=\"status rejected\"\n      *ngIf=\"relativeStorage.shouldShowRejected()\"\n      (click)=\"relativeStorage.recomputeStateProperties()\"\n    >\n      <lf-tooltip [help]=\"retryValidateText\">\n        <lf-icon icon=\"warning\"></lf-icon>\n      </lf-tooltip>\n    </div>\n\n    <!-- (Debug) value state -->\n    <lf-value-state [path]=\"path\"></lf-value-state>\n\n    <!-- Input group -->\n    <div\n      class=\"lf-checkable-group input-group input-group-sm\"\n      [class.is-warning]=\"relativeStorage.shouldShowWarning()\"\n      [class.is-invalid]=\"relativeStorage.shouldShowError()\"\n    >\n      <!-- Code -->\n      <div *ngIf=\"code\" class=\"input-group-prepend\">\n        <div class=\"input-group-text\">{{ code }}</div>\n      </div>\n\n      <!-- Checkboxes -->\n      <div\n        class=\"lf-checkables form-control\"\n        [class.is-warning]=\"relativeStorage.shouldShowWarning()\"\n        [class.is-invalid]=\"relativeStorage.shouldShowError()\"\n        [class.is-readonly]=\"isReadOnly\"\n        [class.display-inline]=\"displayInline\"\n      >\n        <!-- Text for when there are no options -->\n        <div\n          *ngIf=\"optionsStatus !== 'pending' && options.length === 0\"\n          class=\"no-options\"\n        >\n          {{ noOptionsText }}\n        </div>\n\n        <!-- Pending options -->\n        <div *ngIf=\"optionsStatus === 'pending'\" class=\"pending-options\"></div>\n\n        <div\n          class=\"lf-checkable input-group input-group-sm\"\n          *ngFor=\"let option of options; let i = index\"\n        >\n          <!-- Code of each checkbox -->\n          <div *ngIf=\"option.code\" class=\"input-group-prepend\">\n            <div class=\"input-group-text\">{{ option.code }}</div>\n          </div>\n\n          <!-- Checkbox -->\n          <label\n            class=\"form-control custom-control custom-checkbox\"\n            [class.is-warning]=\"relativeStorage.shouldShowWarning()\"\n            [class.is-invalid]=\"relativeStorage.shouldShowError()\"\n            [class.is-readonly]=\"isReadOnly\"\n          >\n            <!-- Actual checkbox -->\n            <!--\n              Note that checkboxes don't support `readonly`, so we use\n              the `disabled` attribute\n            -->\n            <input\n              #focusTarget\n              type=\"checkbox\"\n              class=\"custom-control-input\"\n              [name]=\"path + '[' + i + ']'\"\n              [value]=\"i\"\n              [checked]=\"isSelected(option)\"\n              [disabled]=\"isReadOnly\"\n              (change)=\"_onOptionChange(option, $event)\"\n              (focus)=\"_onFocus($event)\"\n              (blur)=\"_onBlur($event)\"\n            />\n\n            <!-- Checkbox label -->\n            <span class=\"custom-control-label\">\n              {{ option.label }}\n              <lf-help-tooltip\n                *ngIf=\"option.helpMessage\"\n                [help]=\"option.helpMessage\"\n              ></lf-help-tooltip>\n            </span>\n          </label>\n        </div>\n      </div>\n    </div>\n\n    <!-- Legend -->\n    <div *ngIf=\"legend\" class=\"legend\">{{ legend }}</div>\n  </ng-container>\n</ng-container>\n"
            }] }
];
CheckboxGroupComponent.propDecorators = {
    displayInline: [{ type: Input }]
};
__decorate([
    observable$1,
    __metadata("design:type", Object)
], CheckboxGroupComponent.prototype, "displayInline", void 0);
__decorate([
    computed,
    __metadata("design:type", Boolean),
    __metadata("design:paramtypes", [])
], CheckboxGroupComponent.prototype, "showIsRequired", null);
__decorate([
    action,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", void 0)
], CheckboxGroupComponent.prototype, "_onOptionChange", null);
if (false) {
    /**
     * Whether the list of checkbox inputs should be displayed inline. The
     * component will still display in "block" mode in small screens.
     * @type {?}
     */
    CheckboxGroupComponent.prototype.displayInline;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * LF checkbox component.
 */
class CheckboxComponent extends InlineValue {
    /**
     * Whether to show the "is required" icon on the component.
     * @return {?}
     */
    get showIsRequired() {
        if (!this.relativeStorage.isNullable()) {
            /** @type {?} */
            const allowedValues = this.relativeStorage.getStateProperty('.', 'allowedValues');
            return (allowedValues && allowedValues.length === 1 && allowedValues[0] === true);
        }
        /** @type {?} */
        const isRequired = this.relativeStorage.getStateProperty('.', 'isRequired');
        return !!isRequired;
    }
    /**
     * Action to run whenever the value of the checkbox changes.
     * @param {?} evt `change` event.
     * @return {?}
     */
    _onCheckboxChange(evt) {
        this.relativeStorage.set('.', !!evt.target.checked);
        this.relativeStorage.setDirty();
    }
    /**
     * @protected
     * @return {?}
     */
    validatePath() {
        /** @type {?} */
        const schema = this.relativeStorage.schema();
        if (!isBooleanSchema(schema)) {
            throw pathError(this.path, 'Schema must be of the "boolean" type');
        }
    }
}
CheckboxComponent.decorators = [
    { type: Component, args: [{
                selector: 'lf-checkbox',
                changeDetection: ChangeDetectionStrategy.OnPush,
                template: "<ng-container *mobxAutorun>\n  <ng-container *ngIf=\"shouldDisplay\">\n    <!-- Required icon -->\n    <lf-icon\n      class=\"is-required\"\n      icon=\"asterisk\"\n      *ngIf=\"showIsRequired\"\n    ></lf-icon>\n\n    <!-- Pending validation -->\n    <div\n      class=\"status pending\"\n      *ngIf=\"relativeStorage.shouldShowPending()\"\n    ></div>\n    <!-- Rejected validation -->\n    <div\n      class=\"status rejected\"\n      *ngIf=\"relativeStorage.shouldShowRejected()\"\n      (click)=\"relativeStorage.recomputeStateProperties()\"\n    >\n      <lf-tooltip [help]=\"retryValidateText\">\n        <lf-icon icon=\"warning\"></lf-icon>\n      </lf-tooltip>\n    </div>\n\n    <!-- (Debug) value state -->\n    <lf-value-state [path]=\"path\"></lf-value-state>\n\n    <!-- Input group -->\n    <div\n      class=\"lf-checkable input-group input-group-sm\"\n      [class.is-warning]=\"relativeStorage.shouldShowWarning()\"\n      [class.is-invalid]=\"relativeStorage.shouldShowError()\"\n    >\n      <!-- Code -->\n      <div *ngIf=\"code\" class=\"input-group-prepend\">\n        <div class=\"input-group-text\">{{ code }}</div>\n      </div>\n\n      <!-- Checkbox -->\n      <label\n        class=\"form-control custom-control custom-checkbox\"\n        [class.is-warning]=\"relativeStorage.shouldShowWarning()\"\n        [class.is-invalid]=\"relativeStorage.shouldShowError()\"\n        [class.is-readonly]=\"isReadOnly\"\n      >\n        <!-- Actual checkbox -->\n        <!--\n          Note that checkboxes don't support `readonly`, so we use the\n          `disabled` attribute\n        -->\n        <input\n          #focusTarget\n          type=\"checkbox\"\n          class=\"custom-control-input\"\n          [name]=\"path\"\n          [checked]=\"relativeStorage.get()\"\n          [indeterminate]=\"relativeStorage.get() === null\"\n          (change)=\"_onCheckboxChange($event)\"\n          [disabled]=\"isReadOnly\"\n          (focus)=\"_onFocus($event)\"\n          (blur)=\"_onBlur($event)\"\n        />\n\n        <!-- Label -->\n        <span class=\"custom-control-label\">\n          <span *ngIf=\"label\" class=\"label\">{{ label }}</span>\n          <lf-help-tooltip\n            *ngIf=\"helpMessage\"\n            [help]=\"helpMessage\"\n          ></lf-help-tooltip>\n        </span>\n      </label>\n    </div>\n\n    <!-- Legend -->\n    <div *ngIf=\"legend\" class=\"legend\">{{ legend }}</div>\n  </ng-container>\n</ng-container>\n"
            }] }
];
__decorate([
    computed,
    __metadata("design:type", Boolean),
    __metadata("design:paramtypes", [])
], CheckboxComponent.prototype, "showIsRequired", null);
__decorate([
    action,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], CheckboxComponent.prototype, "_onCheckboxChange", null);

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Date-range schema.
 * @record
 */
function DateRangeSchema() { }
if (false) {
    /**
     * Minimum date that the date-range represented by this schema is allowed to
     * have.
     * @type {?|undefined}
     */
    DateRangeSchema.prototype.minDate;
    /**
     * Maximum date that the date-range represented by this schema is allowed to
     * have.
     * @type {?|undefined}
     */
    DateRangeSchema.prototype.maxDate;
    /**
     * Options to construct the start date schema with (`minDate` and `maxDate`
     * are not allowed).
     * @type {?|undefined}
     */
    DateRangeSchema.prototype.startDateOptions;
    /**
     * Options to construct the end date schema with (`minDate` and `maxDate` are
     * not allowed).
     * @type {?|undefined}
     */
    DateRangeSchema.prototype.endDateOptions;
}
/**
 * Creates a new date-range schema object.
 * @param {?=} options Options used to create the schema.
 * @return {?} Date-range schema.
 */
function dateRangeSchema(options = {}) {
    /** @type {?} */
    const startDateOptions = options.startDateOptions || {};
    /** @type {?} */
    const endDateOptions = options.endDateOptions || {};
    // When the date-range isn't computed, we apply `minDate` and `maxDate` to the
    // inner tuple elements for automatic validation. However, we need to fix the
    // storage path so that the user can write the `minDate` and `maxDate` as if
    // they were defined on the tuple itself
    if (options.computedValue === undefined) {
        /** @type {?} */
        const minDate = typeof options.minDate === 'function'
            ? (/**
             * @param {?} ctx
             * @return {?}
             */
            ctx => ctx.schema().minDate(ctx.relativeStorage('..')))
            : options.minDate;
        /** @type {?} */
        const maxDate = typeof options.maxDate === 'function'
            ? (/**
             * @param {?} ctx
             * @return {?}
             */
            ctx => ctx.schema().maxDate(ctx.relativeStorage('..')))
            : options.maxDate;
        startDateOptions.minDate = minDate;
        // Disallow an end date that comes before the start date
        endDateOptions.minDate = (/**
         * @param {?} ctx
         * @return {?}
         */
        ctx => ctx.get('0'));
        endDateOptions.maxDate = maxDate;
    }
    // Date-ranges are nullable by default (TODO: should this not be the case, to
    // make them consistent with other schema definitions?)
    return (/** @type {?} */ (tupleSchema([dateSchema(startDateOptions), dateSchema(endDateOptions)], Object.assign({ isNullable: true }, options))));
}
/**
 * Returns whether a given schema is a date-range schema.
 * @param {?} schema Schema to check.
 * @return {?} `true` when the provided schema is a date-range schema.
 */
function isDateRangeSchema(schema) {
    return (isTupleSchema(schema) &&
        schema.elementsSchemas.length === 2 &&
        isDateSchema(schema.elementsSchemas[0]) &&
        isDateSchema(schema.elementsSchemas[1]));
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * LF date-range component.
 */
class DateRangeComponent extends InlineValue {
    constructor() {
        super(...arguments);
        // Variable used to prevent a cycle when updating the datepicker and the app's
        // value
        this._settingValue = false;
    }
    // Angular lifecycle =========================================================
    /**
     * @return {?}
     */
    ngOnInit() {
        extendObservable(this, { _dummyValue: undefined }, { _dummyValue: observable.struct });
        super.ngOnInit();
        this._disposeDateRangeAutorun = autorun((/**
         * @return {?}
         */
        () => {
            /** @type {?} */
            const value = this.relativeStorage.getAsJS();
            if (!this._settingValue) {
                this._dummyValue = value;
            }
            else {
                this._settingValue = false;
            }
        }), { name: 'dateRangeAutorunSetDummyValue' });
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        super.ngOnDestroy();
        this._disposeDateRangeAutorun();
    }
    // I18n ======================================================================
    /**
     * Format for a single date.
     * @return {?}
     */
    get dateFormat() {
        return this.translate(I18N_DATE_FORMAT_KEY);
    }
    /**
     * Separator used between the dates in the date-range.
     * @return {?}
     */
    get separator() {
        return this.translate(I18N_DATE_RANGE_SEPARATOR_KEY) || '-';
    }
    /**
     * Placeholder of the `input` field.
     * @return {?}
     */
    get placeholder() {
        /** @type {?} */
        const datePlaceholder = this.translate(I18N_DATE_PLACEHOLDER_KEY);
        return datePlaceholder
            ? `${datePlaceholder}${this.separator}${datePlaceholder}`
            : '';
    }
    // ===========================================================================
    /**
     * Whether to show the "is required" icon on the component.
     * @return {?}
     */
    get showIsRequired() {
        /** @type {?} */
        const isRequired = this.relativeStorage.getStateProperty('.', 'isRequired');
        return !!isRequired;
    }
    /**
     * Configuration to use by Bootstrap datepicker.
     * @return {?}
     */
    get bsConfig() {
        return {
            containerClass: 'theme-default',
            rangeInputFormat: this.dateFormat,
            rangeSeparator: this.separator,
        };
    }
    /**
     * Action to run whenever the date range is modified via the Bootstrap
     * datepicker UI.
     * @param {?} tuple Tuple representing the range.
     * @return {?}
     */
    _onDateRangePickerChange(tuple) {
        // FIXME: When the `rangeInputFormat` is not the default one, manually
        // setting the input causes `tuple` to contain `[null, null]`; this should
        // probably be fixed from ngx-bootstrap's side
        if (tuple && (!tuple[0] || !tuple[1])) {
            tuple = null;
        }
        this._settingValue = true;
        this.relativeStorage.set('.', tuple);
        this.relativeStorage.setDirty('.', true);
        // Clicking on the pop-up should count as "touching"
        this.relativeStorage.setTouched('.', true);
    }
    /**
     * @param {?} evt
     * @return {?}
     */
    _onBlur(evt) {
        if (!this.relativeStorage.isComputed()) {
            // Make sure to set inner dates as touched, so that if they have errors
            // (min/max issues) then they properly show
            this.relativeStorage.setTouched('.', true);
        }
        this.blur.emit(evt);
    }
    /**
     * @protected
     * @return {?}
     */
    validatePath() {
        /** @type {?} */
        const schema = this.relativeStorage.schema();
        if (!isDateRangeSchema(schema) || !isNullableSchema(schema)) {
            throw pathError(this.path, 'Schema must be of the "date-range" type and "nullable"');
        }
    }
}
DateRangeComponent.decorators = [
    { type: Component, args: [{
                selector: 'lf-date-range',
                changeDetection: ChangeDetectionStrategy.OnPush,
                template: "<ng-container *mobxAutorun>\n  <ng-container *ngIf=\"shouldDisplay\">\n    <!-- Label -->\n    <span *ngIf=\"label\" class=\"label\">{{ label }}</span>\n    <lf-help-tooltip *ngIf=\"helpMessage\" [help]=\"helpMessage\"></lf-help-tooltip>\n\n    <!-- Required icon -->\n    <lf-icon\n      class=\"is-required\"\n      icon=\"asterisk\"\n      *ngIf=\"showIsRequired\"\n    ></lf-icon>\n\n    <!-- Pending validation -->\n    <div\n      class=\"status pending\"\n      *ngIf=\"relativeStorage.shouldShowPending()\"\n    ></div>\n    <!-- Rejected validation -->\n    <div\n      class=\"status rejected\"\n      *ngIf=\"relativeStorage.shouldShowRejected()\"\n      (click)=\"relativeStorage.recomputeStateProperties()\"\n    >\n      <lf-tooltip [help]=\"retryValidateText\">\n        <lf-icon icon=\"warning\"></lf-icon>\n      </lf-tooltip>\n    </div>\n\n    <!-- (Debug) value state -->\n    <lf-value-state [path]=\"path\"></lf-value-state>\n\n    <!-- Input group -->\n    <div\n      class=\"input-group input-group-sm\"\n      [class.is-warning]=\"relativeStorage.shouldShowWarning()\"\n      [class.is-invalid]=\"relativeStorage.shouldShowError()\"\n    >\n      <!-- Code -->\n      <div *ngIf=\"code\" class=\"input-group-prepend\">\n        <div class=\"input-group-text\">{{ code }}</div>\n      </div>\n\n      <!-- Input -->\n      <!--\n        FIXME: We're currently setting `disabled` to prevent the date-picker\n        from popping up, such should probably be fixed from the date-picker's\n        side which should look at whether `readonly` is set\n      -->\n      <input\n        #focusTarget\n        bsDaterangepicker\n        #drp=\"bsDaterangepicker\"\n        [bsConfig]=\"bsConfig\"\n        type=\"text\"\n        class=\"form-control date-input text-right\"\n        [class.is-warning]=\"relativeStorage.shouldShowWarning()\"\n        [class.is-invalid]=\"relativeStorage.shouldShowError()\"\n        [class.is-readonly]=\"isReadOnly\"\n        [name]=\"path\"\n        [bsValue]=\"_dummyValue\"\n        [placeholder]=\"placeholder\"\n        [readonly]=\"isReadOnly\"\n        [disabled]=\"isReadOnly\"\n        autocomplete=\"off\"\n        (bsValueChange)=\"_onDateRangePickerChange($event)\"\n        (focus)=\"_onFocus($event)\"\n        (blur)=\"_onBlur($event)\"\n      />\n\n      <!-- Date icon -->\n      <div class=\"input-group-append\" (click)=\"!isReadOnly && drp.toggle()\">\n        <div class=\"input-group-text\"><lf-icon icon=\"calendar\"></lf-icon></div>\n      </div>\n    </div>\n\n    <!-- Legend -->\n    <div *ngIf=\"legend\" class=\"legend\">{{ legend }}</div>\n  </ng-container>\n</ng-container>\n"
            }] }
];
__decorate([
    computed,
    __metadata("design:type", String),
    __metadata("design:paramtypes", [])
], DateRangeComponent.prototype, "dateFormat", null);
__decorate([
    computed,
    __metadata("design:type", String),
    __metadata("design:paramtypes", [])
], DateRangeComponent.prototype, "separator", null);
__decorate([
    computed,
    __metadata("design:type", String),
    __metadata("design:paramtypes", [])
], DateRangeComponent.prototype, "placeholder", null);
__decorate([
    computed,
    __metadata("design:type", Boolean),
    __metadata("design:paramtypes", [])
], DateRangeComponent.prototype, "showIsRequired", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], DateRangeComponent.prototype, "bsConfig", null);
__decorate([
    action,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Array]),
    __metadata("design:returntype", void 0)
], DateRangeComponent.prototype, "_onDateRangePickerChange", null);
if (false) {
    /** @type {?} */
    DateRangeComponent.prototype._dummyValue;
    /**
     * @type {?}
     * @protected
     */
    DateRangeComponent.prototype._settingValue;
    /**
     * @type {?}
     * @protected
     */
    DateRangeComponent.prototype._disposeDateRangeAutorun;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * LF date component.
 */
class DateComponent extends InlineValue {
    // I18n ======================================================================
    /**
     * Format of the date.
     * @return {?}
     */
    get dateFormat() {
        return this.translate(I18N_DATE_FORMAT_KEY) || 'YYYY-MM-DD';
    }
    /**
     * Placeholder of the `input` field.
     * @return {?}
     */
    get placeholder() {
        return this.translate(I18N_DATE_PLACEHOLDER_KEY) || '';
    }
    // ===========================================================================
    /**
     * Whether to show the "is required" icon on the component.
     * @return {?}
     */
    get showIsRequired() {
        /** @type {?} */
        const isRequired = this.relativeStorage.getStateProperty('.', 'isRequired');
        return !!isRequired;
    }
    /**
     * Config for the `BsDatepicker`.
     * @return {?}
     */
    get bsConfig() {
        return {
            containerClass: 'theme-default',
            dateInputFormat: this.dateFormat,
        };
    }
    /**
     * Action to run whenever the date `input` changes.
     * @param {?} date Date to set.
     * @return {?}
     */
    _onDatePickerChange(date) {
        // FIXME: This function is being called as soon as the component loads, thus
        // the comparison against the current date (which shouldn't be needed).
        date = !date || isNaN(+date) ? null : date;
        /** @type {?} */
        const currentDate = this.relativeStorage.get();
        if (!comparer.structural(currentDate, date)) {
            this.relativeStorage.set('.', date);
            this.relativeStorage.setDirty();
            // Clicking on the pop-up should count as "touching"
            this.relativeStorage.setTouched();
        }
    }
    /**
     * @protected
     * @return {?}
     */
    validatePath() {
        /** @type {?} */
        const schema = this.relativeStorage.schema();
        if (!isDateSchema(schema) || !isNullableSchema(schema)) {
            throw pathError(this.path, 'Schema must be of the "date" type and "nullable"');
        }
    }
}
DateComponent.decorators = [
    { type: Component, args: [{
                selector: 'lf-date',
                changeDetection: ChangeDetectionStrategy.OnPush,
                template: "<ng-container *mobxAutorun>\n  <ng-container *ngIf=\"shouldDisplay\">\n    <!-- Label -->\n    <span *ngIf=\"label\" class=\"label\">{{ label }}</span>\n    <lf-help-tooltip *ngIf=\"helpMessage\" [help]=\"helpMessage\"></lf-help-tooltip>\n\n    <!-- Required icon -->\n    <lf-icon\n      class=\"is-required\"\n      icon=\"asterisk\"\n      *ngIf=\"showIsRequired\"\n    ></lf-icon>\n\n    <!-- Pending validation -->\n    <div\n      class=\"status pending\"\n      *ngIf=\"relativeStorage.shouldShowPending()\"\n    ></div>\n    <!-- Rejected validation -->\n    <div\n      class=\"status rejected\"\n      *ngIf=\"relativeStorage.shouldShowRejected()\"\n      (click)=\"relativeStorage.recomputeStateProperties()\"\n    >\n      <lf-tooltip [help]=\"retryValidateText\">\n        <lf-icon icon=\"warning\"></lf-icon>\n      </lf-tooltip>\n    </div>\n\n    <!-- (Debug) value state -->\n    <lf-value-state [path]=\"path\"></lf-value-state>\n\n    <!-- Input group -->\n    <div\n      class=\"input-group input-group-sm\"\n      [class.is-warning]=\"relativeStorage.shouldShowWarning()\"\n      [class.is-invalid]=\"relativeStorage.shouldShowError()\"\n    >\n      <!-- Code -->\n      <div *ngIf=\"code\" class=\"input-group-prepend\">\n        <div class=\"input-group-text\">{{ code }}</div>\n      </div>\n\n      <!-- Input -->\n      <!--\n        FIXME: We're currently setting `disabled` to prevent the date-picker from\n        popping up, such should probably be fixed from the date-picker's side\n        which should look at whether `readonly` is set\n      -->\n      <input\n        #focusTarget\n        bsDatepicker\n        #dp=\"bsDatepicker\"\n        [bsConfig]=\"bsConfig\"\n        type=\"text\"\n        class=\"form-control date-input text-right\"\n        [class.is-warning]=\"relativeStorage.shouldShowWarning()\"\n        [class.is-invalid]=\"relativeStorage.shouldShowError()\"\n        [class.is-readonly]=\"isReadOnly\"\n        [name]=\"path\"\n        [bsValue]=\"relativeStorage.get()\"\n        [placeholder]=\"placeholder\"\n        [readonly]=\"isReadOnly\"\n        [disabled]=\"isReadOnly\"\n        autocomplete=\"off\"\n        (bsValueChange)=\"_onDatePickerChange($event)\"\n        (focus)=\"_onFocus($event)\"\n        (blur)=\"_onBlur($event)\"\n      />\n\n      <!-- Date icon -->\n      <div class=\"input-group-append\" (click)=\"!isReadOnly && dp.toggle()\">\n        <div class=\"input-group-text\"><lf-icon icon=\"calendar\"></lf-icon></div>\n      </div>\n    </div>\n\n    <!-- Legend -->\n    <div *ngIf=\"legend\" class=\"legend\">{{ legend }}</div>\n  </ng-container>\n</ng-container>\n"
            }] }
];
__decorate([
    computed,
    __metadata("design:type", String),
    __metadata("design:paramtypes", [])
], DateComponent.prototype, "dateFormat", null);
__decorate([
    computed,
    __metadata("design:type", String),
    __metadata("design:paramtypes", [])
], DateComponent.prototype, "placeholder", null);
__decorate([
    computed,
    __metadata("design:type", Boolean),
    __metadata("design:paramtypes", [])
], DateComponent.prototype, "showIsRequired", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], DateComponent.prototype, "bsConfig", null);
__decorate([
    action,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Date]),
    __metadata("design:returntype", void 0)
], DateComponent.prototype, "_onDatePickerChange", null);

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class DropdownItemComponent {
    /**
     * @param {?} _host
     */
    constructor(_host) {
        this.itemType = 'item';
        this.isActive = false;
        this.isDisabled = false;
        this._tagName = _host.nativeElement.tagName.toLowerCase();
    }
    // Classes for each type of dropdown item
    /**
     * @return {?}
     */
    get _hostClassDropdownItem() {
        return this.itemType === 'item';
    }
    /**
     * @return {?}
     */
    get _hostClassDropdownDivider() {
        return this.itemType === 'divider';
    }
    /**
     * @return {?}
     */
    get _hostClassDropdownItemText() {
        return this.itemType === 'text';
    }
    /**
     * @return {?}
     */
    get _hostClassDropdownHeader() {
        return this.itemType === 'header';
    }
    // Set disable (attribute for buttons, class for others)
    /**
     * @return {?}
     */
    get _hostClassDisabled() {
        return (this.itemType === 'item' && this._tagName !== 'button' && this.isDisabled);
    }
    /**
     * @return {?}
     */
    get _hostDisabled() {
        return this.itemType === 'item' &&
            this._tagName === 'button' &&
            this.isDisabled
            ? ''
            : undefined;
    }
}
DropdownItemComponent.decorators = [
    { type: Component, args: [{
                selector: 'lf-dropdown-item, [lf-dropdown-item]',
                changeDetection: ChangeDetectionStrategy.OnPush,
                template: '<ng-container *mobxAutorun><ng-content></ng-content></ng-container>'
            }] }
];
/** @nocollapse */
DropdownItemComponent.ctorParameters = () => [
    { type: ElementRef }
];
DropdownItemComponent.propDecorators = {
    itemType: [{ type: Input }],
    isActive: [{ type: Input }, { type: HostBinding, args: ['class.active',] }],
    isDisabled: [{ type: Input }],
    _hostClassDropdownItem: [{ type: HostBinding, args: ['class.dropdown-item',] }],
    _hostClassDropdownDivider: [{ type: HostBinding, args: ['class.dropdown-divider',] }],
    _hostClassDropdownItemText: [{ type: HostBinding, args: ['class.dropdown-item-text',] }],
    _hostClassDropdownHeader: [{ type: HostBinding, args: ['class.dropdown-header',] }],
    _hostClassDisabled: [{ type: HostBinding, args: ['class.disabled',] }],
    _hostDisabled: [{ type: HostBinding, args: ['attr.disabled',] }]
};
__decorate([
    observable$1,
    __metadata("design:type", String)
], DropdownItemComponent.prototype, "itemType", void 0);
__decorate([
    observable$1,
    __metadata("design:type", Object)
], DropdownItemComponent.prototype, "isActive", void 0);
__decorate([
    observable$1,
    __metadata("design:type", Object)
], DropdownItemComponent.prototype, "isDisabled", void 0);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], DropdownItemComponent.prototype, "_hostClassDropdownItem", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], DropdownItemComponent.prototype, "_hostClassDropdownDivider", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], DropdownItemComponent.prototype, "_hostClassDropdownItemText", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], DropdownItemComponent.prototype, "_hostClassDropdownHeader", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], DropdownItemComponent.prototype, "_hostClassDisabled", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], DropdownItemComponent.prototype, "_hostDisabled", null);
if (false) {
    /** @type {?} */
    DropdownItemComponent.prototype.itemType;
    /** @type {?} */
    DropdownItemComponent.prototype.isActive;
    /** @type {?} */
    DropdownItemComponent.prototype.isDisabled;
    /**
     * @type {?}
     * @protected
     */
    DropdownItemComponent.prototype._tagName;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class DropdownMenuComponent {
    constructor() {
        this.alignRight = false;
        this._hostClassDropdownMenu = true;
    }
}
DropdownMenuComponent.decorators = [
    { type: Component, args: [{
                selector: 'lf-dropdown-menu, [lf-dropdown-menu]',
                changeDetection: ChangeDetectionStrategy.OnPush,
                template: '<ng-container *mobxAutorun><ng-content></ng-content></ng-container>'
            }] }
];
DropdownMenuComponent.propDecorators = {
    alignRight: [{ type: Input }, { type: HostBinding, args: ['class.dropdown-menu-right',] }],
    _hostClassDropdownMenu: [{ type: HostBinding, args: ['class.dropdown-menu',] }]
};
__decorate([
    observable$1,
    __metadata("design:type", Object)
], DropdownMenuComponent.prototype, "alignRight", void 0);
if (false) {
    /** @type {?} */
    DropdownMenuComponent.prototype.alignRight;
    /** @type {?} */
    DropdownMenuComponent.prototype._hostClassDropdownMenu;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * LF dropdown component.
 */
class DropdownComponent extends Toggleable {
    constructor() {
        super(...arguments);
        this.position = 'down';
        this.offset = 0;
        this.flip = true;
        this.boundary = 'scrollParent';
        this.reference = 'toggle';
        this.display = 'dynamic';
        this.shown = new EventEmitter();
        this.hidden = new EventEmitter();
        this._hostClassBtnGroup = true;
    }
    // Angular lifecycle =========================================================
    /**
     * @return {?}
     */
    ngOnInit() {
        this.host = this.injector.get(ElementRef);
        super.ngOnInit();
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        this.createDropdown();
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.destroyDropdown();
        super.ngOnDestroy();
    }
    // ===========================================================================
    /**
     * @return {?}
     */
    get _hostClassDropleft() {
        return this.position === 'left';
    }
    /**
     * @return {?}
     */
    get _hostClassDropright() {
        return this.position === 'right';
    }
    /**
     * @return {?}
     */
    get _hostClassDropup() {
        return this.position === 'up';
    }
    /**
     * Creates a new dropdown and starts listening to events.
     * @private
     * @return {?}
     */
    createDropdown() {
        /** @type {?} */
        const target = this._toggle
            ? this._toggle.nativeElement
            : this.host.nativeElement.children[0];
        this.$toggle = $(target)
            .attr('data-toggle', 'dropdown')
            .attr('aria-haspopup', 'true')
            .attr('aria-expanded', 'false')
            .dropdown({
            offset: this.offset,
            flip: this.flip,
            boundary: this.boundary instanceof ElementRef
                ? this.boundary.nativeElement
                : this.boundary,
            reference: this.reference instanceof ElementRef
                ? this.reference.nativeElement
                : this.reference,
            display: this.display,
        });
        this.$host = $(this.host.nativeElement);
        // Make UI match state at startup
        if (this.isVisible) {
            this.$toggle.dropdown('toggle');
        }
        // There seems to be no way of updating these settings, so the most we can
        // do is destroy and reconstruct the modal whenever they change. TODO:
        // Figure out a better approach for this
        this.disposeOnOptionsChange = reaction((/**
         * @return {?}
         */
        () => [
            this.offset,
            this.flip,
            this.boundary,
            this.reference,
            this.display,
        ]), (/**
         * @return {?}
         */
        () => {
            this.destroyDropdown();
            this.createDropdown();
        }), { name: 'dropdownReactionOnOptionsChange' });
        // Set the component's state if the dropdown is shown without calling
        // `show()`
        this.$host.on('show.bs.dropdown', (/**
         * @return {?}
         */
        () => {
            this.isAnimating = true;
            if (!this.isVisible) {
                this.show();
            }
        }));
        // Set the component's state if the dropdown is hidden without calling
        // `hide()`
        this.$host.on('hide.bs.dropdown', (/**
         * @return {?}
         */
        () => {
            this.isAnimating = true;
            if (this.isVisible) {
                this.hide();
            }
        }));
        // Runs when the showing animation ends
        this.$host.on('shown.bs.dropdown', (/**
         * @return {?}
         */
        () => {
            this.isAnimating = false;
            this.shown.emit();
            // `isVisible` may change faster than the animations occur, force the UI
            // state to match `isVisible`
            if (!this.isVisible) {
                this.hide();
            }
        }));
        // Runs when the hiding animation ends
        this.$host.on('hidden.bs.dropdown', (/**
         * @return {?}
         */
        () => {
            this.isAnimating = false;
            this.hidden.emit();
            // `isVisible` may change faster than the animations occur, force the UI
            // state to match `isVisible`
            if (this.isVisible) {
                this.show();
            }
        }));
        // Open the dropdown when `show` is called
        this.showSubscription = this._show.subscribe((/**
         * @return {?}
         */
        () => {
            if (!this.isAnimating && !this.$host.hasClass('show')) {
                this.$toggle.dropdown('toggle');
            }
        }));
        // Close the dropdown when `hide` is called
        this.hideSubscription = this._hide.subscribe((/**
         * @return {?}
         */
        () => {
            if (!this.isAnimating && this.$host.hasClass('show')) {
                this.$toggle.dropdown('toggle');
            }
        }));
    }
    /**
     * Unsubscribes from all events and destroys the dropdown.
     * @private
     * @return {?}
     */
    destroyDropdown() {
        this.disposeOnOptionsChange && this.disposeOnOptionsChange();
        if (this.$host) {
            this.$host.off('show.bs.dropdown');
            this.$host.off('hide.bs.dropdown');
            this.$host.off('shown.bs.dropdown');
            this.$host.off('hidden.bs.dropdown');
        }
        this.showSubscription && this.showSubscription.unsubscribe();
        this.hideSubscription && this.hideSubscription.unsubscribe();
        this.$toggle &&
            this.$toggle
                .removeAttr('data-toggle')
                .removeAttr('aria-haspopup')
                .removeAttr('aria-expanded')
                .dropdown('dispose');
    }
}
DropdownComponent.decorators = [
    { type: Component, args: [{
                selector: 'lf-dropdown, [lf-dropdown]',
                changeDetection: ChangeDetectionStrategy.OnPush,
                providers: [{ provide: PathBased, useExisting: DropdownComponent }],
                template: '<ng-container *mobxAutorun><ng-content></ng-content></ng-container>'
            }] }
];
DropdownComponent.propDecorators = {
    position: [{ type: Input }],
    offset: [{ type: Input }],
    flip: [{ type: Input }],
    boundary: [{ type: Input }],
    reference: [{ type: Input }],
    display: [{ type: Input }],
    shown: [{ type: Output }],
    hidden: [{ type: Output }],
    _hostClassBtnGroup: [{ type: HostBinding, args: ['class.btn-group',] }],
    _toggle: [{ type: ContentChild, args: ['dropdownToggle', { static: false, read: ElementRef },] }],
    _hostClassDropleft: [{ type: HostBinding, args: ['class.dropleft',] }],
    _hostClassDropright: [{ type: HostBinding, args: ['class.dropright',] }],
    _hostClassDropup: [{ type: HostBinding, args: ['class.dropup',] }]
};
__decorate([
    observable$1,
    __metadata("design:type", String)
], DropdownComponent.prototype, "position", void 0);
__decorate([
    observable$1,
    __metadata("design:type", Object)
], DropdownComponent.prototype, "offset", void 0);
__decorate([
    observable$1,
    __metadata("design:type", Object)
], DropdownComponent.prototype, "flip", void 0);
__decorate([
    observable$1,
    __metadata("design:type", Object)
], DropdownComponent.prototype, "boundary", void 0);
__decorate([
    observable$1,
    __metadata("design:type", Object)
], DropdownComponent.prototype, "reference", void 0);
__decorate([
    observable$1,
    __metadata("design:type", String)
], DropdownComponent.prototype, "display", void 0);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], DropdownComponent.prototype, "_hostClassDropleft", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], DropdownComponent.prototype, "_hostClassDropright", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], DropdownComponent.prototype, "_hostClassDropup", null);
if (false) {
    /** @type {?} */
    DropdownComponent.prototype.position;
    /** @type {?} */
    DropdownComponent.prototype.offset;
    /** @type {?} */
    DropdownComponent.prototype.flip;
    /** @type {?} */
    DropdownComponent.prototype.boundary;
    /** @type {?} */
    DropdownComponent.prototype.reference;
    /** @type {?} */
    DropdownComponent.prototype.display;
    /** @type {?} */
    DropdownComponent.prototype.shown;
    /** @type {?} */
    DropdownComponent.prototype.hidden;
    /** @type {?} */
    DropdownComponent.prototype._hostClassBtnGroup;
    /** @type {?} */
    DropdownComponent.prototype._toggle;
    /**
     * @type {?}
     * @private
     */
    DropdownComponent.prototype.host;
    /**
     * @type {?}
     * @private
     */
    DropdownComponent.prototype.$host;
    /**
     * @type {?}
     * @private
     */
    DropdownComponent.prototype.$toggle;
    /**
     * @type {?}
     * @private
     */
    DropdownComponent.prototype.isAnimating;
    /**
     * @type {?}
     * @private
     */
    DropdownComponent.prototype.disposeOnOptionsChange;
    /**
     * @type {?}
     * @private
     */
    DropdownComponent.prototype.showSubscription;
    /**
     * @type {?}
     * @private
     */
    DropdownComponent.prototype.hideSubscription;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * LF modal component.
 */
class ModalComponent extends Toggleable {
    constructor() {
        super(...arguments);
        /**
         * Whether to center the modal vertically on screen.
         */
        this.isCentered = false;
        /**
         * Whether to animate the modal component.
         */
        this.animate = true;
        /**
         * Whether to enable closing the modal by pressing the `Esc` key.
         */
        this.closeWithEsc = true;
        /**
         * Whether to show a dark background underneath the modal when it appears.
         */
        this.showBackdrop = true;
        /**
         * Whether to disable closing the modal by clicking on the backdrop.
         */
        this.ignoreBackdropClick = false;
        /**
         * If the modal contains `<input>`, `<select>`, or `<textarea>` elements
         * within, focus the first found element after the modal opens.
         */
        this.autoFocus = false;
        /**
         * Event emitter which emits an event whenever the modal finishes showing.
         */
        this.shown = new EventEmitter();
        /**
         * Event emitter which emits an event whenever the modal finishes hiding.
         */
        this.hidden = new EventEmitter();
    }
    // Angular lifecycle =========================================================
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        this.createModal();
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.destroyModal();
        super.ngOnDestroy();
    }
    // ===========================================================================
    /**
     * Method to call on long modals when their content changes, which forces
     * their height to also change.
     * @return {?}
     */
    refresh() {
        this.$modal.modal('handleUpdate');
    }
    /**
     * @return {?}
     */
    show() {
        return new Promise((/**
         * @param {?} resolve
         * @return {?}
         */
        resolve => {
            super.show();
            if (this.isAnimating) {
                this.$modal.one('shown.bs.modal', resolve);
            }
            else {
                resolve();
            }
        }));
    }
    /**
     * @return {?}
     */
    hide() {
        return new Promise((/**
         * @param {?} resolve
         * @return {?}
         */
        resolve => {
            super.hide();
            if (this.isAnimating) {
                this.$modal.one('hidden.bs.modal', resolve);
            }
            else {
                resolve();
            }
        }));
    }
    /**
     * @return {?}
     */
    toggle() {
        return this.isVisible ? this.hide() : this.show();
    }
    /**
     * Creates a new modal and starts listening to events.
     * @private
     * @return {?}
     */
    createModal() {
        this.$modal = $(this._modal.nativeElement).modal({
            backdrop: this.showBackdrop && this.ignoreBackdropClick
                ? 'static'
                : this.showBackdrop,
            keyboard: this.closeWithEsc,
            show: this.isVisible,
        });
        // There seems to be no way of updating these settings, so the most we can
        // do is destroy and reconstruct the modal whenever they change. TODO:
        // Figure out a better approach for this, perhaps implement this
        // functionality ourselves
        this.disposeOnOptionsChange = reaction((/**
         * @return {?}
         */
        () => [this.closeWithEsc, this.showBackdrop, this.ignoreBackdropClick]), (/**
         * @return {?}
         */
        () => this.destroyModal().then((/**
         * @return {?}
         */
        () => this.createModal()))), { name: 'modalReactionOnOptionsChange' });
        // Set the component's state if the modal is shown unconventionally
        // (i.e. without calling `show()`)
        this.$modal.on('show.bs.modal', (/**
         * @return {?}
         */
        () => {
            this.isAnimating = true;
            if (!this.isVisible) {
                this.show();
            }
        }));
        // Set the component's state if the modal is hidden unconventionally
        // (i.e. without calling `hide()`), this happens, for instance, when
        // pressing `Esc` or when pressing the area outside the modal
        this.$modal.on('hide.bs.modal', (/**
         * @return {?}
         */
        () => {
            this.isAnimating = true;
            if (this.isVisible) {
                this.hide();
            }
        }));
        // Runs when the showing animation ends
        this.$modal.on('shown.bs.modal', (/**
         * @return {?}
         */
        () => {
            this.isAnimating = false;
            this.shown.emit();
            // `isVisible` may change faster than the animations occur, force the UI
            // state to match `isVisible`
            if (!this.isVisible) {
                this.hide();
            }
            // Focus the first input of the modal's body once it opens
            if (this.autoFocus) {
                /** @type {?} */
                const firstInput = $('.modal-body input, .modal-body select, .modal-body textarea', this.$modal).get(0);
                if (firstInput) {
                    firstInput.focus();
                }
            }
        }));
        // Runs when the hiding animation ends
        this.$modal.on('hidden.bs.modal', (/**
         * @return {?}
         */
        () => {
            this.isAnimating = false;
            this.hidden.emit();
            // `isVisible` may change faster than the animations occur, force the UI
            // state to match `isVisible`
            if (this.isVisible) {
                this.show();
            }
        }));
        // Open the modal when `show` is called
        this.onShowSubscription = this._show.subscribe((/**
         * @return {?}
         */
        () => {
            if (!this.isAnimating) {
                this.$modal.modal('show');
            }
        }));
        // Close the modal when `hide` is called
        this.onHideSubscription = this._hide.subscribe((/**
         * @return {?}
         */
        () => {
            if (!this.isAnimating) {
                this.$modal.modal('hide');
            }
        }));
    }
    /**
     * Unsubscribes from all events and destroys the modal (returns a promise that
     * resolves after the modal has been destroyed).
     * @private
     * @return {?} Promise that resolves when the modal has been destroyed.
     */
    // FIXME: This returns a promise because we are forced to hide the modal prior
    // to removing it; otherwise a leftover backdrop will stay within the document
    // (this is probably an upstream Bootstrap bug)
    destroyModal() {
        this.disposeOnOptionsChange && this.disposeOnOptionsChange();
        if (this.$modal) {
            this.$modal.off('show.bs.modal');
            this.$modal.off('hide.bs.modal');
            this.$modal.off('shown.bs.modal');
            this.$modal.off('hidden.bs.modal');
        }
        this.onShowSubscription && this.onShowSubscription.unsubscribe();
        this.onHideSubscription && this.onHideSubscription.unsubscribe();
        if (this.isVisible) {
            return new Promise((/**
             * @param {?} resolve
             * @return {?}
             */
            resolve => {
                // If we don't hide the modal prior to disposing of it, then a leftover
                // backdrop will stay in the document
                this.hide();
                this.$modal.one('hidden.bs.modal', (/**
                 * @return {?}
                 */
                () => {
                    $(this).modal('dispose');
                    resolve();
                }));
            }));
        }
        else {
            this.$modal.modal('dispose');
            return Promise.resolve();
        }
    }
}
ModalComponent.decorators = [
    { type: Component, args: [{
                selector: 'lf-modal',
                changeDetection: ChangeDetectionStrategy.OnPush,
                providers: [{ provide: PathBased, useExisting: ModalComponent }],
                template: "<ng-container *mobxAutorun>\n  <div\n    #modal\n    class=\"modal\"\n    [class.fade]=\"animate\"\n    tabindex=\"-1\"\n    role=\"dialog\"\n    aria-hidden=\"true\"\n  >\n    <div\n      class=\"modal-dialog {{ size ? 'modal-' + size : '' }}\"\n      [class.modal-dialog-centered]=\"isCentered\"\n    >\n      <div class=\"modal-content\" role=\"document\"><ng-content></ng-content></div>\n    </div>\n  </div>\n</ng-container>\n"
            }] }
];
ModalComponent.propDecorators = {
    size: [{ type: Input }],
    isCentered: [{ type: Input }],
    animate: [{ type: Input }],
    closeWithEsc: [{ type: Input }],
    showBackdrop: [{ type: Input }],
    ignoreBackdropClick: [{ type: Input }],
    autoFocus: [{ type: Input }],
    shown: [{ type: Output }],
    hidden: [{ type: Output }],
    _modal: [{ type: ViewChild, args: ['modal', { static: false },] }]
};
__decorate([
    observable$1,
    __metadata("design:type", String)
], ModalComponent.prototype, "size", void 0);
__decorate([
    observable$1,
    __metadata("design:type", Object)
], ModalComponent.prototype, "isCentered", void 0);
__decorate([
    observable$1,
    __metadata("design:type", Object)
], ModalComponent.prototype, "animate", void 0);
__decorate([
    observable$1,
    __metadata("design:type", Object)
], ModalComponent.prototype, "closeWithEsc", void 0);
__decorate([
    observable$1,
    __metadata("design:type", Object)
], ModalComponent.prototype, "showBackdrop", void 0);
__decorate([
    observable$1,
    __metadata("design:type", Object)
], ModalComponent.prototype, "ignoreBackdropClick", void 0);
__decorate([
    observable$1,
    __metadata("design:type", Object)
], ModalComponent.prototype, "autoFocus", void 0);
if (false) {
    /**
     * Size of the modal component. See available sizes at
     * [Bootstrap documentation](
     * https://getbootstrap.com/docs/4.0/components/modal/#optional-sizes).
     * @type {?}
     */
    ModalComponent.prototype.size;
    /**
     * Whether to center the modal vertically on screen.
     * @type {?}
     */
    ModalComponent.prototype.isCentered;
    /**
     * Whether to animate the modal component.
     * @type {?}
     */
    ModalComponent.prototype.animate;
    /**
     * Whether to enable closing the modal by pressing the `Esc` key.
     * @type {?}
     */
    ModalComponent.prototype.closeWithEsc;
    /**
     * Whether to show a dark background underneath the modal when it appears.
     * @type {?}
     */
    ModalComponent.prototype.showBackdrop;
    /**
     * Whether to disable closing the modal by clicking on the backdrop.
     * @type {?}
     */
    ModalComponent.prototype.ignoreBackdropClick;
    /**
     * If the modal contains `<input>`, `<select>`, or `<textarea>` elements
     * within, focus the first found element after the modal opens.
     * @type {?}
     */
    ModalComponent.prototype.autoFocus;
    /**
     * Event emitter which emits an event whenever the modal finishes showing.
     * @type {?}
     */
    ModalComponent.prototype.shown;
    /**
     * Event emitter which emits an event whenever the modal finishes hiding.
     * @type {?}
     */
    ModalComponent.prototype.hidden;
    /** @type {?} */
    ModalComponent.prototype._modal;
    /**
     * @type {?}
     * @private
     */
    ModalComponent.prototype.$modal;
    /**
     * @type {?}
     * @private
     */
    ModalComponent.prototype.isAnimating;
    /**
     * @type {?}
     * @private
     */
    ModalComponent.prototype.disposeOnOptionsChange;
    /**
     * @type {?}
     * @private
     */
    ModalComponent.prototype.onShowSubscription;
    /**
     * @type {?}
     * @private
     */
    ModalComponent.prototype.onHideSubscription;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * LF form component.
 */
class FormComponent extends Value {
    // Angular lifecycle =========================================================
    /**
     * @return {?}
     */
    ngOnInit() {
        super.ngOnInit();
    }
    // I18n ======================================================================
    /**
     * Text used for the "help" action.
     * @return {?}
     */
    get helpActionText() {
        return this.translate(I18N_FORM_HELP_ACTION_TEXT_KEY);
    }
    /**
     * Text used for the "remove" action.
     * @return {?}
     */
    get removeActionText() {
        return this.translate(I18N_FORM_REMOVE_ACTION_TEXT_KEY);
    }
    /**
     * Message displayed to the user to confirm removal of the form.
     * @return {?}
     */
    get ensureRemoveText() {
        return this.translate(I18N_FORM_ENSURE_REMOVE_TEXT_KEY);
    }
    /**
     * Text for the "cancel" button in the removal modal.
     * @return {?}
     */
    get cancelRemoveButtonText() {
        return this.translate(I18N_FORM_CANCEL_REMOVE_BUTTON_TEXT_KEY);
    }
    /**
     * Text for the "confirm" button in the removal modal.
     * @return {?}
     */
    get confirmRemoveButtonText() {
        return this.translate(I18N_FORM_CONFIRM_REMOVE_BUTTON_TEXT_KEY);
    }
    // ===========================================================================
    /**
     * Whether this form can be removed (if it is `nullable` or its parent is a
     * list or map).
     * @return {?}
     */
    get isRemovable() {
        return (this.relativeStorage.isNullable() || this.relativeStorage.isRemovable());
    }
    /**
     * Removes this form by either setting it to `null` (when it is `nullable` or
     * removing it from its parent list or map).
     * @return {?}
     */
    remove() {
        if (this.relativeStorage.isNullable()) {
            this.relativeStorage.set('.', null);
        }
        else {
            this.relativeStorage.remove();
        }
    }
    /**
     * Form-related actions shown on the top of the form.
     * @return {?}
     */
    get _formActions() {
        /** @type {?} */
        const actions = [];
        // Help action
        if (this.helpMessage !== undefined) {
            actions.push({
                text: this.helpActionText,
                class: 'help',
                style: 'outline-primary',
                icon: 'question-circle',
                callback: (/**
                 * @return {?}
                 */
                () => this.showHelpModal()),
            });
        }
        // Remove form action
        if (this.isRemovable) {
            actions.push({
                text: this.removeActionText,
                class: 'remove-form',
                style: 'outline-danger',
                icon: 'trash',
                callback: (/**
                 * @return {?}
                 */
                () => this.showRemoveModal()),
            });
        }
        return actions;
    }
    /**
     * @return {?}
     */
    _cancelRemove() {
        this._removeModal.hide();
    }
    /**
     * @return {?}
     */
    _confirmRemove() {
        return __awaiter(this, void 0, void 0, /** @this {!FormComponent} */ function* () {
            yield this._removeModal.hide();
            this.remove();
        });
    }
    /**
     * @private
     * @return {?}
     */
    showHelpModal() {
        this._helpModal.show();
    }
    /**
     * @private
     * @return {?}
     */
    showRemoveModal() {
        this._removeModal.show();
    }
}
FormComponent.decorators = [
    { type: Component, args: [{
                selector: 'lf-form',
                changeDetection: ChangeDetectionStrategy.OnPush,
                providers: [{ provide: PathBased, useExisting: FormComponent }],
                template: "<ng-container *mobxAutorun>\n  <form\n    *ngIf=\"relativeStorage.has() && relativeStorage.get() !== null\"\n    class=\"container\"\n    [name]=\"path\"\n    (submit)=\"$event.preventDefault()\"\n  >\n    <!-- Form title -->\n    <div class=\"form-title-container\">\n      <h1 class=\"form-title\">{{ label }}</h1>\n      <!-- Form actions -->\n      <lf-actions-menu\n        class=\"form-actions\"\n        [actions]=\"_formActions\"\n        buttonsSize=\"sm\"\n        [alignMenusRight]=\"true\"\n      >\n      </lf-actions-menu>\n    </div>\n\n    <!-- Form content -->\n    <ng-content></ng-content>\n  </form>\n\n  <!-- Help modal -->\n  <lf-modal #helpModal class=\"help-modal\" size=\"lg\" *ngIf=\"helpMessage\">\n    <lf-modal-header>{{ helpActionText }}</lf-modal-header>\n    <lf-modal-body [innerHTML]=\"helpMessage\"></lf-modal-body>\n  </lf-modal>\n\n  <!-- Modal to confirm form deletion -->\n  <lf-modal *ngIf=\"isRemovable\" #removeModal class=\"remove-modal\">\n    <lf-modal-header>{{ removeActionText }}</lf-modal-header>\n    <lf-modal-body>\n      <p>{{ ensureRemoveText }}</p>\n    </lf-modal-body>\n    <lf-modal-footer>\n      <button\n        lf-button\n        class=\"cancel-remove\"\n        [style]=\"'outline-secondary'\"\n        size=\"sm\"\n        (click)=\"_cancelRemove()\"\n      >\n        {{ cancelRemoveButtonText }}\n      </button>\n      <button\n        lf-button\n        class=\"confirm-remove\"\n        [style]=\"'outline-danger'\"\n        size=\"sm\"\n        (click)=\"_confirmRemove()\"\n      >\n        {{ confirmRemoveButtonText }}\n      </button>\n    </lf-modal-footer>\n  </lf-modal>\n</ng-container>\n"
            }] }
];
FormComponent.propDecorators = {
    _helpModal: [{ type: ViewChild, args: ['helpModal', { static: false },] }],
    _removeModal: [{ type: ViewChild, args: ['removeModal', { static: false },] }]
};
__decorate([
    computed,
    __metadata("design:type", String),
    __metadata("design:paramtypes", [])
], FormComponent.prototype, "helpActionText", null);
__decorate([
    computed,
    __metadata("design:type", String),
    __metadata("design:paramtypes", [])
], FormComponent.prototype, "removeActionText", null);
__decorate([
    computed,
    __metadata("design:type", String),
    __metadata("design:paramtypes", [])
], FormComponent.prototype, "ensureRemoveText", null);
__decorate([
    computed,
    __metadata("design:type", String),
    __metadata("design:paramtypes", [])
], FormComponent.prototype, "cancelRemoveButtonText", null);
__decorate([
    computed,
    __metadata("design:type", String),
    __metadata("design:paramtypes", [])
], FormComponent.prototype, "confirmRemoveButtonText", null);
__decorate([
    computed,
    __metadata("design:type", Boolean),
    __metadata("design:paramtypes", [])
], FormComponent.prototype, "isRemovable", null);
__decorate([
    action,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], FormComponent.prototype, "remove", null);
__decorate([
    computed,
    __metadata("design:type", Array),
    __metadata("design:paramtypes", [])
], FormComponent.prototype, "_formActions", null);
if (false) {
    /**
     * Modal used to show the form's help message.
     * @type {?}
     */
    FormComponent.prototype._helpModal;
    /**
     * Modal used to ask for confirmation when attempting to remove the form.
     * @type {?}
     */
    FormComponent.prototype._removeModal;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class HelpTooltipComponent {
}
HelpTooltipComponent.decorators = [
    { type: Component, args: [{
                selector: 'lf-help-tooltip',
                changeDetection: ChangeDetectionStrategy.OnPush,
                template: "<lf-tooltip [help]=\"help\" boundary=\"window\">\n  <lf-icon icon=\"question-circle\"></lf-icon>\n</lf-tooltip>\n"
            }] }
];
HelpTooltipComponent.propDecorators = {
    help: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    HelpTooltipComponent.prototype.help;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * LF icon component.
 */
class IconComponent {
    constructor() {
        this.fixedWidth = false;
        this.spin = false;
        this.pulse = false;
        this.ariaHidden = true;
    }
    /**
     * @return {?}
     */
    get classes() {
        /** @type {?} */
        const classes = {
            fa: true,
            ['fa-' + this.icon]: true,
            'fa-fw': this.fixedWidth,
            'fa-spin': this.spin,
            'fa-pulse': this.pulse,
        };
        if (this.size) {
            classes['fa-' + this.size] = true;
        }
        return classes;
    }
}
IconComponent.decorators = [
    { type: Component, args: [{
                selector: 'lf-icon',
                changeDetection: ChangeDetectionStrategy.OnPush,
                template: "<ng-container *mobxAutorun>\n  <i\n    *ngIf=\"icon\"\n    [ngClass]=\"classes\"\n    [attr.aria-hidden]=\"ariaHidden ? 'true' : 'false'\"\n  >\n  </i>\n</ng-container>\n"
            }] }
];
IconComponent.propDecorators = {
    icon: [{ type: Input }],
    size: [{ type: Input }],
    fixedWidth: [{ type: Input }],
    spin: [{ type: Input }],
    pulse: [{ type: Input }],
    ariaHidden: [{ type: Input }]
};
__decorate([
    observable$1,
    __metadata("design:type", String)
], IconComponent.prototype, "icon", void 0);
__decorate([
    observable$1,
    __metadata("design:type", String)
], IconComponent.prototype, "size", void 0);
__decorate([
    observable$1,
    __metadata("design:type", Object)
], IconComponent.prototype, "fixedWidth", void 0);
__decorate([
    observable$1,
    __metadata("design:type", Object)
], IconComponent.prototype, "spin", void 0);
__decorate([
    observable$1,
    __metadata("design:type", Object)
], IconComponent.prototype, "pulse", void 0);
__decorate([
    observable$1,
    __metadata("design:type", Object)
], IconComponent.prototype, "ariaHidden", void 0);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], IconComponent.prototype, "classes", null);
if (false) {
    /** @type {?} */
    IconComponent.prototype.icon;
    /** @type {?} */
    IconComponent.prototype.size;
    /** @type {?} */
    IconComponent.prototype.fixedWidth;
    /** @type {?} */
    IconComponent.prototype.spin;
    /** @type {?} */
    IconComponent.prototype.pulse;
    /** @type {?} */
    IconComponent.prototype.ariaHidden;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Component that represents the body of a Bootstrap modal.
 */
class ModalBodyComponent {
    constructor() {
        this._hostClassModalBody = true;
    }
}
ModalBodyComponent.decorators = [
    { type: Component, args: [{
                selector: 'lf-modal-body, [lf-modal-body]',
                template: '<ng-content></ng-content>'
            }] }
];
ModalBodyComponent.propDecorators = {
    _hostClassModalBody: [{ type: HostBinding, args: ['class.modal-body',] }]
};
if (false) {
    /** @type {?} */
    ModalBodyComponent.prototype._hostClassModalBody;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Component that represents the footer of a Bootstrap modal.
 */
class ModalFooterComponent {
    constructor() {
        this._hostClassModalFooter = true;
    }
}
ModalFooterComponent.decorators = [
    { type: Component, args: [{
                selector: 'lf-modal-footer, [lf-modal-footer]',
                template: '<ng-content></ng-content>'
            }] }
];
ModalFooterComponent.propDecorators = {
    _hostClassModalFooter: [{ type: HostBinding, args: ['class.modal-footer',] }]
};
if (false) {
    /** @type {?} */
    ModalFooterComponent.prototype._hostClassModalFooter;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Component that represents the header of a Bootstrap modal.
 */
class ModalHeaderComponent {
    /**
     * @param {?} modalComponent
     */
    constructor(modalComponent) {
        this.modalComponent = modalComponent;
        /**
         * Whether to display a button to close the modal.
         */
        this.showCloseButton = true;
        // The modal header must always have the `modal-header` class.
        this._hostClassModalHeader = true;
    }
    // I18n ======================================================================
    /**
     * Tooltip text used by the modal's close button.
     * @return {?}
     */
    get closeButtonTooltip() {
        return this.modalComponent.translate(I18N_CLOSE_BUTTON_TOOLTIP_KEY);
    }
    // ===========================================================================
    /**
     * Hides the modal.
     * @return {?}
     */
    hide() {
        this.modalComponent.hide();
    }
}
ModalHeaderComponent.decorators = [
    { type: Component, args: [{
                selector: 'lf-modal-header, [lf-modal-header]',
                changeDetection: ChangeDetectionStrategy.OnPush,
                template: "<ng-container *mobxAutorun>\n  <!-- Title -->\n  <h5 class=\"modal-title\"><ng-content></ng-content></h5>\n\n  <!-- Close button -->\n  <button\n    *ngIf=\"showCloseButton\"\n    type=\"button\"\n    class=\"close\"\n    [title]=\"closeButtonTooltip\"\n    [attr.aria-label]=\"closeButtonTooltip\"\n    (click)=\"hide()\"\n  >\n    <span aria-hidden=\"true\">&times;</span>\n  </button>\n</ng-container>\n"
            }] }
];
/** @nocollapse */
ModalHeaderComponent.ctorParameters = () => [
    { type: ModalComponent }
];
ModalHeaderComponent.propDecorators = {
    showCloseButton: [{ type: Input }],
    _hostClassModalHeader: [{ type: HostBinding, args: ['class.modal-header',] }]
};
__decorate([
    observable$1,
    __metadata("design:type", Object)
], ModalHeaderComponent.prototype, "showCloseButton", void 0);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], ModalHeaderComponent.prototype, "closeButtonTooltip", null);
if (false) {
    /**
     * Whether to display a button to close the modal.
     * @type {?}
     */
    ModalHeaderComponent.prototype.showCloseButton;
    /** @type {?} */
    ModalHeaderComponent.prototype._hostClassModalHeader;
    /**
     * @type {?}
     * @private
     */
    ModalHeaderComponent.prototype.modalComponent;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * LF number component.
 */
class NumberComponent extends InlineValue {
    constructor() {
        super(...arguments);
        /**
         * Number of decimal digits to show.
         */
        this.scale = 0;
        /**
         * Number of digits to show on the left side of the number.
         */
        this.leadingZeroes = 1;
        /**
         * Whether the number is currently being edited.
         */
        this._isEditing = false;
    }
    // I18n ======================================================================
    /**
     * Number's prefix.
     * @return {?}
     */
    get prefix() {
        return this.translate(I18N_NUMBER_PREFIX_KEY) || '';
    }
    /**
     * Number's suffix.
     * @return {?}
     */
    get suffix() {
        return this.translate(I18N_NUMBER_SUFFIX_KEY) || '';
    }
    /**
     * Number's decimal separator.
     * @return {?}
     */
    get decimalSeparator() {
        return this.translate(I18N_NUMBER_DECIMAL_SEPARATOR_KEY) || '.';
    }
    /**
     * Number's thousands separator.
     * @return {?}
     */
    get thousandsSeparator() {
        return this.translate(I18N_NUMBER_THOUSANDS_SEPARATOR_KEY) || '';
    }
    // ===========================================================================
    /**
     * Whether to show the "is required" icon on the component.
     * @return {?}
     */
    get showIsRequired() {
        /** @type {?} */
        const isRequired = this.relativeStorage.getStateProperty('.', 'isRequired');
        return !!isRequired;
    }
    /**
     * Whether the number being displayed represents an integer number.
     * @return {?}
     */
    get isInteger() {
        return !!this.relativeStorage.schema().isInteger;
    }
    /**
     * Whether the number is in "editing" mode.
     * @return {?}
     */
    get isEditing() {
        return this._isEditing;
    }
    // Begin "editing" mode.
    /**
     * @param {?} evt
     * @return {?}
     */
    _onFocus(evt) {
        if (!this.isReadOnly) {
            asap((/**
             * @return {?}
             */
            () => {
                /** @type {?} */
                const caret = this._getEditCaretPosition(evt.target.value, evt.target.selectionEnd);
                runInAction('setNumberAsEditing', (/**
                 * @return {?}
                 */
                () => (this._isEditing = true)));
                evt.target.value = this.formatValueForEdit(this.relativeStorage.get());
                evt.target.setSelectionRange(caret, caret, 'none');
            }));
        }
        super._onFocus(evt);
    }
    // Quit "editing" mode.
    /**
     * @param {?} evt
     * @return {?}
     */
    _onBlur(evt) {
        if (!this.isReadOnly) {
            /** @type {?} */
            let value = evt.target.value === ''
                ? null
                : // Support numbers written with specified decimal separator
                    Number(evt.target.value.replace(this.decimalSeparator, '.'));
            // Reject `NaN`
            value = value === value ? value : null;
            // Convert to integer when needed
            if (value !== null && this.isInteger) {
                value = Math.round(value * Math.pow(10, this.scale));
            }
            this.relativeStorage.set('.', value);
            this.relativeStorage.setDirty();
            this._isEditing = false;
            evt.target.value = this.formatValueForDisplay(this.relativeStorage.get());
        }
        super._onBlur(evt);
    }
    /**
     * Formats the value to be displayed.
     * @param {?} value Value to format.
     * @return {?} Formatted value.
     */
    formatValueForDisplay(value) {
        // Force `value` to be a string
        /** @type {?} */
        let stringValue;
        if (value == null || value !== value /*`NaN`*/) {
            return '';
        }
        else if (typeof value === 'number') {
            // Convert the number to an integer taking into consideration whether
            // numbers represent integers or floats
            stringValue = String(Math.round(value * Math.pow(10, (this.isInteger ? 0 : this.scale))));
        }
        /** @type {?} */
        const isNegative = stringValue[0] === '-';
        /** @type {?} */
        const digits = stringValue
            .replace(/\D+/g, '')
            .replace(/^0+([^0]|0$)/, '$1');
        if (digits === '') {
            return '';
        }
        // Build the formatted value
        /** @type {?} */
        const sign = isNegative ? '-' : '';
        /** @type {?} */
        let stringNumber;
        /** @type {?} */
        let intPart = this.scale > 0 ? digits.slice(0, -this.scale) : digits;
        if (intPart.length < this.leadingZeroes) {
            /** @type {?} */
            let zeroes = '';
            for (let i = 0; i < this.leadingZeroes - intPart.length; ++i) {
                zeroes += '0';
            }
            intPart = zeroes + intPart;
        }
        intPart = intPart.replace(/\B(?=(\d{3})+\b)/g, this.thousandsSeparator);
        if (this.scale > 0) {
            /** @type {?} */
            let zeroes = '';
            for (let i = 0; i < Math.max(this.scale - digits.length, 0); ++i) {
                zeroes += '0';
            }
            /** @type {?} */
            const decimalPart = zeroes + digits.slice(-this.scale);
            stringNumber = intPart + this.decimalSeparator + decimalPart;
        }
        else {
            stringNumber = intPart;
        }
        return sign + this.prefix + stringNumber + this.suffix;
    }
    // Formats the value so that it can be edited by the user
    /**
     * @param {?} value
     * @return {?}
     */
    formatValueForEdit(value) {
        if (value == null) {
            return '';
        }
        if (this.isInteger) {
            value = value / Math.pow(10, this.scale);
        }
        // Show specified decimal separator
        return `${value}`.replace('.', this.decimalSeparator);
    }
    // Position of the caret on the editable text when the user clicks on the
    // formatted text
    /**
     * @protected
     * @param {?} formattedValue
     * @param {?} caret
     * @return {?}
     */
    _getEditCaretPosition(formattedValue, caret) {
        /** @type {?} */
        const leftOfCaret = formattedValue.slice(0, caret);
        /** @type {?} */
        const minusSize = leftOfCaret[0] === '-' ? 1 : 0;
        /** @type {?} */
        const decimalSeparatorSize = leftOfCaret.indexOf(this.decimalSeparator) !== -1 ? 1 : 0;
        /** @type {?} */
        const digitsSize = leftOfCaret
            .replace(/\D+/g, '')
            .replace(/^0+([^0]|0$)/, '$1').length;
        return minusSize + decimalSeparatorSize + digitsSize;
    }
    /**
     * @protected
     * @return {?}
     */
    validatePath() {
        /** @type {?} */
        const schema = this.relativeStorage.schema();
        if (!isNumberSchema(schema) || !isNullableSchema(schema)) {
            throw pathError(this.path, 'Schema must be of the "number" type and "nullable"');
        }
    }
}
NumberComponent.decorators = [
    { type: Component, args: [{
                selector: 'lf-number',
                changeDetection: ChangeDetectionStrategy.OnPush,
                template: "<ng-container *mobxAutorun>\n  <ng-container *ngIf=\"shouldDisplay\">\n    <!-- Label -->\n    <span *ngIf=\"label\" class=\"label\">{{ label }}</span>\n    <lf-help-tooltip *ngIf=\"helpMessage\" [help]=\"helpMessage\"></lf-help-tooltip>\n\n    <!-- Required icon -->\n    <lf-icon\n      class=\"is-required\"\n      icon=\"asterisk\"\n      *ngIf=\"showIsRequired\"\n    ></lf-icon>\n\n    <!-- Pending validation -->\n    <div\n      class=\"status pending\"\n      *ngIf=\"relativeStorage.shouldShowPending()\"\n    ></div>\n    <!-- Rejected validation -->\n    <div\n      class=\"status rejected\"\n      *ngIf=\"relativeStorage.shouldShowRejected()\"\n      (click)=\"relativeStorage.recomputeStateProperties()\"\n    >\n      <lf-tooltip [help]=\"retryValidateText\">\n        <lf-icon icon=\"warning\"></lf-icon>\n      </lf-tooltip>\n    </div>\n\n    <!-- (Debug) value state -->\n    <lf-value-state [path]=\"path\"></lf-value-state>\n\n    <!-- Input group -->\n    <div\n      class=\"input-group input-group-sm\"\n      [class.is-warning]=\"relativeStorage.shouldShowWarning()\"\n      [class.is-invalid]=\"relativeStorage.shouldShowError()\"\n    >\n      <!-- Code -->\n      <div *ngIf=\"code\" class=\"input-group-prepend\">\n        <div class=\"input-group-text\">{{ code }}</div>\n      </div>\n\n      <!-- Input -->\n      <input\n        #focusTarget\n        type=\"text\"\n        class=\"form-control text-right\"\n        [class.is-warning]=\"relativeStorage.shouldShowWarning()\"\n        [class.is-invalid]=\"relativeStorage.shouldShowError()\"\n        [class.is-readonly]=\"isReadOnly\"\n        [name]=\"path\"\n        [value]=\"formatValueForDisplay(relativeStorage.get())\"\n        [readonly]=\"isReadOnly\"\n        autocomplete=\"off\"\n        (focus)=\"_onFocus($event)\"\n        (blur)=\"_onBlur($event)\"\n      />\n    </div>\n\n    <!-- Legend -->\n    <div *ngIf=\"legend\" class=\"legend\">{{ legend }}</div>\n  </ng-container>\n</ng-container>\n"
            }] }
];
NumberComponent.propDecorators = {
    scale: [{ type: Input }],
    leadingZeroes: [{ type: Input }]
};
__decorate([
    observable$1,
    __metadata("design:type", Object)
], NumberComponent.prototype, "scale", void 0);
__decorate([
    observable$1,
    __metadata("design:type", Object)
], NumberComponent.prototype, "leadingZeroes", void 0);
__decorate([
    observable$1,
    __metadata("design:type", Object)
], NumberComponent.prototype, "_isEditing", void 0);
__decorate([
    computed,
    __metadata("design:type", String),
    __metadata("design:paramtypes", [])
], NumberComponent.prototype, "prefix", null);
__decorate([
    computed,
    __metadata("design:type", String),
    __metadata("design:paramtypes", [])
], NumberComponent.prototype, "suffix", null);
__decorate([
    computed,
    __metadata("design:type", String),
    __metadata("design:paramtypes", [])
], NumberComponent.prototype, "decimalSeparator", null);
__decorate([
    computed,
    __metadata("design:type", String),
    __metadata("design:paramtypes", [])
], NumberComponent.prototype, "thousandsSeparator", null);
__decorate([
    computed,
    __metadata("design:type", Boolean),
    __metadata("design:paramtypes", [])
], NumberComponent.prototype, "showIsRequired", null);
__decorate([
    computed,
    __metadata("design:type", Boolean),
    __metadata("design:paramtypes", [])
], NumberComponent.prototype, "isInteger", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], NumberComponent.prototype, "isEditing", null);
__decorate([
    action,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], NumberComponent.prototype, "_onFocus", null);
__decorate([
    action,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], NumberComponent.prototype, "_onBlur", null);
if (false) {
    /**
     * Number of decimal digits to show.
     * @type {?}
     */
    NumberComponent.prototype.scale;
    /**
     * Number of digits to show on the left side of the number.
     * @type {?}
     */
    NumberComponent.prototype.leadingZeroes;
    /**
     * Whether the number is currently being edited.
     * @type {?}
     * @protected
     */
    NumberComponent.prototype._isEditing;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Abstract class used to represent components where a single option is selected
 * from a set of options.
 * @abstract
 */
class SingleSelectionValue extends SelectionValue {
    /**
     * Whether to show the "is required" icon on the component.
     * @return {?}
     */
    get showIsRequired() {
        if (!this.relativeStorage.isNullable()) {
            return false;
        }
        /** @type {?} */
        const isRequired = this.relativeStorage.getStateProperty('.', 'isRequired');
        return !!isRequired;
    }
    /**
     * Index of the selected option; `-1` when none of the available options is
     * selected.
     * @return {?}
     */
    // Not `computed` since `options` cannot be accessed within a `computed` value
    get selectedOptionIndex() {
        /** @type {?} */
        const val = this.relativeStorage.get();
        return this.options.findIndex((/**
         * @param {?} opt
         * @return {?}
         */
        opt => comparer.structural(val, opt.value)));
    }
}
__decorate([
    computed,
    __metadata("design:type", Boolean),
    __metadata("design:paramtypes", [])
], SingleSelectionValue.prototype, "showIsRequired", null);

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Component used for selecting from a list of options by using inputs of type
 * `radio`.
 */
class RadioComponent extends SingleSelectionValue {
    constructor() {
        super(...arguments);
        /**
         * Whether the list of radio inputs should be displayed inline. The component
         * will still display in "block" mode in small screens.
         */
        this.displayInline = false;
    }
    /**
     * Action to run whenever the value of the `radio` input changes.
     * @param {?} evt `change` event.
     * @return {?}
     */
    _onRadioChange(evt) {
        this.relativeStorage.set('.', this.options[evt.target.value].value);
        this.relativeStorage.setDirty();
    }
}
RadioComponent.decorators = [
    { type: Component, args: [{
                selector: 'lf-radio',
                changeDetection: ChangeDetectionStrategy.OnPush,
                template: "<ng-container *mobxAutorun>\n  <ng-container *ngIf=\"shouldDisplay\">\n    <!-- Label -->\n    <span *ngIf=\"label\" class=\"label\">{{ label }}</span>\n    <lf-help-tooltip *ngIf=\"helpMessage\" [help]=\"helpMessage\"></lf-help-tooltip>\n\n    <!-- Required icon -->\n    <lf-icon\n      class=\"is-required\"\n      icon=\"asterisk\"\n      *ngIf=\"showIsRequired\"\n    ></lf-icon>\n\n    <!-- Pending validation -->\n    <div\n      class=\"status pending\"\n      *ngIf=\"relativeStorage.shouldShowPending()\"\n    ></div>\n    <!-- Rejected validation -->\n    <div\n      class=\"status rejected\"\n      *ngIf=\"relativeStorage.shouldShowRejected()\"\n      (click)=\"relativeStorage.recomputeStateProperties()\"\n    >\n      <lf-tooltip [help]=\"retryValidateText\">\n        <lf-icon icon=\"warning\"></lf-icon>\n      </lf-tooltip>\n    </div>\n\n    <!-- (Debug) value state -->\n    <lf-value-state [path]=\"path\"></lf-value-state>\n\n    <!-- Input group -->\n    <div\n      class=\"lf-checkable-group input-group input-group-sm\"\n      [class.is-warning]=\"relativeStorage.shouldShowWarning()\"\n      [class.is-invalid]=\"relativeStorage.shouldShowError()\"\n    >\n      <!-- Code -->\n      <div *ngIf=\"code\" class=\"input-group-prepend\">\n        <div class=\"input-group-text\">{{ code }}</div>\n      </div>\n\n      <!-- Radios -->\n      <div\n        class=\"lf-checkables form-control\"\n        [class.is-warning]=\"relativeStorage.shouldShowWarning()\"\n        [class.is-invalid]=\"relativeStorage.shouldShowError()\"\n        [class.is-readonly]=\"isReadOnly\"\n        [class.display-inline]=\"displayInline\"\n      >\n        <!-- Text for when there are no options -->\n        <div\n          *ngIf=\"optionsStatus !== 'pending' && options.length === 0\"\n          class=\"no-options\"\n        >\n          {{ noOptionsText }}\n        </div>\n\n        <!-- Pending options -->\n        <div *ngIf=\"optionsStatus === 'pending'\" class=\"pending-options\"></div>\n\n        <div\n          class=\"lf-checkable input-group input-group-sm\"\n          *ngFor=\"let option of options; let i = index\"\n        >\n          <!-- Code of each radio -->\n          <div *ngIf=\"option.code\" class=\"input-group-prepend\">\n            <div class=\"input-group-text\">{{ option.code }}</div>\n          </div>\n\n          <!-- Radio -->\n          <label\n            class=\"form-control custom-control custom-radio\"\n            [class.is-warning]=\"relativeStorage.shouldShowWarning()\"\n            [class.is-invalid]=\"relativeStorage.shouldShowError()\"\n            [class.is-readonly]=\"isReadOnly\"\n          >\n            <!-- Actual radio -->\n            <!--\n              Note that radios don't support `readonly`, so we use the\n              `disabled` attribute\n            -->\n            <input\n              #focusTarget\n              type=\"radio\"\n              class=\"custom-control-input\"\n              [name]=\"path + '[' + i + ']'\"\n              [value]=\"i\"\n              [checked]=\"selectedOptionIndex === i\"\n              [disabled]=\"isReadOnly\"\n              (change)=\"_onRadioChange($event)\"\n              (focus)=\"_onFocus($event)\"\n              (blur)=\"_onBlur($event)\"\n            />\n\n            <!-- Radio label -->\n            <span class=\"custom-control-label\">\n              <span>{{ option.label }}</span>\n              <lf-help-tooltip\n                *ngIf=\"option.helpMessage\"\n                [help]=\"option.helpMessage\"\n              ></lf-help-tooltip>\n            </span>\n          </label>\n        </div>\n      </div>\n    </div>\n\n    <!-- Legend -->\n    <div *ngIf=\"legend\" class=\"legend\">{{ legend }}</div>\n  </ng-container>\n</ng-container>\n"
            }] }
];
RadioComponent.propDecorators = {
    displayInline: [{ type: Input }]
};
__decorate([
    observable$1,
    __metadata("design:type", Object)
], RadioComponent.prototype, "displayInline", void 0);
__decorate([
    action,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], RadioComponent.prototype, "_onRadioChange", null);
if (false) {
    /**
     * Whether the list of radio inputs should be displayed inline. The component
     * will still display in "block" mode in small screens.
     * @type {?}
     */
    RadioComponent.prototype.displayInline;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Component used for selecting from a list of options in a dropdown `<select>`
 * menu.
 */
class SelectComponent extends SingleSelectionValue {
    /**
     * Action to run whenever the `<select>` values changes.
     * @param {?} evt `change` event.
     * @return {?}
     */
    _onSelectChange(evt) {
        this.relativeStorage.set('.', this.options[+evt.target.value].value);
        this.relativeStorage.setDirty();
    }
}
SelectComponent.decorators = [
    { type: Component, args: [{
                selector: 'lf-select',
                changeDetection: ChangeDetectionStrategy.OnPush,
                template: "<ng-container *mobxAutorun>\n  <ng-container *ngIf=\"shouldDisplay\">\n    <!-- Label -->\n    <span *ngIf=\"label\" class=\"label\">{{ label }}</span>\n    <lf-help-tooltip *ngIf=\"helpMessage\" [help]=\"helpMessage\"></lf-help-tooltip>\n\n    <!-- Required icon -->\n    <lf-icon\n      class=\"is-required\"\n      icon=\"asterisk\"\n      *ngIf=\"showIsRequired\"\n    ></lf-icon>\n\n    <!-- Pending validation -->\n    <div\n      class=\"status pending\"\n      *ngIf=\"relativeStorage.shouldShowPending() || optionsStatus === 'pending'\"\n    ></div>\n    <!-- Rejected validation -->\n    <div\n      class=\"status rejected\"\n      *ngIf=\"\n        relativeStorage.shouldShowRejected() || optionsStatus === 'rejected'\n      \"\n      (click)=\"relativeStorage.recomputeStateProperties()\"\n    >\n      <lf-tooltip [help]=\"retryValidateText\">\n        <lf-icon icon=\"warning\"></lf-icon>\n      </lf-tooltip>\n    </div>\n\n    <!-- (Debug) value state -->\n    <lf-value-state [path]=\"path\"></lf-value-state>\n\n    <!-- Input group -->\n    <div\n      class=\"input-group input-group-sm\"\n      [class.is-warning]=\"relativeStorage.shouldShowWarning()\"\n      [class.is-invalid]=\"relativeStorage.shouldShowError()\"\n    >\n      <!-- Code -->\n      <div *ngIf=\"code\" class=\"input-group-prepend\">\n        <div class=\"input-group-text\">{{ code }}</div>\n      </div>\n\n      <!-- Select element -->\n      <!--\n        Note that selects don't support `readonly`, so we use the `disabled`\n        attribute; we add an `is-readonly` class to style it differently\n      -->\n      <select\n        #focusTarget\n        class=\"form-control form-control-sm custom-select\"\n        [class.is-warning]=\"relativeStorage.shouldShowWarning()\"\n        [class.is-invalid]=\"relativeStorage.shouldShowError()\"\n        [class.is-readonly]=\"isReadOnly\"\n        [name]=\"path\"\n        [disabled]=\"isReadOnly\"\n        (change)=\"_onSelectChange($event)\"\n        (focus)=\"_onFocus($event)\"\n        (blur)=\"_onBlur($event)\"\n      >\n        <!-- Default option -->\n        <option\n          [selected]=\"selectedOptionIndex === -1\"\n          disabled\n          hidden\n        ></option>\n\n        <!-- Disabled option for when there are no options -->\n        <option *ngIf=\"options.length === 0\" disabled>\n          {{ noOptionsText }}\n        </option>\n\n        <!-- User defined options -->\n        <option\n          *ngFor=\"let option of options; let i = index\"\n          [value]=\"i\"\n          [selected]=\"selectedOptionIndex === i\"\n          [disabled]=\"option.isDisabled\"\n        >\n          <!-- Option code -->\n          <ng-container *ngIf=\"option.code\">{{ option.code }}.</ng-container>\n\n          <!-- Option label -->\n          {{ option.label }}\n        </option>\n      </select>\n    </div>\n\n    <!-- Legend -->\n    <div *ngIf=\"legend\" class=\"legend\">{{ legend }}</div>\n  </ng-container>\n</ng-container>\n"
            }] }
];
__decorate([
    action,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], SelectComponent.prototype, "_onSelectChange", null);

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Width of the selection column (in `px`).
 * @type {?}
 */
const SELECTION_COLUMN_WIDTH = 25.6;
/**
 * LF table component.
 *
 * Example usage:
 * ```html
 * <lf-table path="/path/to/table">
 *   <lf-table-header>
 *     <lf-table-column id="field1"></lf-table-column>
 *     <lf-table-column id="field2"></lf-table-column>
 *   </lf-table-header>
 *
 *   <ng-template #lfTableRowTemplate let-id="id">
 *     <tr lf-table-row [path]="id">
 *       <td lf-table-cell><lf-text path="field1"></lf-text></td>
 *       <td lf-table-cell><lf-number path="field2"></lf-number></td>
 *     </tr>
 *   </ng-template>
 * </lf-table>
 * ```
 */
class TableComponent extends InlineValue {
    constructor() {
        super(...arguments);
        /**
         * Height (in `px`) of a single row (virtual scroll settings). All rows must
         * have the same height.
         */
        this.rowHeight = 42;
        /**
         * Maximum number of visible rows at any given time.
         */
        this.maxVisibleRows = 10;
        /**
         * Size of the virtual scroll's template cache
         * (`cdkVirtualForTemplateCacheSize` input of Angular CDK's virtual for
         * directive).
         */
        this.virtualScrollTemplateCacheSize = 20;
        /**
         * Whether to allow addition of new rows.
         */
        this.allowRowAddition = true;
        /**
         * Whether to allow removal of rows.
         */
        this.allowRowRemoval = true;
        // Selected rows
        this.selectedRows = new Map();
        // Identifier of the row that was last selected/unselected; used to implement
        // selections while pressing shift (behaviour similar to how Gmail operates)
        this.lastSelectedId = null;
        // Position of the scrollable element after the latest scroll
        this.lastScrollPosition = [0, 0];
    }
    // Angular lifecycle =========================================================
    /**
     * @return {?}
     */
    ngOnInit() {
        this.scrollbarUtils = this.injector.get(ScrollbarUtils);
        super.ngOnInit();
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        super.ngAfterViewInit();
        // React to scrolling on the scrollable body. Adapted from:
        // https://github.com/angular/components/blob/master/src/cdk/scrolling/virtual-scroll-viewport.ts
        /** @type {?} */
        const scrollableBodyEl = this._scrollableBody.elementRef.nativeElement;
        /** @type {?} */
        const scrollableHeaderEl = this._scrollableHeader.nativeElement;
        this.scrollSubscription = this._scrollableBody
            .elementScrolled()
            .pipe(auditTime(0, typeof requestAnimationFrame !== 'undefined'
            ? animationFrameScheduler
            : asapScheduler))
            .subscribe((/**
         * @return {?}
         */
        () => {
            // Save last scroll position to restore scroll position when needed
            this.lastScrollPosition = [
                scrollableBodyEl.scrollTop,
                scrollableBodyEl.scrollLeft,
            ];
            // Sync header scroll with body scroll
            scrollableHeaderEl.scrollLeft = scrollableBodyEl.scrollLeft;
        }));
        // Set the table's scroll position whenever the path changes (and when the
        // component is first initialised)
        this.disposeObservePath = observe(this, 'path', (/**
         * @param {?} __0
         * @return {?}
         */
        ({ oldValue, newValue }) => {
            // Save old scroll value
            if (oldValue) {
                this.lfStorage.setStateProperty(oldValue, this.scrollPositionStatePropName(oldValue), this.lastScrollPosition);
            }
            // Use previous scroll values
            /** @type {?} */
            const savedScrollPosition = this.lfStorage.getStateProperty(newValue, this.scrollPositionStatePropName(newValue)) || [0, 0];
            this.lastScrollPosition = savedScrollPosition;
            asap((/**
             * @return {?}
             */
            () => this._scrollableBody.scrollTo({
                top: savedScrollPosition[0],
                left: savedScrollPosition[1],
            })));
        }), true);
        // Remove "selected rows" of non-existent rows (because they were removed
        // by an outside operation, such as loading from a file)
        this.disposeObserveValue = observe(this, '_value', (/**
         * @return {?}
         */
        () => {
            for (const id of Array.from(this.selectedRows.keys())) {
                if (!this.relativeStorage.has(`${id}`)) {
                    this.selectedRows.delete(id);
                }
            }
        }));
        // Refresh the virtual scroll viewport whenever its height changes
        this.disposeObserveBodyHeight = observe(this, '_scrollableBodyHeight', (/**
         * @return {?}
         */
        () => {
            this._scrollableBody.checkViewportSize();
            this.checkHasHorizontalOverflow();
        }));
        // Update the status of the horizontal overflow whenever the window viewport
        // size changes
        this.viewportSubscription = this.injector
            .get(ViewportRuler)
            .change()
            .subscribe((/**
         * @return {?}
         */
        () => {
            this.checkHasHorizontalOverflow();
        }));
        asap((/**
         * @return {?}
         */
        () => this.checkHasHorizontalOverflow())); // Update on start-up
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        super.ngOnDestroy();
        // Save scroll value
        this.lfStorage.setStateProperty(this.path, this.scrollPositionStatePropName(this.path), this.lastScrollPosition);
        this.scrollSubscription && this.scrollSubscription.unsubscribe();
        this.disposeObservePath && this.disposeObservePath();
        this.disposeObserveValue && this.disposeObserveValue();
        this.disposeObserveBodyHeight && this.disposeObserveBodyHeight();
        this.viewportSubscription && this.viewportSubscription.unsubscribe();
    }
    // I18n ======================================================================
    /**
     * @return {?}
     */
    get _addRowText() {
        return this.translate(I18N_TABLE_ADD_ROW_ACTION_TEXT_KEY);
    }
    /**
     * @return {?}
     */
    get _removeRowsText() {
        return this.translate(I18N_TABLE_REMOVE_ROWS_ACTION_TEXT_KEY);
    }
    /**
     * @return {?}
     */
    get _noRowsText() {
        return this.translate(I18N_TABLE_NO_ROWS_TEXT_KEY);
    }
    /**
     * @return {?}
     */
    get _columnLabels() {
        return this.translate(I18N_TABLE_COLUMN_LABELS_KEY);
    }
    /**
     * @param {?} columnId
     * @return {?}
     */
    _columnLabel(columnId) {
        return this._columnLabels && this._columnLabels[columnId];
    }
    // ===========================================================================
    /**
     * Schema of this table's value.
     * @return {?}
     */
    get _schema() {
        return (/** @type {?} */ (this.relativeStorage.schema()));
    }
    /**
     * Table's value.
     * @return {?}
     */
    get _value() {
        return this.relativeStorage.get().slice();
    }
    /**
     * Whether to show the "is required" icon on the component.
     * @return {?}
     */
    get _showIsRequired() {
        /** @type {?} */
        const minSize = this.relativeStorage.getStateProperty('.', 'minSize');
        return minSize !== undefined && minSize > 0;
    }
    /**
     * Table actions.
     * @return {?}
     */
    get _tableActions() {
        /** @type {?} */
        const actions = [];
        // Remove rows action
        if (this.allowsRowRemoval && !this.noRowsSelected) {
            actions.push({
                text: this._removeRowsText,
                class: 'remove-rows',
                style: 'outline-danger',
                icon: 'trash',
                callback: (/**
                 * @return {?}
                 */
                () => this._onClickRemoveRows()),
            });
        }
        // New row action
        /** @type {?} */
        const maxSize = this.relativeStorage.getStateProperty('.', 'maxSize');
        if (this.allowsRowAddition &&
            (maxSize == null || this._value.length < maxSize)) {
            actions.push({
                text: this._addRowText,
                class: 'add-row',
                style: 'outline-primary',
                icon: 'plus-circle',
                callback: (/**
                 * @return {?}
                 */
                () => this._onClickNewRow()),
            });
        }
        return actions;
    }
    /**
     * Height (in px) that the scrollable body element should have.
     * @return {?}
     */
    get _scrollableBodyHeight() {
        return (Math.min(this._value.length, this.maxVisibleRows) * this.rowHeight +
            this.horizontalScrollbarHeight);
    }
    /**
     * Minimum width (in px) that the table should have.
     * @param {?} includeScrollbarColumn Whether the scrollbar column should be
     * included in the computation (should be set to `true` for the header table).
     * @return {?} Minimum width that the table should have.
     */
    _tableMinWidth(includeScrollbarColumn) {
        return (this._tableHeader.tableMinWidth +
            (this.allowsRowRemoval ? SELECTION_COLUMN_WIDTH : 0) +
            (includeScrollbarColumn ? this.verticalScrollbarWidth : 0));
    }
    /**
     * Maximum width (in px) that the table should have.
     * @param {?} includeScrollbarColumn Whether the scrollbar column should be
     * included in the computation (should be set to `true` for the header table).
     * @return {?} Maximum width that the table should have.
     */
    _tableMaxWidth(includeScrollbarColumn) {
        return (this._tableHeader.tableMaxWidth +
            (this.allowsRowRemoval ? SELECTION_COLUMN_WIDTH : 0) +
            (includeScrollbarColumn ? this.verticalScrollbarWidth : 0));
    }
    /**
     * Widths (as strings with correct units) of each column of the table.
     * @param {?} includeScrollbarColumn Whether the scrollbar column should be
     * included (should be set to `true` for the header table).
     * @return {?} Widths of each column of the table.
     */
    _columnWidths(includeScrollbarColumn) {
        return (this.allowsRowRemoval
            ? [SELECTION_COLUMN_WIDTH + 'px']
            : []).concat(this._tableHeader.columnWidths.concat(includeScrollbarColumn ? [this.verticalScrollbarWidth + 'px'] : []));
    }
    /**
     * Whether new rows can be added to the table (takes into consideration if the
     * table is in "read only" mode).
     * @return {?}
     */
    get allowsRowAddition() {
        return !this.isReadOnly && this.allowRowAddition;
    }
    /**
     * Whether rows can be removed from the table (takes into consideration if the
     * table is in "read only" mode).
     * @return {?}
     */
    get allowsRowRemoval() {
        return !this.isReadOnly && this.allowRowRemoval;
    }
    /**
     * Number of rows that have been selected.
     * @return {?}
     */
    get numberOfRowsSelected() {
        return this.selectedRows.size;
    }
    /**
     * Whether all rows are selected.
     * @return {?}
     */
    get allRowsSelected() {
        return this.selectedRows.size === this._value.length;
    }
    /**
     * Whether no rows are selected.
     * @return {?}
     */
    get noRowsSelected() {
        return this.selectedRows.size === 0;
    }
    /**
     * Adds a new row to the table.
     * @return {?}
     */
    addNewRow() {
        this.relativeStorage.push('.', undefined);
        // Scroll to bottom (shows newly inserted row)
        asap((/**
         * @return {?}
         */
        () => this.scrollToIndex(this._value.length - 1)));
    }
    /**
     * Gets the selection status of the row with the provided id.
     * @param {?} id Id of the row for which to check selection status.
     * @return {?} Whether the row with the provided id is selected.
     */
    getRowSelectedStatus(id) {
        return this.allowsRowRemoval && this.selectedRows.has(id);
    }
    /**
     * Sets the selection status of the row with the provided id.
     * @param {?} id Id of the row on which to set selection status.
     * @param {?} selectedStatus `true` to set selected, `false` to set unselected.
     * @return {?}
     */
    setRowSelectedStatus(id, selectedStatus) {
        if (!this.relativeStorage.has(`${id}`)) {
            throw pathError(this.path, `Invalid id: ${id}`);
        }
        if (selectedStatus) {
            this.selectedRows.set(id, true);
        }
        else {
            this.selectedRows.delete(id);
        }
        this.lastSelectedId = id;
    }
    /**
     * Sets the selection status of all rows from the last row that has had its
     * selection status set to the row with the provided id.
     * @param {?} id Id of the row until which to set selection status.
     * @param {?} selectedStatus `true` to set selected, `false` to set unselected.
     * @return {?}
     */
    setRowSelectedStatusFromLastSelected(id, selectedStatus) {
        if (!this.relativeStorage.has(`${id}`)) {
            throw pathError(this.path, `Invalid id: ${id}`);
        }
        /** @type {?} */
        const lastSelectedId = this.lastSelectedId === null ||
            !this.relativeStorage.has(`${this.lastSelectedId}`)
            ? id
            : this.lastSelectedId;
        /** @type {?} */
        let idx;
        /** @type {?} */
        let lastSelectedIdx;
        /** @type {?} */
        const isTable = isTableSchema(this._schema);
        if (isTable) {
            idx = this._value.findIndex((/**
             * @param {?} row
             * @return {?}
             */
            row => getRowId(row) === id));
            lastSelectedIdx =
                id === lastSelectedId
                    ? idx
                    : this._value.findIndex((/**
                     * @param {?} row
                     * @return {?}
                     */
                    row => getRowId(row) === lastSelectedId));
        }
        else {
            idx = (/** @type {?} */ (id));
            lastSelectedIdx = (/** @type {?} */ (lastSelectedId));
        }
        for (let i = Math.min(lastSelectedIdx, idx), l = Math.max(lastSelectedIdx, idx); i <= l; ++i) {
            /** @type {?} */
            const iId = isTable ? getRowId(this._value[i]) : i;
            if (selectedStatus) {
                this.selectedRows.set(iId, true);
            }
            else {
                this.selectedRows.delete(iId);
            }
        }
        this.lastSelectedId = id;
    }
    /**
     * Unselects all rows.
     * @return {?}
     */
    unselectAllRows() {
        // Doing this seems to be way faster than calling `selectedRows.clear()`
        // with, for example, 200000 rows
        this.selectedRows = new Map();
    }
    /**
     * Sets the selection status of all rows.
     * @param {?} selectedStatus `true` to set selected, `false` to set unselected.
     * @return {?}
     */
    setAllRowsSelectedStatus(selectedStatus) {
        if (selectedStatus) {
            /** @type {?} */
            const isTable = isTableSchema(this._schema);
            this._value.forEach((/**
             * @param {?} el
             * @param {?} i
             * @return {?}
             */
            (el, i) => {
                /** @type {?} */
                const id = isTable ? getRowId(el) : i;
                this.selectedRows.set(id, true);
            }));
        }
        else {
            this.unselectAllRows();
        }
    }
    /**
     * Toggles the status of all rows (if they are all selected, then they all
     * become unselected, otherwise they all become selected).
     * @return {?}
     */
    toggleAllRowsSelectedStatus() {
        this.setAllRowsSelectedStatus(!this.allRowsSelected);
    }
    /**
     * Removes all selected rows.
     * @return {?}
     */
    removeSelectedRows() {
        /** @type {?} */
        const isTable = isTableSchema(this._schema);
        this.relativeStorage.set('.', this._value.filter((/**
         * @param {?} el
         * @param {?} i
         * @return {?}
         */
        (el, i) => {
            /** @type {?} */
            const id = isTable ? getRowId(el) : i;
            return !this.selectedRows.has(id);
        })));
        this.unselectAllRows();
        this.lastSelectedId = null;
    }
    /**
     * Scrolls to the row with the provided id.
     * @param {?} id Identifier of the row to scroll to.
     * @return {?}
     */
    scrollToId(id) {
        if (!this.relativeStorage.has(`${id}`)) {
            throw pathError(this.path, `Invalid id: ${id}`);
        }
        /** @type {?} */
        const index = isListSchema(this._schema)
            ? ((/** @type {?} */ (id)))
            : this._value.findIndex((/**
             * @param {?} row
             * @return {?}
             */
            row => getRowId(row) === id));
        this.scrollToIndex(index);
    }
    /**
     * Scrolls to row with the provided index.
     * @param {?} index Index of the row to scroll to.
     * @return {?}
     */
    scrollToIndex(index) {
        this._scrollableBody.scrollToIndex(index);
    }
    /**
     * Track either by index or reference, depending on the type of value for list
     * schemas; or by row id, when dealing with table schemas.
     * @return {?}
     */
    get _trackBy() {
        if (isListSchema(this._schema)) {
            /** @type {?} */
            const elementsType = this._schema.elementsSchema.type;
            return ['boolean', 'number', 'string'].indexOf(elementsType) === -1
                ? (/**
                 * @param {?} _
                 * @param {?} el
                 * @return {?}
                 */
                (_, el) => el)
                : (/**
                 * @param {?} i
                 * @return {?}
                 */
                i => i);
        }
        else {
            return (/**
             * @param {?} _
             * @param {?} row
             * @return {?}
             */
            (_, row) => getRowId(row));
        }
    }
    /**
     * Identifier of the given element with the given index.
     * @param {?} el Element for which to get identifier.
     * @param {?} index Index of the element in the value.
     * @return {?} Identifier for the given element at the given index.
     */
    _id(el, index) {
        return isListSchema(this._schema) ? index : getRowId(el);
    }
    /**
     * Action to run when the "remove rows" button is clicked.
     * @return {?}
     */
    _onClickRemoveRows() {
        this.removeSelectedRows();
        this.relativeStorage.setDirty();
    }
    /**
     * Action to run when the "new row" button is clicked.
     * @return {?}
     */
    _onClickNewRow() {
        this.addNewRow();
        this.relativeStorage.setDirty();
    }
    /**
     * @protected
     * @return {?}
     */
    validatePath() {
        /** @type {?} */
        const schema = this.relativeStorage.schema();
        if ((!isListSchema(schema) && !isTableSchema(schema)) ||
            isNullableSchema(schema)) {
            throw pathError(this.path, 'Schema must be of the "list" or "table" type and not "nullable"');
        }
    }
    // Scroll to the activated row when one is activated.
    /**
     * @protected
     * @param {?} activatedPath
     * @return {?}
     */
    onActivate(activatedPath) {
        /** @type {?} */
        const relevantPart = this.lfStorage
            .pathInfo(activatedPath)
            .find((/**
         * @param {?} __0
         * @return {?}
         */
        ({ path }) => path.length > this.path.length));
        /** @type {?} */
        const id = relevantPart && (/** @type {?} */ (relevantPart.id));
        if (id !== undefined && this.relativeStorage.has(`${id}`)) {
            this.scrollToId(id);
        }
    }
    /**
     * State property name used to save the position of the scroll within the
     * table.
     * @private
     * @param {?} path Path of table.
     * @return {?} State property name.
     */
    scrollPositionStatePropName(path) {
        // Add the path to the name of the property when the value is a child of a
        // computed value (since children states are shared with the parent computed
        // value)
        return `lfTableScrollPosition${this.relativeStorage.isComputed(appendToPath(path, '..'))
            ? `[${path}]`
            : ''}`;
    }
    /**
     * Update the status of the "horizontal overflow".
     * @private
     * @return {?}
     */
    checkHasHorizontalOverflow() {
        this.hasHorizontalOverflow = this.scrollbarUtils.hasHorizontalOverflow(this._scrollableBody.elementRef.nativeElement);
    }
    /**
     * Width (in px) of the vertical scroll bar, when existent (`0` otherwise).
     * @private
     * @return {?}
     */
    get verticalScrollbarWidth() {
        return this._value.length > this.maxVisibleRows
            ? this.scrollbarUtils.verticalScrollbarWidth
            : 0;
    }
    /**
     * Height (in px) of the horizontal scroll bar, when existent (`0` otherwise).
     * @private
     * @return {?}
     */
    get horizontalScrollbarHeight() {
        return this.hasHorizontalOverflow
            ? this.scrollbarUtils.horizontalScrollbalHeight
            : 0;
    }
}
TableComponent.decorators = [
    { type: Component, args: [{
                selector: 'lf-table',
                changeDetection: ChangeDetectionStrategy.OnPush,
                providers: [{ provide: PathBased, useExisting: TableComponent }],
                template: "<ng-container *mobxAutorun>\n  <ng-container *ngIf=\"shouldDisplay\">\n    <div class=\"row label\">\n      <!-- Code + label -->\n      <div class=\"code-label-container\">\n        <!-- Code -->\n        <div\n          *ngIf=\"code\"\n          class=\"input-group input-group-sm code-container mr-2\"\n          [class.is-warning]=\"relativeStorage.shouldShowWarning()\"\n          [class.is-invalid]=\"relativeStorage.shouldShowError()\"\n        >\n          <div class=\"input-group-prepend\">\n            <div class=\"input-group-text code\">{{ code }}</div>\n          </div>\n        </div>\n\n        <!-- Label -->\n        <span *ngIf=\"label\" class=\"label-text\">{{ label }}</span>\n        <lf-help-tooltip\n          *ngIf=\"helpMessage\"\n          [help]=\"helpMessage\"\n        ></lf-help-tooltip>\n      </div>\n\n      <!-- Table actions -->\n      <lf-actions-menu\n        class=\"table-actions text-right\"\n        [actions]=\"_tableActions\"\n        buttonsSize=\"sm\"\n        [alignMenusRight]=\"true\"\n      >\n      </lf-actions-menu>\n    </div>\n\n    <!-- Required icon -->\n    <lf-icon\n      class=\"is-required\"\n      icon=\"asterisk\"\n      *ngIf=\"_showIsRequired\"\n    ></lf-icon>\n\n    <!-- Pending validation -->\n    <div\n      class=\"status pending\"\n      *ngIf=\"relativeStorage.shouldShowPending()\"\n    ></div>\n    <!-- Rejected validation -->\n    <div\n      class=\"status rejected\"\n      *ngIf=\"relativeStorage.shouldShowRejected()\"\n      (click)=\"relativeStorage.recomputeStateProperties()\"\n    >\n      <lf-tooltip [help]=\"retryValidateText\">\n        <lf-icon icon=\"warning\"></lf-icon>\n      </lf-tooltip>\n    </div>\n\n    <!-- (Debug) value state -->\n    <lf-value-state [path]=\"path\"></lf-value-state>\n\n    <!-- Container of whole table -->\n    <div\n      #targetElement\n      class=\"table-container\"\n      [class.is-warning]=\"relativeStorage.shouldShowWarning()\"\n      [class.is-invalid]=\"relativeStorage.shouldShowError()\"\n    >\n      <!-- Header table container (scrollable horizontally) -->\n      <div #scrollableHeader class=\"scrollable-header\">\n        <table\n          class=\"table table-sm header-table\"\n          [style.minWidth]=\"_tableMinWidth(true) + 'px'\"\n          [style.maxWidth]=\"_tableMaxWidth(true) + 'px'\"\n        >\n          <!-- Table columns -->\n          <colgroup>\n            <col\n              *ngFor=\"let width of _columnWidths(true)\"\n              [attr.width]=\"width\"\n            />\n          </colgroup>\n\n          <!-- Table header -->\n          <thead>\n            <tr *ngFor=\"let row of _tableHeader.rows; let first = first\">\n              <!-- Selection column -->\n              <th\n                *ngIf=\"first && allowsRowRemoval\"\n                class=\"table-cell-select\"\n                [attr.rowSpan]=\"_tableHeader.numberOfRows\"\n              >\n                <label\n                  class=\"select-all-checkbox custom-control custom-checkbox\"\n                >\n                  <input\n                    type=\"checkbox\"\n                    class=\"custom-control-input\"\n                    [disabled]=\"_value.length === 0\"\n                    [checked]=\"allRowsSelected && _value.length > 0\"\n                    [indeterminate]=\"!allRowsSelected && !noRowsSelected\"\n                    (click)=\"toggleAllRowsSelectedStatus()\"\n                  />\n                  <span class=\"custom-control-label\"></span>\n                </label>\n              </th>\n              <!-- User defined columns -->\n              <th\n                *ngFor=\"let column of row\"\n                [attr.rowSpan]=\"column.rowSpan\"\n                [attr.colSpan]=\"column.colSpan\"\n              >\n                <!-- TODO: Allow help messages/templates? -->\n                <ng-container *ngIf=\"column.id\">{{\n                  _columnLabel(column.id)\n                }}</ng-container>\n              </th>\n              <!-- Scrollbar compensation column -->\n              <th *ngIf=\"first\" [attr.rowSpan]=\"_tableHeader.numberOfRows\"></th>\n            </tr>\n          </thead>\n        </table>\n      </div>\n\n      <!-- Body table container (scrollable content) -->\n      <cdk-virtual-scroll-viewport\n        #scrollableBody\n        class=\"scrollable-body\"\n        [itemSize]=\"rowHeight\"\n        [style.height]=\"_scrollableBodyHeight + 'px'\"\n      >\n        <table\n          class=\"table table-sm body-table\"\n          [style.minWidth]=\"_tableMinWidth(false) + 'px'\"\n          [style.maxWidth]=\"_tableMaxWidth(false) + 'px'\"\n        >\n          <!-- Table columns (except scrollbar column) -->\n          <colgroup>\n            <col\n              *ngFor=\"let width of _columnWidths(false)\"\n              [attr.width]=\"width\"\n            />\n          </colgroup>\n\n          <!-- Table body -->\n          <tbody>\n            <!-- Row -->\n            <ng-container\n              *cdkVirtualFor=\"\n                let item of _value;\n                let index = index;\n                let count = count;\n                let first = first;\n                let last = last;\n                let even = even;\n                let odd = odd;\n                templateCacheSize: virtualScrollTemplateCacheSize;\n                trackBy: _trackBy\n              \"\n              [ngTemplateOutlet]=\"_rowTemplate\"\n              [ngTemplateOutletContext]=\"{\n                $implicit: item,\n                id: _id(item, index),\n                index: index,\n                count: count,\n                first: first,\n                last: last,\n                even: even,\n                odd: odd\n              }\"\n            ></ng-container>\n          </tbody>\n        </table>\n      </cdk-virtual-scroll-viewport>\n\n      <!-- No items -->\n      <div *ngIf=\"_value.length === 0\" class=\"no-rows\">\n        <ng-container *ngIf=\"!_noRowsTemplate\">\n          {{ _noRowsText }}\n        </ng-container>\n        <ng-container\n          *ngIf=\"_noRowsTemplate\"\n          [ngTemplateOutlet]=\"_noRowsTemplate\"\n        >\n        </ng-container>\n      </div>\n    </div>\n\n    <!-- Legend -->\n    <div *ngIf=\"legend\" class=\"legend\">{{ legend }}</div>\n  </ng-container>\n</ng-container>\n"
            }] }
];
TableComponent.propDecorators = {
    rowHeight: [{ type: Input }],
    maxVisibleRows: [{ type: Input }],
    virtualScrollTemplateCacheSize: [{ type: Input }],
    allowRowAddition: [{ type: Input }],
    allowRowRemoval: [{ type: Input }],
    _tableHeader: [{ type: ContentChild, args: [TableHeaderDirective, { static: false },] }],
    _rowTemplate: [{ type: ContentChild, args: ['lfTableRowTemplate', { static: false },] }],
    _noRowsTemplate: [{ type: ContentChild, args: ['lfTableNoRowsTemplate', { static: false },] }],
    _scrollableHeader: [{ type: ViewChild, args: ['scrollableHeader', { static: false },] }],
    _scrollableBody: [{ type: ViewChild, args: ['scrollableBody', { static: false },] }]
};
__decorate([
    observable$1,
    __metadata("design:type", Object)
], TableComponent.prototype, "rowHeight", void 0);
__decorate([
    observable$1,
    __metadata("design:type", Object)
], TableComponent.prototype, "maxVisibleRows", void 0);
__decorate([
    observable$1,
    __metadata("design:type", Object)
], TableComponent.prototype, "allowRowAddition", void 0);
__decorate([
    observable$1,
    __metadata("design:type", Object)
], TableComponent.prototype, "allowRowRemoval", void 0);
__decorate([
    observable$1,
    __metadata("design:type", Boolean)
], TableComponent.prototype, "hasHorizontalOverflow", void 0);
__decorate([
    observable$1,
    __metadata("design:type", Map)
], TableComponent.prototype, "selectedRows", void 0);
__decorate([
    computed,
    __metadata("design:type", String),
    __metadata("design:paramtypes", [])
], TableComponent.prototype, "_addRowText", null);
__decorate([
    computed,
    __metadata("design:type", String),
    __metadata("design:paramtypes", [])
], TableComponent.prototype, "_removeRowsText", null);
__decorate([
    computed,
    __metadata("design:type", String),
    __metadata("design:paramtypes", [])
], TableComponent.prototype, "_noRowsText", null);
__decorate([
    computed,
    __metadata("design:type", Array),
    __metadata("design:paramtypes", [])
], TableComponent.prototype, "_columnLabels", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], TableComponent.prototype, "_schema", null);
__decorate([
    computed,
    __metadata("design:type", Array),
    __metadata("design:paramtypes", [])
], TableComponent.prototype, "_value", null);
__decorate([
    computed,
    __metadata("design:type", Boolean),
    __metadata("design:paramtypes", [])
], TableComponent.prototype, "_showIsRequired", null);
__decorate([
    computed,
    __metadata("design:type", Array),
    __metadata("design:paramtypes", [])
], TableComponent.prototype, "_tableActions", null);
__decorate([
    computed,
    __metadata("design:type", Number),
    __metadata("design:paramtypes", [])
], TableComponent.prototype, "_scrollableBodyHeight", null);
__decorate([
    computed,
    __metadata("design:type", Boolean),
    __metadata("design:paramtypes", [])
], TableComponent.prototype, "allowsRowAddition", null);
__decorate([
    computed,
    __metadata("design:type", Boolean),
    __metadata("design:paramtypes", [])
], TableComponent.prototype, "allowsRowRemoval", null);
__decorate([
    computed,
    __metadata("design:type", Number),
    __metadata("design:paramtypes", [])
], TableComponent.prototype, "numberOfRowsSelected", null);
__decorate([
    computed,
    __metadata("design:type", Boolean),
    __metadata("design:paramtypes", [])
], TableComponent.prototype, "allRowsSelected", null);
__decorate([
    computed,
    __metadata("design:type", Boolean),
    __metadata("design:paramtypes", [])
], TableComponent.prototype, "noRowsSelected", null);
__decorate([
    action,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], TableComponent.prototype, "addNewRow", null);
__decorate([
    action,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Boolean]),
    __metadata("design:returntype", void 0)
], TableComponent.prototype, "setRowSelectedStatus", null);
__decorate([
    action,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Boolean]),
    __metadata("design:returntype", void 0)
], TableComponent.prototype, "setRowSelectedStatusFromLastSelected", null);
__decorate([
    action,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], TableComponent.prototype, "unselectAllRows", null);
__decorate([
    action,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Boolean]),
    __metadata("design:returntype", void 0)
], TableComponent.prototype, "setAllRowsSelectedStatus", null);
__decorate([
    action,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], TableComponent.prototype, "toggleAllRowsSelectedStatus", null);
__decorate([
    action,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], TableComponent.prototype, "removeSelectedRows", null);
__decorate([
    computed,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [])
], TableComponent.prototype, "_trackBy", null);
__decorate([
    action,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], TableComponent.prototype, "_onClickRemoveRows", null);
__decorate([
    action,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], TableComponent.prototype, "_onClickNewRow", null);
__decorate([
    action,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", void 0)
], TableComponent.prototype, "onActivate", null);
__decorate([
    action,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], TableComponent.prototype, "checkHasHorizontalOverflow", null);
__decorate([
    computed,
    __metadata("design:type", Number),
    __metadata("design:paramtypes", [])
], TableComponent.prototype, "verticalScrollbarWidth", null);
__decorate([
    computed,
    __metadata("design:type", Number),
    __metadata("design:paramtypes", [])
], TableComponent.prototype, "horizontalScrollbarHeight", null);
if (false) {
    /**
     * Height (in `px`) of a single row (virtual scroll settings). All rows must
     * have the same height.
     * @type {?}
     */
    TableComponent.prototype.rowHeight;
    /**
     * Maximum number of visible rows at any given time.
     * @type {?}
     */
    TableComponent.prototype.maxVisibleRows;
    /**
     * Size of the virtual scroll's template cache
     * (`cdkVirtualForTemplateCacheSize` input of Angular CDK's virtual for
     * directive).
     * @type {?}
     */
    TableComponent.prototype.virtualScrollTemplateCacheSize;
    /**
     * Whether to allow addition of new rows.
     * @type {?}
     */
    TableComponent.prototype.allowRowAddition;
    /**
     * Whether to allow removal of rows.
     * @type {?}
     */
    TableComponent.prototype.allowRowRemoval;
    /** @type {?} */
    TableComponent.prototype._tableHeader;
    /** @type {?} */
    TableComponent.prototype._rowTemplate;
    /** @type {?} */
    TableComponent.prototype._noRowsTemplate;
    /** @type {?} */
    TableComponent.prototype._scrollableHeader;
    /** @type {?} */
    TableComponent.prototype._scrollableBody;
    /**
     * @type {?}
     * @private
     */
    TableComponent.prototype.scrollbarUtils;
    /**
     * @type {?}
     * @private
     */
    TableComponent.prototype.hasHorizontalOverflow;
    /**
     * @type {?}
     * @private
     */
    TableComponent.prototype.selectedRows;
    /**
     * @type {?}
     * @private
     */
    TableComponent.prototype.lastSelectedId;
    /**
     * @type {?}
     * @private
     */
    TableComponent.prototype.lastScrollPosition;
    /**
     * @type {?}
     * @private
     */
    TableComponent.prototype.scrollSubscription;
    /**
     * @type {?}
     * @private
     */
    TableComponent.prototype.disposeObservePath;
    /**
     * @type {?}
     * @private
     */
    TableComponent.prototype.disposeObserveValue;
    /**
     * @type {?}
     * @private
     */
    TableComponent.prototype.disposeObserveBodyHeight;
    /**
     * @type {?}
     * @private
     */
    TableComponent.prototype.viewportSubscription;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * LF table-cell component.
 */
class TableCellComponent {
    /**
     * @param {?} parentTableComponent
     */
    constructor(parentTableComponent) {
        this.parentTableComponent = parentTableComponent;
    }
    /**
     * Height of the cell.
     * @return {?}
     */
    get _hostStyleHeight() {
        return this.parentTableComponent.rowHeight + 'px';
    }
}
TableCellComponent.decorators = [
    { type: Component, args: [{
                // tslint:disable-next-line:component-selector
                selector: 'td[lf-table-cell], th[lf-table-cell]',
                changeDetection: ChangeDetectionStrategy.OnPush,
                template: "<ng-container *mobxAutorun>\n  <div class=\"table-cell-container\">\n    <div class=\"table-cell-inner\"><ng-content></ng-content></div>\n  </div>\n</ng-container>\n"
            }] }
];
/** @nocollapse */
TableCellComponent.ctorParameters = () => [
    { type: TableComponent }
];
TableCellComponent.propDecorators = {
    _hostStyleHeight: [{ type: HostBinding, args: ['style.height',] }]
};
__decorate([
    computed,
    __metadata("design:type", String),
    __metadata("design:paramtypes", [])
], TableCellComponent.prototype, "_hostStyleHeight", null);
if (false) {
    /**
     * @type {?}
     * @private
     */
    TableCellComponent.prototype.parentTableComponent;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * LF table-row component.
 */
class TableRowComponent extends Value {
    // Angular lifecycle =========================================================
    /**
     * @return {?}
     */
    ngOnInit() {
        super.ngOnInit();
        // Table rows must be children of tables
        if (!(this.parentPathBasedComponent instanceof TableComponent)) {
            throw pathError(this.path, 'This component must be a child of a `TableComponent`');
        }
    }
    // ===========================================================================
    /**
     * Whether the row can be removed.
     * @return {?}
     */
    get allowsRemoval() {
        return this.parentPathBasedComponent.allowsRowRemoval;
    }
    /**
     * Whether the row is selected (adds an `is-selected` class to the row).
     * @return {?}
     */
    get isSelected() {
        return this.parentPathBasedComponent.getRowSelectedStatus((/** @type {?} */ (this.relativeStorage.currentId)));
    }
    /**
     * Class that should be added to the row when the row should show as having an
     * error.
     * @return {?}
     */
    get _classIsInvalid() {
        return !!(this.relativeStorage.has() && this.relativeStorage.shouldShowError());
    }
    /**
     * Class that should be added to the row when the row should show as having a
     * warning.
     * @return {?}
     */
    get _classIsWarning() {
        return !!(this.relativeStorage.has() && this.relativeStorage.shouldShowWarning());
    }
    /**
     * Sets the row's selection status.
     * @param {?} selectedStatus `true` to set selected, `false` to set unselected.
     * @return {?}
     */
    setSelectedStatus(selectedStatus) {
        this.parentPathBasedComponent.setRowSelectedStatus((/** @type {?} */ (this.relativeStorage.currentId)), selectedStatus);
    }
    /**
     * Sets the row selection status of all rows from the last row that has had
     * its selection status set to this row.
     * @param {?} selectedStatus `true` to set selected, `false` to set unselected.
     * @return {?}
     */
    setSelectedStatusFromLastSelected(selectedStatus) {
        this.parentPathBasedComponent.setRowSelectedStatusFromLastSelected((/** @type {?} */ (this.relativeStorage.currentId)), selectedStatus);
    }
    /**
     * Action to run when the selection checkbox label is clicked: added as a fix
     * for Firefox, where `shift+click` on the label won't trigger a click on the
     * `input`; as such we manually call the event.
     * @param {?} event Click event.
     * @param {?} checkboxElement Element on which the even should be called instead.
     * @return {?}
     */
    _onLabelClick(event, checkboxElement) {
        this._onSelectionCheckboxClick(event);
        checkboxElement.focus();
        event.preventDefault();
    }
    /**
     * Action to run when the selection checkbox is clicked.
     * @param {?} event Click event.
     * @return {?}
     */
    _onSelectionCheckboxClick(event) {
        if (event.shiftKey) {
            this.setSelectedStatusFromLastSelected(!this.isSelected);
        }
        else {
            this.setSelectedStatus(!this.isSelected);
        }
        event.stopPropagation(); // Don't call the label event
    }
}
TableRowComponent.decorators = [
    { type: Component, args: [{
                // tslint:disable-next-line: component-selector
                selector: 'tr[lf-table-row]',
                changeDetection: ChangeDetectionStrategy.OnPush,
                providers: [{ provide: PathBased, useExisting: TableRowComponent }],
                template: "<ng-container *mobxAutorun>\n  <ng-container *ngIf=\"relativeStorage.has()\">\n    <!-- Cell used to hold a checkbox that marks the row as selected -->\n    <td lf-table-cell *ngIf=\"allowsRemoval\" class=\"table-cell-select\">\n      <label\n        class=\"custom-control custom-checkbox\"\n        (click)=\"_onLabelClick($event, checkboxElement)\"\n      >\n        <input\n          #checkboxElement\n          type=\"checkbox\"\n          class=\"custom-control-input\"\n          [checked]=\"isSelected\"\n          (click)=\"_onSelectionCheckboxClick($event)\"\n        />\n        <span class=\"custom-control-label\"></span>\n      </label>\n    </td>\n    <!-- User defined cells -->\n    <ng-content select=\"th[lf-table-cell], td[lf-table-cell]\"></ng-content>\n  </ng-container>\n</ng-container>\n"
            }] }
];
TableRowComponent.propDecorators = {
    isSelected: [{ type: HostBinding, args: ['class.is-selected',] }],
    _classIsInvalid: [{ type: HostBinding, args: ['class.is-invalid',] }],
    _classIsWarning: [{ type: HostBinding, args: ['class.is-warning',] }]
};
__decorate([
    computed,
    __metadata("design:type", Boolean),
    __metadata("design:paramtypes", [])
], TableRowComponent.prototype, "allowsRemoval", null);
__decorate([
    computed,
    __metadata("design:type", Boolean),
    __metadata("design:paramtypes", [])
], TableRowComponent.prototype, "isSelected", null);
__decorate([
    computed,
    __metadata("design:type", Boolean),
    __metadata("design:paramtypes", [])
], TableRowComponent.prototype, "_classIsInvalid", null);
__decorate([
    computed,
    __metadata("design:type", Boolean),
    __metadata("design:paramtypes", [])
], TableRowComponent.prototype, "_classIsWarning", null);
if (false) {
    /**
     * @type {?}
     * @protected
     */
    TableRowComponent.prototype.parentPathBasedComponent;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Component used for editing a "single line" LF string.
 */
class TextComponent extends InlineValue {
    /**
     * Whether to show the "is required" icon on the component.
     * @return {?}
     */
    get showIsRequired() {
        /** @type {?} */
        const minLength = this.relativeStorage.getStateProperty('.', 'minLength');
        return minLength !== undefined && minLength > 0;
    }
    /**
     * Action to run on every `input` event of the text input.
     * @param {?} evt `input` event.
     * @return {?}
     */
    _onInput(evt) {
        this.relativeStorage.set('.', evt.target.value);
        this.relativeStorage.setDirty();
    }
    /**
     * @protected
     * @return {?}
     */
    validatePath() {
        /** @type {?} */
        const schema = this.relativeStorage.schema();
        if (!isStringSchema(schema) || isNullableSchema(schema)) {
            throw pathError(this.path, 'Schema must be of the "string" type and not "nullable"');
        }
    }
}
TextComponent.decorators = [
    { type: Component, args: [{
                selector: 'lf-text',
                changeDetection: ChangeDetectionStrategy.OnPush,
                template: "<ng-container *mobxAutorun>\n  <ng-container *ngIf=\"shouldDisplay\">\n    <!-- Label -->\n    <span *ngIf=\"label\" class=\"label\">{{ label }}</span>\n    <lf-help-tooltip *ngIf=\"helpMessage\" [help]=\"helpMessage\"></lf-help-tooltip>\n\n    <!-- Required icon -->\n    <lf-icon\n      class=\"is-required\"\n      icon=\"asterisk\"\n      *ngIf=\"showIsRequired\"\n    ></lf-icon>\n\n    <!-- Pending validation -->\n    <div\n      class=\"status pending\"\n      *ngIf=\"relativeStorage.shouldShowPending()\"\n    ></div>\n    <!-- Rejected validation -->\n    <div\n      class=\"status rejected\"\n      *ngIf=\"relativeStorage.shouldShowRejected()\"\n      (click)=\"relativeStorage.recomputeStateProperties()\"\n    >\n      <lf-tooltip [help]=\"retryValidateText\">\n        <lf-icon icon=\"warning\"></lf-icon>\n      </lf-tooltip>\n    </div>\n\n    <!-- (Debug) value state -->\n    <lf-value-state [path]=\"path\"></lf-value-state>\n\n    <!-- Input group -->\n    <div\n      class=\"input-group input-group-sm\"\n      [class.is-warning]=\"relativeStorage.shouldShowWarning()\"\n      [class.is-invalid]=\"relativeStorage.shouldShowError()\"\n    >\n      <!-- Code -->\n      <div *ngIf=\"code\" class=\"input-group-prepend\">\n        <div class=\"input-group-text\">{{ code }}</div>\n      </div>\n\n      <!-- Input -->\n      <input\n        #focusTarget\n        type=\"text\"\n        class=\"form-control\"\n        [class.is-warning]=\"relativeStorage.shouldShowWarning()\"\n        [class.is-invalid]=\"relativeStorage.shouldShowError()\"\n        [class.is-readonly]=\"isReadOnly\"\n        [name]=\"path\"\n        [value]=\"relativeStorage.get()\"\n        [attr.minLength]=\"relativeStorage.getStateProperty('.', 'minLength')\"\n        [attr.maxLength]=\"relativeStorage.getStateProperty('.', 'maxLength')\"\n        [readonly]=\"isReadOnly\"\n        autocomplete=\"off\"\n        (input)=\"_onInput($event)\"\n        (focus)=\"_onFocus($event)\"\n        (blur)=\"_onBlur($event)\"\n      />\n    </div>\n\n    <!-- Legend -->\n    <div *ngIf=\"legend\" class=\"legend\">{{ legend }}</div>\n  </ng-container>\n</ng-container>\n"
            }] }
];
__decorate([
    computed,
    __metadata("design:type", Boolean),
    __metadata("design:paramtypes", [])
], TextComponent.prototype, "showIsRequired", null);
__decorate([
    action,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], TextComponent.prototype, "_onInput", null);

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Component used for editing a "multiple lines" LF string.
 */
class TextareaComponent extends InlineValue {
    constructor() {
        super(...arguments);
        /**
         * Initial height of the textarea (CSS measurement string).
         */
        this.initialHeight = '6rem';
    }
    /**
     * Whether to show the "is required" icon on the component.
     * @return {?}
     */
    get showIsRequired() {
        /** @type {?} */
        const minLength = this.relativeStorage.getStateProperty('.', 'minLength');
        return minLength !== undefined && minLength > 0;
    }
    /**
     * Action to run on every `input` event of the textarea input.
     * @param {?} evt `input` event.
     * @return {?}
     */
    _onInput(evt) {
        this.relativeStorage.set('.', evt.target.value);
        this.relativeStorage.setDirty();
    }
    /**
     * @protected
     * @return {?}
     */
    validatePath() {
        /** @type {?} */
        const schema = this.relativeStorage.schema();
        if (!isStringSchema(schema) || isNullableSchema(schema)) {
            throw pathError(this.path, 'Schema must be of the "string" type and not "nullable"');
        }
    }
}
TextareaComponent.decorators = [
    { type: Component, args: [{
                selector: 'lf-textarea',
                changeDetection: ChangeDetectionStrategy.OnPush,
                template: "<ng-container *mobxAutorun>\n  <ng-container *ngIf=\"shouldDisplay\">\n    <!-- Label -->\n    <span *ngIf=\"label\" class=\"label\">{{ label }}</span>\n    <lf-help-tooltip *ngIf=\"helpMessage\" [help]=\"helpMessage\"></lf-help-tooltip>\n\n    <!-- Required icon -->\n    <lf-icon\n      class=\"is-required\"\n      icon=\"asterisk\"\n      *ngIf=\"showIsRequired\"\n    ></lf-icon>\n\n    <!-- Pending validation -->\n    <div\n      class=\"status pending\"\n      *ngIf=\"relativeStorage.shouldShowPending()\"\n    ></div>\n    <!-- Rejected validation -->\n    <div\n      class=\"status rejected\"\n      *ngIf=\"relativeStorage.shouldShowRejected()\"\n      (click)=\"relativeStorage.recomputeStateProperties()\"\n    >\n      <lf-tooltip [help]=\"retryValidateText\">\n        <lf-icon icon=\"warning\"></lf-icon>\n      </lf-tooltip>\n    </div>\n\n    <!-- (Debug) value state -->\n    <lf-value-state [path]=\"path\"></lf-value-state>\n\n    <!-- Input group -->\n    <div\n      class=\"input-group input-group-sm\"\n      [class.is-warning]=\"relativeStorage.shouldShowWarning()\"\n      [class.is-invalid]=\"relativeStorage.shouldShowError()\"\n    >\n      <!-- Code -->\n      <div *ngIf=\"code\" class=\"input-group-prepend\">\n        <div class=\"input-group-text\">{{ code }}</div>\n      </div>\n\n      <!-- Textarea -->\n      <textarea\n        #focusTarget\n        type=\"text\"\n        class=\"form-control\"\n        [class.is-warning]=\"relativeStorage.shouldShowWarning()\"\n        [class.is-invalid]=\"relativeStorage.shouldShowError()\"\n        [class.is-readonly]=\"isReadOnly\"\n        [style.height]=\"initialHeight\"\n        [name]=\"path\"\n        [value]=\"relativeStorage.get()\"\n        [attr.minLength]=\"relativeStorage.getStateProperty('.', 'minLength')\"\n        [attr.maxLength]=\"relativeStorage.getStateProperty('.', 'maxLength')\"\n        [readonly]=\"isReadOnly\"\n        autocomplete=\"off\"\n        (input)=\"_onInput($event)\"\n        (focus)=\"_onFocus($event)\"\n        (blur)=\"_onBlur($event)\"\n      ></textarea>\n    </div>\n\n    <!-- Legend -->\n    <div *ngIf=\"legend\" class=\"legend\">{{ legend }}</div>\n  </ng-container>\n</ng-container>\n"
            }] }
];
TextareaComponent.propDecorators = {
    initialHeight: [{ type: Input }]
};
__decorate([
    computed,
    __metadata("design:type", Boolean),
    __metadata("design:paramtypes", [])
], TextareaComponent.prototype, "showIsRequired", null);
__decorate([
    action,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], TextareaComponent.prototype, "_onInput", null);
if (false) {
    /**
     * Initial height of the textarea (CSS measurement string).
     * @type {?}
     */
    TextareaComponent.prototype.initialHeight;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Lf tooltip component.
 */
class TooltipComponent {
    constructor() {
        this.placement = 'top';
        this.delay = 0;
    }
}
TooltipComponent.decorators = [
    { type: Component, args: [{
                selector: 'lf-tooltip',
                changeDetection: ChangeDetectionStrategy.OnPush,
                template: "<ng-container *mobxAutorun>\n  <span [tooltip]=\"help\" [placement]=\"placement\" [delay]=\"delay\">\n    <ng-content></ng-content>\n  </span>\n</ng-container>\n"
            }] }
];
TooltipComponent.propDecorators = {
    help: [{ type: Input }],
    placement: [{ type: Input }],
    delay: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    TooltipComponent.prototype.help;
    /** @type {?} */
    TooltipComponent.prototype.placement;
    /** @type {?} */
    TooltipComponent.prototype.delay;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Entry for an item of the tree-nav.
 */
class TreeEntryComponent extends PathBased {
    constructor() {
        super(...arguments);
        /**
         * Whether this tree-entry is to be displayed as the root one (in which case
         * its label isn't shown anywhere and its children are displayed one after the
         * other). The entry can be displayed as root only when its schema is complex.
         * A button is displayed at the end of the list when there are optional values
         * to be added or to add a new element in case the schema is a list.
         */
        this.displayAsRoot = true;
    }
    // Angular lifecycle =========================================================
    /**
     * @return {?}
     */
    ngOnInit() {
        this.lfRouter = this.injector.get(LfRouter);
        super.ngOnInit();
        // Validate the path whenever `displayAsRoot` changes
        this.disposeOnDisplayAsRootChanges = observe(this, 'displayAsRoot', (/**
         * @return {?}
         */
        () => {
            this.validatePath();
        }));
        // Set `componentLevel` when not defined
        if (this._componentLevel === undefined) {
            this._componentLevel = this.matchingRouteComponentLevel(this.matchingRoutes[0]);
        }
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        super.ngOnDestroy();
        this.disposeOnDisplayAsRootChanges();
    }
    // I18n ======================================================================
    /**
     * Label associated with this value.
     * @return {?}
     */
    get label() {
        /** @type {?} */
        const label = this.translate(I18N_VALUE_LABEL_KEY);
        return label === undefined
            ? this.relativeStorage.currentPath === '/'
                ? '/'
                : this.relativeStorage.currentId
            : label;
    }
    /**
     * Message used to ask the user to retry validating the value.
     * @return {?}
     */
    get retryValidateText() {
        return this.translate(I18N_VALUE_RETRY_VALIDATE_TEXT_KEY);
    }
    /**
     * @return {?}
     */
    get addOptionalValueButtonText() {
        return this.translate(I18N_TREE_NAV_ADD_OPTIONAL_VALUE_BUTTON_TEXT_KEY);
    }
    /**
     * @return {?}
     */
    get addValueToListButtonText() {
        return this.translate(I18N_TREE_NAV_ADD_VALUE_TO_LIST_BUTTON_TEXT_KEY);
    }
    // ===========================================================================
    /**
     * @return {?}
     */
    get matchingRoutes() {
        return this.lfRouter.matchingRoutes(this.path, { navigateToParent: false });
    }
    /**
     * @return {?}
     */
    get navigateableRoute() {
        return this.lfRouter.navigateableRouteFromMatchingRoutes(this.matchingRoutes);
    }
    /**
     * @return {?}
     */
    get url() {
        return (this.navigateableRoute &&
            this.lfRouter.createUrlFromNavigateableRoute(this.navigateableRoute));
    }
    /**
     * @return {?}
     */
    get isActive() {
        return this.lfRouter.isActive(this.path);
    }
    /**
     * @return {?}
     */
    get isList() {
        /** @type {?} */
        const schema = this.relativeStorage.schema();
        return isListSchema(schema) || isTableSchema(schema);
    }
    /**
     * @return {?}
     */
    get childrenIds() {
        return this.lfStorage.childrenIds(this.path).filter((/**
         * @param {?} id
         * @return {?}
         */
        (id) => {
            /** @type {?} */
            const path = appendToPath(this.path, id);
            /** @type {?} */
            const matchingRoutes = this.lfRouter
                .matchingRoutes(path, { navigateToParent: false })
                .filter((/**
             * @param {?} matchingRoute
             * @return {?}
             */
            matchingRoute => this.matchingRouteComponentLevel(matchingRoute) ===
                this._componentLevel));
            return matchingRoutes.length > 0;
        }));
    }
    /**
     * @return {?}
     */
    get nonNullChildrenIds() {
        return this.childrenIds.filter((/**
         * @param {?} id
         * @return {?}
         */
        id => this.relativeStorage.get(id.toString()) !== null));
    }
    /**
     * @return {?}
     */
    get nullChildrenIds() {
        return this.childrenIds.filter((/**
         * @param {?} id
         * @return {?}
         */
        id => this.relativeStorage.get(id.toString()) === null));
    }
    /**
     * @param {?} id
     * @return {?}
     */
    childLabel(id) {
        /** @type {?} */
        const childPath = appendToPath(this.path, id);
        return this.lfI18n.getFromPath(childPath, I18N_VALUE_LABEL_KEY) || id;
    }
    /**
     * @return {?}
     */
    addToList() {
        const { schema, value } = this.lfStorage
            .pathInfo(this.path, { fetchValues: true })
            .slice(-1)[0];
        this.lfStorage.push(this.path, undefined);
        this.lfRouter.navigate(appendToPath(this.path, isListSchema(schema)
            ? value.length - 1
            : getRowId(value[value.length - 1])), { queryParamsHandling: 'merge' });
    }
    /**
     * @param {?} id
     * @return {?}
     */
    initializeChild(id) {
        this.relativeStorage.initialize(id);
        this.lfRouter.navigate(appendToPath(this.path, id), {
            queryParamsHandling: 'merge',
        });
    }
    /**
     * @protected
     * @return {?}
     */
    validatePath() {
        if (this.matchingRoutes.length === 0) {
            throw pathError(this.path, 'The path or one of its children must have an associated route');
        }
        /** @type {?} */
        const schema = this.relativeStorage.schema();
        if (this.displayAsRoot && !isComplexSchema(schema)) {
            throw pathError(this.path, 'Schema must be complex to display the entry as "root"');
        }
    }
    /**
     * The "component" level of the provided matching route (basically the number
     * of outlets used by the route).
     * @private
     * @param {?} matchingRoute Matching of a route with a provided path.
     * @return {?} Component level for the matching route.
     */
    matchingRouteComponentLevel(matchingRoute) {
        return matchingRoute.routes.reduce((/**
         * @param {?} level
         * @param {?} route
         * @return {?}
         */
        (level, route) => level + (route.component || route.redirectTo !== undefined ? 1 : 0)), 0);
    }
}
TreeEntryComponent.decorators = [
    { type: Component, args: [{
                selector: 'lf-tree-entry',
                changeDetection: ChangeDetectionStrategy.OnPush,
                providers: [{ provide: PathBased, useExisting: TreeEntryComponent }],
                template: "<ng-container *mobxAutorun>\n  <ng-container *ngIf=\"relativeStorage.has() && relativeStorage.get() !== null\">\n    <!-- Root entry (only show children) -->\n    <ng-container *ngIf=\"displayAsRoot\">\n      <!-- Children -->\n      <ng-container [ngTemplateOutlet]=\"childrenTemplate\"></ng-container>\n\n      <!-- Action at the bottom (with label) -->\n      <ng-container\n        [ngTemplateOutlet]=\"actionsTemplate\"\n        [ngTemplateOutletContext]=\"{\n          addOptionalValueButtonText: addOptionalValueButtonText,\n          addValueToListButtonText: addValueToListButtonText\n        }\"\n      ></ng-container>\n    </ng-container>\n\n    <!-- Non-root entry (show entry + children) -->\n    <div\n      *ngIf=\"!displayAsRoot\"\n      [ngClass]=\"{'nav-item': true, active: isActive}\"\n      [class.is-warning]=\"relativeStorage.shouldShowWarning()\"\n      [class.is-invalid]=\"relativeStorage.shouldShowError()\"\n    >\n      <!-- Item content -->\n      <div class=\"item-content\">\n        <!-- Icon + label (either in link form or not, depending on whether a\n             link is available) -->\n        <a\n          *ngIf=\"url\"\n          class=\"nav-link\"\n          [routerLink]=\"url\"\n          queryParamsHandling=\"merge\"\n        >\n          <ng-container [ngTemplateOutlet]=\"linkTemplate\"></ng-container>\n        </a>\n        <div *ngIf=\"!url\" class=\"nav-link is-disabled\">\n          <ng-container [ngTemplateOutlet]=\"linkTemplate\"></ng-container>\n        </div>\n\n        <!-- Pending validation -->\n        <div\n          class=\"status pending\"\n          *ngIf=\"relativeStorage.shouldShowPending()\"\n        ></div>\n        <!-- Rejected validation -->\n        <div\n          class=\"status rejected\"\n          *ngIf=\"relativeStorage.shouldShowRejected()\"\n          (click)=\"relativeStorage.recomputeStateProperties()\"\n        >\n          <lf-tooltip [help]=\"retryValidateText\">\n            <lf-icon icon=\"warning\"></lf-icon>\n          </lf-tooltip>\n        </div>\n\n        <!-- Actions -->\n        <ng-container\n          [ngTemplateOutlet]=\"actionsTemplate\"\n          [ngTemplateOutletContext]=\"{alignRight: true}\"\n        ></ng-container>\n      </div>\n\n      <!-- Children -->\n      <ng-container\n        *ngIf=\"isActive\"\n        [ngTemplateOutlet]=\"childrenTemplate\"\n      ></ng-container>\n    </div>\n  </ng-container>\n</ng-container>\n\n<ng-template #linkTemplate>\n  <!-- Icon for items with sub-items, either \"v\" or \">\" or bars for lists -->\n  <span *ngIf=\"isList || nonNullChildrenIds.length > 0\" class=\"item-icon\">\n    <lf-icon\n      [icon]=\"isList ? 'bars' : isActive ? 'angle-down' : 'angle-right'\"\n      [fixedWidth]=\"true\"\n    >\n    </lf-icon>\n  </span>\n\n  <!-- Entry label -->\n  <div class=\"item-label\">{{ label }}</div>\n</ng-template>\n\n<!-- Template to show all children -->\n<ng-template #childrenTemplate>\n  <!-- Children -->\n  <ng-container *ngFor=\"let id of nonNullChildrenIds\">\n    <nav class=\"nav flex-column\">\n      <lf-tree-entry\n        [path]=\"id\"\n        [displayAsRoot]=\"false\"\n        [_componentLevel]=\"_componentLevel\"\n      ></lf-tree-entry>\n    </nav>\n  </ng-container>\n</ng-template>\n\n<!-- Template to show the item's action (if appropriate), either a button to add\n     a new element to the list, or a menu to add an optional value; possibly,\n     each action may be labeled and the menu aligned to the right -->\n<ng-template\n  #actionsTemplate\n  let-addOptionalText=\"addOptionalValueButtonText\"\n  let-addToListText=\"addValueToListButtonText\"\n  let-alignRight=\"alignRight\"\n>\n  <!-- Add optional value -->\n  <div *ngIf=\"!isList && nullChildrenIds.length > 0\" class=\"value-add\">\n    <lf-dropdown>\n      <button\n        lf-button\n        [style]=\"'outline-secondary'\"\n        size=\"sm\"\n        [dropdownToggle]=\"true\"\n        [block]=\"true\"\n      >\n        <lf-icon icon=\"plus-circle\"></lf-icon> {{ addOptionalText }}\n      </button>\n      <!-- Align to the right when not -->\n      <lf-dropdown-menu [alignRight]=\"alignRight\">\n        <button\n          lf-dropdown-item\n          *ngFor=\"let id of nullChildrenIds\"\n          (click)=\"initializeChild(id)\"\n        >\n          {{ childLabel(id) }}\n        </button>\n      </lf-dropdown-menu>\n    </lf-dropdown>\n  </div>\n\n  <!-- Add value to list -->\n  <div *ngIf=\"isList\" class=\"value-add\">\n    <button\n      lf-button\n      [style]=\"'outline-secondary'\"\n      [isDisabled]=\"\n        relativeStorage.size() >=\n        relativeStorage.getStateProperty('.', 'maxSize')\n      \"\n      size=\"sm\"\n      (click)=\"addToList()\"\n    >\n      <lf-icon icon=\"plus-circle\"></lf-icon> {{ addToListText }}\n    </button>\n  </div>\n</ng-template>\n"
            }] }
];
TreeEntryComponent.propDecorators = {
    displayAsRoot: [{ type: Input }],
    _componentLevel: [{ type: Input }]
};
__decorate([
    observable$1,
    __metadata("design:type", Object)
], TreeEntryComponent.prototype, "displayAsRoot", void 0);
__decorate([
    computed,
    __metadata("design:type", String),
    __metadata("design:paramtypes", [])
], TreeEntryComponent.prototype, "label", null);
__decorate([
    computed,
    __metadata("design:type", String),
    __metadata("design:paramtypes", [])
], TreeEntryComponent.prototype, "retryValidateText", null);
__decorate([
    computed,
    __metadata("design:type", String),
    __metadata("design:paramtypes", [])
], TreeEntryComponent.prototype, "addOptionalValueButtonText", null);
__decorate([
    computed,
    __metadata("design:type", String),
    __metadata("design:paramtypes", [])
], TreeEntryComponent.prototype, "addValueToListButtonText", null);
__decorate([
    computed,
    __metadata("design:type", Array),
    __metadata("design:paramtypes", [])
], TreeEntryComponent.prototype, "matchingRoutes", null);
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], TreeEntryComponent.prototype, "navigateableRoute", null);
__decorate([
    computed,
    __metadata("design:type", String),
    __metadata("design:paramtypes", [])
], TreeEntryComponent.prototype, "url", null);
__decorate([
    computed,
    __metadata("design:type", Boolean),
    __metadata("design:paramtypes", [])
], TreeEntryComponent.prototype, "isActive", null);
__decorate([
    computed,
    __metadata("design:type", Boolean),
    __metadata("design:paramtypes", [])
], TreeEntryComponent.prototype, "isList", null);
__decorate([
    computed,
    __metadata("design:type", Array),
    __metadata("design:paramtypes", [])
], TreeEntryComponent.prototype, "childrenIds", null);
__decorate([
    computed,
    __metadata("design:type", Array),
    __metadata("design:paramtypes", [])
], TreeEntryComponent.prototype, "nonNullChildrenIds", null);
__decorate([
    computed,
    __metadata("design:type", Array),
    __metadata("design:paramtypes", [])
], TreeEntryComponent.prototype, "nullChildrenIds", null);
__decorate([
    action,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], TreeEntryComponent.prototype, "addToList", null);
__decorate([
    action,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", void 0)
], TreeEntryComponent.prototype, "initializeChild", null);
if (false) {
    /**
     * Whether this tree-entry is to be displayed as the root one (in which case
     * its label isn't shown anywhere and its children are displayed one after the
     * other). The entry can be displayed as root only when its schema is complex.
     * A button is displayed at the end of the list when there are optional values
     * to be added or to add a new element in case the schema is a list.
     * @type {?}
     */
    TreeEntryComponent.prototype.displayAsRoot;
    /**
     * Internal input used to make a tree only show entries that render in the
     * same outlet (values that should render inside other components shouldn't be
     * part of the same tree).
     * @type {?}
     */
    TreeEntryComponent.prototype._componentLevel;
    /**
     * @type {?}
     * @private
     */
    TreeEntryComponent.prototype.lfRouter;
    /**
     * @type {?}
     * @private
     */
    TreeEntryComponent.prototype.disposeOnDisplayAsRootChanges;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Component used to show a tree to navigate between values.
 */
class TreeNavComponent extends PathBased {
    // Angular lifecycle =========================================================
    /**
     * @return {?}
     */
    ngOnInit() {
        this.lfRouter = this.injector.get(LfRouter);
        super.ngOnInit();
    }
    // ===========================================================================
    /**
     * @return {?}
     */
    get _hasRoutes() {
        return (this.lfRouter.matchingRoutes(this.path, { navigateToParent: false })
            .length > 0);
    }
    /**
     * Hide the tree-nav.
     * @return {?}
     */
    hideTreeNav() {
        ((/** @type {?} */ (this.parentPathBasedComponent))).hideTreeNav();
    }
}
TreeNavComponent.decorators = [
    { type: Component, args: [{
                selector: 'lf-tree-nav',
                changeDetection: ChangeDetectionStrategy.OnPush,
                providers: [{ provide: PathBased, useExisting: TreeNavComponent }],
                template: "<ng-container *mobxAutorun>\n  <nav class=\"nav flex-column\" role=\"navigation\">\n    <!-- Small screen: hide tree-nav button -->\n    <div class=\"tree-nav-actions d-md-none\">\n      <button class=\"tree-nav-hide close\" (click)=\"hideTreeNav()\">\n        &times;\n      </button>\n    </div>\n\n    <!-- Recursively render tree entries -->\n    <lf-tree-entry *ngIf=\"_hasRoutes\" [path]=\"path\"></lf-tree-entry>\n  </nav>\n</ng-container>\n"
            }] }
];
__decorate([
    computed,
    __metadata("design:type", Boolean),
    __metadata("design:paramtypes", [])
], TreeNavComponent.prototype, "_hasRoutes", null);
if (false) {
    /**
     * @type {?}
     * @private
     */
    TreeNavComponent.prototype.lfRouter;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Component used to debug information about the state of a value.
 */
class ValueStateComponent extends PathBased {
    constructor() {
        super(...arguments);
        /**
         * Whether the application is in production mode.
         */
        this.inProduction = process.env.NODE_ENV === 'production';
    }
    /**
     * State properties where each "promise" is transformed into an object for
     * easier rendering as a JSON. Also hides the `$mobx` property.
     * @return {?}
     */
    get stateProperties() {
        /** @type {?} */
        const schema = this.relativeStorage.schema();
        /** @type {?} */
        const state = this.relativeStorage.getState();
        /** @type {?} */
        const value = this.relativeStorage.get();
        return Array.from(Object.keys(state.props)).reduce((/**
         * @param {?} obj
         * @param {?} prop
         * @return {?}
         */
        (obj, prop) => {
            /** @type {?} */
            let propValue = null;
            // When the value is `null`, other validations shouldn't be executed
            if (value !== null ||
                !isReservedStateProp(schema, prop) ||
                prop === 'isRequired' ||
                prop === 'validationIssues') {
                propValue = getStateProperty(schema, state, prop);
                if (isPromiseBasedObservable(propValue)) {
                    /** @type {?} */
                    const res = ((/** @type {?} */ (propValue))).value;
                    propValue = {
                        isPromise: true,
                        state: propValue.state,
                        value: res instanceof Error ? `Error(${res.message})` : res,
                    };
                }
            }
            obj[prop] = propValue;
            return obj;
        }), {});
    }
    /**
     * Whether the state properties contain at least one async property.
     * @return {?}
     */
    get hasAsyncProperties() {
        /** @type {?} */
        const props = this.stateProperties;
        return Object.keys(props).some((/**
         * @param {?} prop
         * @return {?}
         */
        prop => {
            /** @type {?} */
            const val = props[prop];
            return typeof val === 'object' && val && val.isPromise;
        }));
    }
}
ValueStateComponent.decorators = [
    { type: Component, args: [{
                selector: 'lf-value-state',
                changeDetection: ChangeDetectionStrategy.OnPush,
                template: "<!-- Show state properties when not in production -->\n<ng-container *ngIf=\"!inProduction\">\n  <ng-container *mobxAutorun>\n    <ng-container *ngIf=\"relativeStorage.has()\">\n      <!-- Icon/button used to toggle the state properties modal -->\n      <lf-icon class=\"value-state-icon\" icon=\"bug\" (click)=\"stateModal.show()\">\n      </lf-icon>\n\n      <!-- Modal displayed the path's information -->\n      <lf-modal #stateModal size=\"lg\">\n        <lf-modal-header>\n          <code>{{ path }}</code>\n        </lf-modal-header>\n        <lf-modal-body>\n          <h6>Value:</h6>\n          <pre>{{ relativeStorage.get() | json }}</pre>\n          <h6>State properties:</h6>\n          <pre>{{ stateProperties | json }}</pre>\n        </lf-modal-body>\n        <lf-modal-footer *ngIf=\"hasAsyncProperties\">\n          <button\n            lf-button\n            [style]=\"'outline-secondary'\"\n            size=\"sm\"\n            (click)=\"relativeStorage.recomputeStateProperties('.', true)\"\n          >\n            Recompute promises\n          </button>\n        </lf-modal-footer>\n      </lf-modal>\n    </ng-container>\n  </ng-container>\n</ng-container>\n"
            }] }
];
__decorate([
    computed,
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [])
], ValueStateComponent.prototype, "stateProperties", null);
__decorate([
    computed,
    __metadata("design:type", Boolean),
    __metadata("design:paramtypes", [])
], ValueStateComponent.prototype, "hasAsyncProperties", null);
if (false) {
    /**
     * Whether the application is in production mode.
     * @type {?}
     */
    ValueStateComponent.prototype.inProduction;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Protocol prefix used for LF links.
 * @type {?}
 */
const LF_URL_PROTOCOL = 'lf://';
/**
 * Validation message directive.
 */
class ValidationIssueDirective extends PathBased {
    // Angular lifecycle =========================================================
    /**
     * @return {?}
     */
    ngOnInit() {
        this.router = this.injector.get(Router);
        this.lfRouter = this.injector.get(LfRouter);
        this.markdownService = this.injector.get(MarkdownService);
        /** @type {?} */
        const renderer = this.markdownService.renderer;
        /** @type {?} */
        const originalLink = renderer.link;
        this.markdownService.renderer.link = (/**
         * @param {?} href
         * @param {?} title
         * @param {?} text
         * @return {?}
         */
        (href, title, text) => {
            // Replace links with the "LF protocol" with the appropriate paths
            if (href && href.startsWith(LF_URL_PROTOCOL)) {
                /** @type {?} */
                const path = resolvePath(this.path, href.slice(LF_URL_PROTOCOL.length));
                /** @type {?} */
                const route = this.lfRouter.navigateableRoute(path);
                if (href === text) {
                    text = this.labelOf(path);
                }
                href = (/** @type {?} */ (route)) && this.lfRouter.createUrlFromNavigateableRoute((/** @type {?} */ (route)));
            }
            /** @type {?} */
            const anchor = originalLink.call(renderer, href, title, text);
            return anchor.replace('<a', '<a class="lfPathLink"');
        });
        super.ngOnInit();
    }
    // ===========================================================================
    /**
     * HTML string with the issue message to display.
     * @return {?}
     */
    get issueMessageHTML() {
        // Value as obtained from the i18n file
        /** @type {?} */
        const i18nValue = this.lfI18n.getFromPath(this.path, `${I18N_VALIDATIONS_KEY}.${this.issue.code}`, { callFunctionWithContext: false });
        // Obtain markdown message from the value
        return this.markdownService.compile(typeof i18nValue === 'function'
            ? i18nValue(this.lfStorage.relativeStorage(this.path), this.issue.data)
            : i18nValue != null
                ? i18nValue
                : this.issue.code);
    }
    // Intercept clicks on links with class `lfPathLink` to use the router.
    // Adapted from Angular's `routerLink` implementation.
    /**
     * @param {?} target
     * @param {?} button
     * @param {?} ctrlKey
     * @param {?} metaKey
     * @param {?} shiftKey
     * @return {?}
     */
    _onClick(target, button, ctrlKey, metaKey, shiftKey) {
        if (button !== 0 ||
            ctrlKey ||
            metaKey ||
            shiftKey ||
            !target.classList.contains('lfPathLink')) {
            return true;
        }
        this.router.navigateByUrl((/** @type {?} */ (target.getAttribute('href'))), {
            queryParamsHandling: 'merge',
        });
        return false;
    }
    /**
     * Label associated with the given path.
     * @private
     * @param {?} path Path from which to fetch label.
     * @return {?} Label of given path.
     */
    labelOf(path) {
        return this.lfI18n.getFromPath(path, I18N_VALUE_LABEL_KEY) || path;
    }
}
ValidationIssueDirective.decorators = [
    { type: Directive, args: [{
                selector: 'lf-validation-issue, [lfValidationIssue]',
                exportAs: 'lfValidationIssue',
            },] }
];
ValidationIssueDirective.propDecorators = {
    issue: [{ type: Input }],
    issueMessageHTML: [{ type: HostBinding, args: ['innerHTML',] }],
    _onClick: [{ type: HostListener, args: ['click', [
                    '$event.target',
                    '$event.button',
                    '$event.ctrlKey',
                    '$event.metaKey',
                    '$event.shiftKey',
                ],] }]
};
__decorate([
    observable$1,
    __metadata("design:type", Object)
], ValidationIssueDirective.prototype, "issue", void 0);
__decorate([
    computed,
    __metadata("design:type", String),
    __metadata("design:paramtypes", [])
], ValidationIssueDirective.prototype, "issueMessageHTML", null);
if (false) {
    /**
     * Issue with message to display.
     * @type {?}
     */
    ValidationIssueDirective.prototype.issue;
    /**
     * @type {?}
     * @private
     */
    ValidationIssueDirective.prototype.router;
    /**
     * @type {?}
     * @private
     */
    ValidationIssueDirective.prototype.lfRouter;
    /**
     * @type {?}
     * @private
     */
    ValidationIssueDirective.prototype.markdownService;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Exported/declared components and directives.
 * @type {?}
 */
const DECLARATIONS = [
    ActionsMenuComponent,
    AlertComponent,
    AppComponent,
    ButtonComponent,
    CardComponent,
    CardContentComponent,
    CheckboxComponent,
    CheckboxGroupComponent,
    DateComponent,
    DateRangeComponent,
    DropdownComponent,
    DropdownItemComponent,
    DropdownMenuComponent,
    FormComponent,
    HelpTooltipComponent,
    IconComponent,
    ModalBodyComponent,
    ModalComponent,
    ModalFooterComponent,
    ModalHeaderComponent,
    NumberComponent,
    RadioComponent,
    SelectComponent,
    TableCellComponent,
    TableComponent,
    TableRowComponent,
    TextareaComponent,
    TextComponent,
    TooltipComponent,
    TopBarComponent,
    TreeEntryComponent,
    TreeNavComponent,
    ValidationIssueDirective,
    ValidationPanelComponent,
    ValueStateComponent,
];
/**
 * LF Bootstrap theme module.
 */
class LfBootstrapThemeModule {
}
LfBootstrapThemeModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    RouterModule.forChild([]),
                    BsDatepickerModule.forRoot(),
                    LfCoreModule,
                    LfTableHeaderModule,
                    MarkdownModule.forRoot(),
                    MobxAngularModule,
                    ScrollingModule,
                    TooltipModule.forRoot(),
                ],
                declarations: DECLARATIONS,
                exports: [
                    LfCoreModule,
                    LfTableHeaderModule,
                    MobxAngularModule,
                    ...DECLARATIONS,
                ],
            },] }
];
/**
 * Theme services that should be provided for each provided `LF_APP_SCHEMA`.
 * Currently empty, reserved for future use.
 * @type {?}
 */
const LF_BOOTSTRAP_THEME_SERVICES = [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Default english translations for Bootstrap theme strings.
 * @type {?}
 */
const I18N_EN_US = {
    '*': {
        // Unload alert service
        [I18N_UNLOAD_ALERT_MESSAGE_KEY]: 'Are you sure you want to leave? Changes that you made may not be saved.',
        // Value
        [I18N_VALUE_RETRY_VALIDATE_TEXT_KEY]: 'Retry validation',
        // Tree-nav
        [I18N_TREE_NAV_ADD_OPTIONAL_VALUE_BUTTON_TEXT_KEY]: 'Add',
        [I18N_TREE_NAV_ADD_VALUE_TO_LIST_BUTTON_TEXT_KEY]: 'Add',
        // Closeable
        [I18N_CLOSE_BUTTON_TOOLTIP_KEY]: 'Close',
        // Actions-menu
        [I18N_ACTIONS_MENU_BUTTON_TEXT_KEY]: 'Actions',
        // Validation panel
        [I18N_VALIDATION_PANEL_ERRORS_TEXT_KEY]: 'Errors:',
        [I18N_VALIDATION_PANEL_WARNINGS_TEXT_KEY]: 'Warnings:',
        [I18N_VALIDATION_PANEL_NO_ISSUES_TEXT_KEY]: 'Everything looks great!',
        [I18N_VALIDATION_PANEL_FOCUS_TEXT_KEY]: 'Focus field',
        [I18N_VALIDATION_PANEL_NEXT_BUTTON_TOOLTIP_KEY]: 'Next field',
        [I18N_VALIDATION_PANEL_PREV_BUTTON_TOOLTIP_KEY]: 'Previous field',
        // Form
        [I18N_FORM_HELP_ACTION_TEXT_KEY]: 'Help',
        [I18N_FORM_REMOVE_ACTION_TEXT_KEY]: 'Remove',
        [I18N_FORM_ENSURE_REMOVE_TEXT_KEY]: 'Please confirm that you wish to remove this form.',
        [I18N_FORM_CANCEL_REMOVE_BUTTON_TEXT_KEY]: 'Cancel',
        [I18N_FORM_CONFIRM_REMOVE_BUTTON_TEXT_KEY]: 'Confirm',
        // Date
        [I18N_DATE_FORMAT_KEY]: 'M/D/YYYY',
        [I18N_DATE_PLACEHOLDER_KEY]: 'm/d/yyyy',
        // Date-range
        [I18N_DATE_RANGE_SEPARATOR_KEY]: ' - ',
        // Number
        [I18N_NUMBER_DECIMAL_SEPARATOR_KEY]: '.',
        [I18N_NUMBER_THOUSANDS_SEPARATOR_KEY]: ',',
        // Table
        [I18N_TABLE_ADD_ROW_ACTION_TEXT_KEY]: 'Add row',
        [I18N_TABLE_REMOVE_ROWS_ACTION_TEXT_KEY]: 'Remove rows',
        [I18N_TABLE_NO_ROWS_TEXT_KEY]: 'No rows to display.',
        // Selection
        [I18N_SELECTION_NO_OPTIONS_TEXT_KEY]: 'No options available.',
        // Validations
        [I18N_VALIDATIONS_KEY]: {
            [IS_REJECTED_CODE]: 'An error occurred validating this field.',
            [IS_REQUIRED_CODE]: 'The field is required.',
            [UNALLOWED_VALUE_CODE]: (/**
             * @param {?} ctx
             * @param {?} __1
             * @return {?}
             */
            (ctx, { allowedValues }) => {
                if (isBooleanSchema(ctx.schema()) && allowedValues.length === 1) {
                    return `The field must${allowedValues[0] === true ? '' : ' not'} be ticked.`;
                }
                return `The field's value must be one of: ${allowedValues.join(', ')}.`;
            }),
            [DATE_OUT_OF_BOUNDS_CODE]: (/**
             * @param {?} _
             * @param {?} __1
             * @return {?}
             */
            (_, { minDate, maxDate }) => {
                // TODO: Format date properly
                return maxDate === undefined
                    ? `The date mustn't be before ${minDate}.`
                    : minDate === undefined
                        ? `The date mustn't be after ${maxDate}.`
                        : `The date must be between ${minDate} and ${maxDate}.`;
            }),
            [SIZE_OUT_OF_BOUNDS_CODE]: (/**
             * @param {?} ctx
             * @param {?} __1
             * @return {?}
             */
            (ctx, { size, minSize, maxSize }) => {
                if (size === 0 && minSize > 0) {
                    return 'The field is required.';
                }
                // Checkbox group
                if (ctx.hasStateProperty('.', `i18n(${I18N_SELECTION_OPTIONS_KEY})`)) {
                    return maxSize === undefined
                        ? `A minimum of ${+(/** @type {?} */ (minSize))} option${+minSize === 1 ? '' : 's'} must be selected.`
                        : minSize === undefined
                            ? `A maximum of ${+maxSize} option${+maxSize === 1 ? '' : 's'} must be selected.`
                            : maxSize === minSize
                                ? `${+maxSize} option${+maxSize === 1 ? '' : 's'} must be selected.`
                                : `Between ${+minSize} and ${+maxSize} options must be selected.`;
                }
                // Tables and form-lists
                return maxSize === undefined
                    ? `The collection must have at least ${+(/** @type {?} */ (minSize))} element${+minSize === 1 ? '' : 's'}.`
                    : minSize === undefined
                        ? `The collection must have no more than ${+maxSize} element${+maxSize === 1 ? '' : 's'}.`
                        : maxSize === minSize
                            ? `The collection must have ${+maxSize} element${+maxSize === 1 ? '' : 's'}.`
                            : `The collection must have between ${+minSize} and ${+maxSize} ` +
                                'elements.';
            }),
            [NUMBER_OUT_OF_BOUNDS_CODE]: (/**
             * @param {?} _
             * @param {?} __1
             * @return {?}
             */
            (_, { min, max }) => {
                return max === undefined
                    ? `The value mustn't be lesser than ${+(/** @type {?} */ (min))}.`
                    : min === undefined
                        ? `The value mustn't be greater than ${+max}.`
                        : `The value must be between ${+min} and ${+max}.`;
            }),
            [LENGTH_OUT_OF_BOUNDS_CODE]: (/**
             * @param {?} _
             * @param {?} __1
             * @return {?}
             */
            (_, { length, minLength, maxLength }) => {
                if (length === 0 && minLength > 0) {
                    return 'The field is required.';
                }
                return maxLength === undefined
                    ? `The value must have at least ${+(/** @type {?} */ (minLength))} character${+(/** @type {?} */ (minLength)) === 1 ? '' : 's'}.`
                    : minLength === undefined
                        ? `The value must have at most ${+maxLength} character${+maxLength === 1 ? '' : 's'}.`
                        : maxLength === minLength
                            ? `The value must have ${+maxLength} character${+maxLength === 1 ? '' : 's'}.`
                            : `The value must have ${+minLength} to ${+maxLength} characters.`;
            }),
        },
    },
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Default portuguese translations for Bootstrap theme strings.
 * @type {?}
 */
const I18N_PT_PT = {
    '*': {
        // Unload alert service
        [I18N_UNLOAD_ALERT_MESSAGE_KEY]: 'Tem a certeza que pretende sair? Alterações feitas podem não estar guardadas.',
        // Value
        [I18N_VALUE_RETRY_VALIDATE_TEXT_KEY]: 'Voltar a tentar validar',
        // Tree-nav
        [I18N_TREE_NAV_ADD_OPTIONAL_VALUE_BUTTON_TEXT_KEY]: 'Adicionar',
        [I18N_TREE_NAV_ADD_VALUE_TO_LIST_BUTTON_TEXT_KEY]: 'Adicionar',
        // Actions-menu
        [I18N_ACTIONS_MENU_BUTTON_TEXT_KEY]: 'Ações',
        // Closeable
        [I18N_CLOSE_BUTTON_TOOLTIP_KEY]: 'Fechar',
        // Validation panel
        [I18N_VALIDATION_PANEL_ERRORS_TEXT_KEY]: 'Erros:',
        [I18N_VALIDATION_PANEL_WARNINGS_TEXT_KEY]: 'Avisos:',
        [I18N_VALIDATION_PANEL_NO_ISSUES_TEXT_KEY]: 'Nenhum problema a reportar!',
        [I18N_VALIDATION_PANEL_FOCUS_TEXT_KEY]: 'Selecionar campo',
        [I18N_VALIDATION_PANEL_NEXT_BUTTON_TOOLTIP_KEY]: 'Campo seguinte',
        [I18N_VALIDATION_PANEL_PREV_BUTTON_TOOLTIP_KEY]: 'Campo anterior',
        // Form
        [I18N_FORM_HELP_ACTION_TEXT_KEY]: 'Ajuda',
        [I18N_FORM_REMOVE_ACTION_TEXT_KEY]: 'Remover',
        [I18N_FORM_ENSURE_REMOVE_TEXT_KEY]: 'Por favor confirme que pretende remover este formulário.',
        [I18N_FORM_CANCEL_REMOVE_BUTTON_TEXT_KEY]: 'Cancelar',
        [I18N_FORM_CONFIRM_REMOVE_BUTTON_TEXT_KEY]: 'Confirmar',
        // Date
        [I18N_DATE_FORMAT_KEY]: 'DD/MM/YYYY',
        [I18N_DATE_PLACEHOLDER_KEY]: 'dd/mm/aaaa',
        // Date-range
        [I18N_DATE_RANGE_SEPARATOR_KEY]: ' - ',
        // Number
        [I18N_NUMBER_DECIMAL_SEPARATOR_KEY]: ',',
        [I18N_NUMBER_THOUSANDS_SEPARATOR_KEY]: ' ',
        // Table
        [I18N_TABLE_ADD_ROW_ACTION_TEXT_KEY]: 'Adicionar linha',
        [I18N_TABLE_REMOVE_ROWS_ACTION_TEXT_KEY]: 'Remover linhas',
        [I18N_TABLE_NO_ROWS_TEXT_KEY]: 'Nenhuma linha a mostrar.',
        // Selection
        [I18N_SELECTION_NO_OPTIONS_TEXT_KEY]: 'Nenhuma opção disponível.',
        // Validations
        [I18N_VALIDATIONS_KEY]: {
            [IS_REJECTED_CODE]: 'Ocorreu um erro ao validar este valor.',
            [IS_REQUIRED_CODE]: 'Campo de preenchimento obrigatório.',
            [UNALLOWED_VALUE_CODE]: (/**
             * @param {?} ctx
             * @param {?} __1
             * @return {?}
             */
            (ctx, { allowedValues }) => {
                if (isBooleanSchema(ctx.schema()) && allowedValues.length === 1) {
                    return `O campo ${allowedValues[0] === true ? 'tem de ser' : 'não pode ser'} marcado.`;
                }
                return `O valor do campo tem de ser um de: ${allowedValues.join(', ')}.`;
            }),
            [DATE_OUT_OF_BOUNDS_CODE]: (/**
             * @param {?} _
             * @param {?} __1
             * @return {?}
             */
            (_, { minDate, maxDate }) => {
                // TODO: Format date properly
                return maxDate === undefined
                    ? `A data não pode ser anterior a ${minDate}.`
                    : minDate === undefined
                        ? `A data não pode ser posterior a ${maxDate}.`
                        : `A data tem de estar entre ${minDate} e ${maxDate}.`;
            }),
            [SIZE_OUT_OF_BOUNDS_CODE]: (/**
             * @param {?} ctx
             * @param {?} __1
             * @return {?}
             */
            (ctx, { size, minSize, maxSize }) => {
                if (size === 0 && minSize > 0) {
                    return 'Campo de preenchimento obrigatório.';
                }
                // Checkbox group
                if (ctx.hasStateProperty('.', `i18n(${I18N_SELECTION_OPTIONS_KEY})`)) {
                    /** @type {?} */
                    const numOptions = (/**
                     * @param {?} n
                     * @return {?}
                     */
                    n => (n === 1 ? 'uma' : n === 2 ? 'duas' : n));
                    return maxSize === undefined
                        ? `Tem de selecionar no mínimo ${numOptions(+(/** @type {?} */ (minSize)))} ${+minSize === 1 ? 'opção' : 'opções'}.`
                        : minSize === undefined
                            ? `Tem de selecionar no máximo ${numOptions(+(/** @type {?} */ (maxSize)))} ${+minSize === 1 ? 'opção' : 'opções'}.`
                            : maxSize === minSize
                                ? `Tem de selecionar ${numOptions(+(/** @type {?} */ (maxSize)))} ${+maxSize === 1 ? 'opção' : 'opções'}.`
                                : `Entre ${numOptions(+minSize)} e ${numOptions(+maxSize)} opções têm de ser selecionadas.`;
                }
                // Tables and form-lists
                return maxSize === undefined
                    ? `A coleção tem de ter no mínimo ${+(/** @type {?} */ (minSize))} elemento${+minSize === 1 ? '' : 's'}.`
                    : minSize === undefined
                        ? `A coleção tem de ter no máximo ${+maxSize} elemento${+maxSize === 1 ? '' : 's'}.`
                        : maxSize === minSize
                            ? `A coleção tem de ter ${+maxSize} elemento${+maxSize === 1 ? '' : 's'}.`
                            : `A coleção tem de ter entre ${+minSize} e ${+maxSize} elementos.`;
            }),
            [NUMBER_OUT_OF_BOUNDS_CODE]: (/**
             * @param {?} _
             * @param {?} __1
             * @return {?}
             */
            (_, { min, max }) => {
                return max === undefined
                    ? `O valor não pode ser menor do que ${+(/** @type {?} */ (min))}.`
                    : min === undefined
                        ? `O valor não pode ser maior do que ${+max}.`
                        : `O valor tem de estar entre ${+min} e ${+max}.`;
            }),
            [LENGTH_OUT_OF_BOUNDS_CODE]: (/**
             * @param {?} _
             * @param {?} __1
             * @return {?}
             */
            (_, { length, minLength, maxLength }) => {
                if (length === 0 && minLength > 0) {
                    return 'Campo de preenchimento obrigatório.';
                }
                return maxLength === undefined
                    ? `O valor tem de ter no mínimo ${+(/** @type {?} */ (minLength))} ${+(/** @type {?} */ (minLength)) === 1 ? 'carácter' : 'caracteres'}.`
                    : minLength === undefined
                        ? `O valor tem de ter no máximo ${+maxLength} ${+maxLength === 1 ? 'carácter' : 'caracteres'}.`
                        : maxLength === minLength
                            ? `O valor tem de ter ${+maxLength} ${+maxLength === 1 ? 'carácter' : 'caracteres'}.`
                            : `O valor tem de ter entre ${+minLength} e ${+maxLength} ` +
                                'caracteres.';
            }),
        },
    },
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { ActionsMenuComponent, AlertComponent, AppComponent, ButtonComponent, CardComponent, CardContentComponent, CheckboxComponent, CheckboxGroupComponent, DateComponent, DateRangeComponent, DropdownComponent, DropdownItemComponent, DropdownMenuComponent, FormComponent, HelpTooltipComponent, I18N_ACTIONS_MENU_BUTTON_TEXT_KEY, I18N_ACTIONS_MENU_KEY, I18N_ACTIONS_MENU_OPTIONS_KEY, I18N_CLOSE_BUTTON_TOOLTIP_KEY, I18N_DATE_FORMAT_KEY, I18N_DATE_PLACEHOLDER_KEY, I18N_DATE_RANGE_SEPARATOR_KEY, I18N_EN_US, I18N_FORM_CANCEL_REMOVE_BUTTON_TEXT_KEY, I18N_FORM_CONFIRM_REMOVE_BUTTON_TEXT_KEY, I18N_FORM_ENSURE_REMOVE_TEXT_KEY, I18N_FORM_HELP_ACTION_TEXT_KEY, I18N_FORM_REMOVE_ACTION_TEXT_KEY, I18N_NUMBER_DECIMAL_SEPARATOR_KEY, I18N_NUMBER_PREFIX_KEY, I18N_NUMBER_SUFFIX_KEY, I18N_NUMBER_THOUSANDS_SEPARATOR_KEY, I18N_PT_PT, I18N_SELECTION_NO_OPTIONS_TEXT_KEY, I18N_SELECTION_OPTIONS_KEY, I18N_TABLE_ADD_ROW_ACTION_TEXT_KEY, I18N_TABLE_COLUMN_LABELS_KEY, I18N_TABLE_NO_ROWS_TEXT_KEY, I18N_TABLE_REMOVE_ROWS_ACTION_TEXT_KEY, I18N_TREE_NAV_ADD_OPTIONAL_VALUE_BUTTON_TEXT_KEY, I18N_TREE_NAV_ADD_VALUE_TO_LIST_BUTTON_TEXT_KEY, I18N_VALIDATIONS_KEY, I18N_VALIDATION_PANEL_ERRORS_TEXT_KEY, I18N_VALIDATION_PANEL_FOCUS_TEXT_KEY, I18N_VALIDATION_PANEL_NEXT_BUTTON_TOOLTIP_KEY, I18N_VALIDATION_PANEL_NO_ISSUES_TEXT_KEY, I18N_VALIDATION_PANEL_PREV_BUTTON_TOOLTIP_KEY, I18N_VALIDATION_PANEL_WARNINGS_TEXT_KEY, I18N_VALUE_CODE_KEY, I18N_VALUE_HELP_MESSAGE_KEY, I18N_VALUE_LABEL_KEY, I18N_VALUE_LEGEND_KEY, I18N_VALUE_RETRY_VALIDATE_TEXT_KEY, IconComponent, InlineValue, LF_BOOTSTRAP_THEME_SERVICES, LF_URL_PROTOCOL, LfBootstrapThemeModule, ModalBodyComponent, ModalComponent, ModalFooterComponent, ModalHeaderComponent, NumberComponent, RadioComponent, SELECTION_COLUMN_WIDTH, SelectComponent, SelectionValue, SingleSelectionValue, TableCellComponent, TableComponent, TableRowComponent, TextComponent, TextareaComponent, Toggleable, TooltipComponent, TopBarComponent, TreeEntryComponent, TreeNavComponent, ValidationIssueDirective, ValidationPanelComponent, Value, ValueStateComponent, dateRangeSchema, isDateRangeSchema };
//# sourceMappingURL=lightweightform-bootstrap-theme.js.map
